unit fMHA_MsgDlg;

{- Borrowed and modified by Roger Schultz, Ph.D.
   Mental Health Strategic Health Care Group, Informatics Section
   Revision Date: 12/1999
   Description: A better dialog box as caption can be modified for
     more informative windows messages.
-}

interface

uses Forms, SysUtils, Classes, Graphics, StdCtrls, Controls,
     Dialogs, WinProcs, WinTypes;

function CreateMsgDlg(const Msg: string; const Caption: string;
                      AType: TMsgDlgType; Buttons: TMsgDlgButtons;
                      HelpCtx: Longint; L, T, H, W: Integer): TForm;

// Message centered on desktop
function MsgDlg(const Msg: string; const Caption: string;
                AType: TMsgDlgType; Buttons: TMsgDlgButtons;
                HelpCtx: Longint): TModalResult;

// Message centered on parent form
function MsgDlgFrm(const Msg: string; const Caption: string;
                AType: TMsgDlgType; Buttons: TMsgDlgButtons;
                HelpCtx: Longint; L, T, H, W: Integer): TModalResult;

implementation


function CreateMsgDlg(const Msg: string; const Caption: string;
                      AType: TMsgDlgType; Buttons: TMsgDlgButtons;
                      HelpCtx: Longint;
                      L{eft}, T{op}, H{eight}, W{eight}: Integer): TForm;
var
  WhereMouse  : TPoint;
  i1, i2,
  ButtonCount : integer;
begin
  Result := CreateMessageDialog(Msg, TMsgDlgType(AType), TMsgDlgButtons(Buttons));

  if Caption <> EmptyStr then
  begin
     Result.Caption := Caption;
  end;
  if L > -1 then // Use -1 for center of desktop
    begin
      i1 := L + ((W - Result.Width) div 2);
      i2 := GetSystemMetrics(SM_CXSCREEN);
      // ensure message does not fall off screen//
//showmessage('i1 '+inttostr(i1)+' i2 '+inttostr(i2)+ ' Res.width '+ inttostr(result.width) +
//  'width '+inttostr(W));
      if i1 + Result.Width > i2 then
        i1 := i2 - (Result.Width + 20)
        else
        if i1 < 0 then i1 := 20;
      Result.Left := i1;
    end;
  if T > -1 then // Use -1 for center of desktop
    begin
      i1 := T + ((H - Result.Height) div 2);
      i2 := GetSystemMetrics(SM_CYSCREEN);
      // ensure message does not fall off screen
      if i1 + Result.Height > i2 then
        i1 := i2 - (Result.Height + 20)
        else
        if i1 < 0 then i1 := 20;
      Result.Top := i1;
    end;

  Result.HelpContext := HelpCtx;
  //Result.ScaleBy(Screen.PixelsPerInch, 96);

  // Place the mouse in the first button.
  // Count the buttons then backup the mouse from
  // the center.
  ButtonCount := -1;
  if mbOk     in Buttons then Inc(ButtonCount);
  if mbNo     in Buttons then Inc(ButtonCount);
  if mbCancel in Buttons then Inc(ButtonCount);
  if mbAbort  in Buttons then Inc(ButtonCount);
  if mbRetry  in Buttons then Inc(ButtonCount);
  if mbIgnore in Buttons then Inc(ButtonCount);
  if mbAll    in Buttons then Inc(ButtonCount);
  if mbHelp   in Buttons then Inc(ButtonCount);

  WhereMouse.X := Result.Left + Result.Width  div 2 + 15 - ButtonCount * 30;
  WhereMouse.Y := Result.Top  + Result.Height div 2 + 30;
  Mouse.CursorPos := WhereMouse;
  Screen.Cursor := crDefault;
  Result.Font.Color := clWindowText;
  Result.Font.Size := 9;

  case AType of
     mtWarning:        MessageBeep(MB_ICONEXCLAMATION);
     mtError:          MessageBeep(MB_ICONSTOP);
     mtInformation:    MessageBeep(MB_ICONINFORMATION);
     mtConfirmation:   MessageBeep(MB_ICONQUESTION);
  end;
end;

function MsgDlgPos(const Msg: string; const Caption: string;
                   AType: TMsgDlgType; Buttons: TMsgDlgButtons;
                   HelpCtx: Longint; L, T, H, W: integer): TModalResult;
var
  Dlg: TForm;
begin
  Result := 0;
  try
     Dlg := CreateMsgDlg(Msg, Caption, AType, Buttons, HelpCtx, L, T, H, W);
     Result := Dlg.ShowModal;
  finally
     Dlg.Free;
  end;
end;

function MsgDlgFrm(const Msg: string; const Caption: string;
                AType: TMsgDlgType; Buttons: TMsgDlgButtons;
                HelpCtx: Longint; L, T, H, W: Integer): TModalResult;
// Left, Top, Height, Width are dimensions from parent form.
// Message is centered on parent form.
begin
  Result := MsgDlgPos(Msg, Caption, AType, Buttons, HelpCtx, L, T, H, W);
end;


function MsgDlg(const Msg: string; const Caption: string;
                AType: TMsgDlgType; Buttons: TMsgDlgButtons;
                HelpCtx: Longint): TModalResult;
// Message is centered on desktop
begin
  Result := MsgDlgPos(Msg, Caption, AType, Buttons, HelpCtx, -1, -1, -1, -1);
end;

end.
