{***************************************************************

 UNIT NAME: fMHA_PatientDemographics
 PACKAGE: Mental Health
 WARNING:
 DATE CREATED: 04/09/01
 SITE NAME: Salt Lake City Mental Health Informatics Section
 DEVELOPERS: CANNON/FELDMAN
 DESCRIPTION: Displays patient demographics, "borrowed" from CPRS
 NOTE: This unit requires Mental Health patch 67, or greater, in
       order to run.

 HISTORY  :

 Required third-party Delphi components and libraries:

****************************************************************}

unit fMHA_PatientDemographics;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls;

type
  TfrmPatientDemographics = class(TForm)
    memDemographics: TRichEdit;
    pnlButton: TPanel;
    pnlCloseButton: TPanel;
    btnClose: TButton;
    btnPrint: TButton;
    btnCopy: TButton;
    btnSave: TButton;
    procedure btnCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnCopyClick(Sender: TObject);
    procedure btnPrintClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmPatientDemographics: TfrmPatientDemographics;

implementation
uses fMHA_Main, TRPCB,
  MFUNStr, ESBDates,
  uCore, uMHA_MsgDlg,
  ORNet, uMHA_DM, ORFn, Printers;

{$R *.DFM}


procedure TfrmPatientDemographics.btnCloseClick(Sender: TObject);
//******************************************************************************
// PURPOSE: Close the form, return to main
//******************************************************************************
begin
  Close;
end;


procedure TfrmPatientDemographics.FormCreate(Sender: TObject);
//******************************************************************************
// PURPOSE: Load pt demographics
//******************************************************************************
var r: TRect;
begin
  //set form height and top to work area (i.e., desktop size minus task bars)
  SystemParametersInfo(SPI_GetWorkArea, 0, @r, 0);
  BoundsRect := r;
  Top := r.Top;
  Height := r.Bottom - r.Top;

  //set form width to 638 and center it
  Width := 638;
  Left := (r.Right - Width) div 2;

  //CPRS uses this color for the memo:  $00F0FBFF

  CallV('ORWPT PTINQ', [Patient.DFN]);
  //line 0 deleted to get rid of control characters
  RPCBrokerV.Results.Delete(0);
  memDemographics.Lines.Assign(RPCBrokerV.Results);
  memDemographics.Lines.Delete(0);
  memDemographics.SelStart := 0;
end;


procedure TfrmPatientDemographics.btnCopyClick(Sender: TObject);
//******************************************************************************
// PURPOSE: Copy patient demographics to clipboard
//******************************************************************************
begin
  with memDemographics do
  begin
    SelectAll;
    CopyToClipboard;
  end;
  MsgDlg('The information has been copied to the Windows clipboard.' + CRLF +
    '' + CRLF +
    'If you like, you may paste it into an open text document.',
    DlgCaption, mtInformation, [mbOK], 0);
end;


procedure TfrmPatientDemographics.btnPrintClick(Sender: TObject);
//******************************************************************************
// PURPOSE: Print patient demographics
//******************************************************************************
var
  presX, presY: Integer;
begin
  with memDemographics do begin
    plaintext := true;
    presX := GetDeviceCaps(printer.handle, LOGPIXELSX);
    presY := GetDeviceCaps(printer.handle, LOGPIXELSY);
    with pagerect do begin
      left := 3 * presX div 4; // .75 inch left margin
      top := presY; // 1 inch top margin
      right := Printer.PageWidth - (3 * presX div 4); // .75 inch right margin
      bottom := Printer.PageHeight - presY; // 1 inch bottom margin
    end;
    memDemographics.Print(MHTest.Code + ' Report');
  end;
end;


procedure TfrmPatientDemographics.btnSaveClick(Sender: TObject);
//******************************************************************************
// PURPOSE: Save patient demographics to rtf file
//******************************************************************************
begin
  with MHA_DataModule.dlgSave do
  begin
    FileName := 'Patient Demographics.rtf';
    Filter := 'Rich text file (*rtf)|*.rtf';
    if Execute then
      memDemographics.Lines.SaveToFile(FileName);
  end;

end;

procedure TfrmPatientDemographics.FormKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = vk_Escape) then
    Close;
end;

end.

