{***************************************************************

 UNIT NAME: fMHA_TestEntry
 PACKAGE:
 WARNING:
 DATE CREATED: Sep, 2000
 SITE NAME: Salt Lake City Mental Health Informatics Section
 DEVELOPERS: FELDMAN/CANNON
 DESCRIPTION: Staff Entry form for MH Testing
 NOTE:

 HISTORY  :

 Required third-party Delphi components and libraries:

****************************************************************}

unit fMHA_TestEntry;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, CheckLst, Grids, ExtCtrls, DBCtrls, Buttons, Fmcntrls, {RXCtrls,}
  Mask, Spin, ComCtrls, ORCtrls, Vawrgrid, AdvGrid, LMDCustomComponent,
  LMDStarter, BaseGrid;

type
  TfrmTestEntry = class(TForm)
    pnlButtons: TPanel;
    pnlResults: TPanel;
    btnNewQuest: TBitBtn;
    stsbrTestEntry: TStatusBar;
    btnBackUp: TBitBtn;
    pnlAskQuestion: TPanel;
    lblQuestion: TLabel;
    grdChoices: TVAWrapGrid;
    memIntro: TRichEdit;
    grdResponses: TAdvStringGrid;
    prgbrTestEntry: TProgressBar;
    lblExit: TLabel;
    btnNextTest: TButton;
    LMDStarterSecurity: TLMDStarter;
    procedure LoadQuestion;
    procedure grdChoicesClick(Sender: TObject);
    procedure FindSkipped;
    procedure StartNextTest;
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure LoadResults;
    procedure grdResponsesClick(Sender: TObject);
    procedure btnNewQuestClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnBackUpClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure UpdateFormDisplay;
    procedure btnNextTestClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Private declarations }
    FResponsesRowClick: boolean;
    FChoicesRowClick: boolean;

    FKeyPressProcessingDone: boolean;

    //Following booleans are for detecting held-down keystrokes.
    FIsKeyHeldDown: Boolean;
    FKeyDownDone: Boolean;
    FKeyPressedDone: Boolean;
    FIsEscapeKey: Boolean;
    //  FSecureDesktopActive: Boolean;
    FbtnCloseClick: Boolean;
  public
    { Public declarations }
  end;

var
  frmTestEntry: TfrmTestEntry;

implementation

uses fMHA_Main, MFUNSTR, uMHA_MHInstruments, uMHA_MsgDlg, ORFn;

{$R *.DFM}

{#Todo1 toggle these, before releasing product}
{$DEFINE RELEASE_VER}
{.$DEFINE DEBUG_VER}

{
 The two defines above are used for debugging purposes. This form logs off the
user after completing a patient-administerred test. To prevent the form from
logging you off during debugging, place a period (.) before the dollar-sign ($)
in the DEFINE RELEASE_VER define. To compile a release version of this code, and
to re-enable logonf functions, remove the period. Note that one of these defines
must be active at any one time. Also note that debug mode causes some warnings,
which may be ignored if not present in the release build.
}

procedure TfrmTestEntry.LoadQuestion;
//******************************************************************************
// PURPOSE: Loads the text of the question (including intro and choices)
//******************************************************************************
var
  iChoiceNum: integer;
begin

  //If patient entry is used, then close the app if the list is completed

  //Check for end of test
  if TestEngine.EndOfTest then
  begin
    if (TestList.TestMode = tmPatient) then
    begin
      btnNextTestClick(Self);
      Exit;
    end
    else
    begin
      Beep;
      stsbrTestEntry.Panels[0].Text := 'That was the last question for the ' +
        MHTest.Code;
      MsgDlg('That was the last question for the ' + MHTest.Code,
        DlgCaption, mtInformation, [mbOK], 0);

    end;
  end;

  if TestItem.ItemNumber < MHTest.NumberOfItems then
    TestItem.ItemNumber := TestItem.ItemNumber + 1;

  //Display number of items if staff entry
  if not (TestList.TestMode = tmPatient) then
    stsbrTestEntry.Panels[2].Text := 'Number of Items:  '
      + IntToStr(MHTest.NumberOfItems) + '      Number Remaining:  '
      + IntToStr(MHTest.NumberOfItems - TestItem.ItemNumber);

  //Use getdata to load data into the TestItem object
  TestItem.GetItemData;

  //Write introduction
  if not (MHTest.Code = 'SAI') and
    (Length(Trim(TestItem.IntroductionText.Text)) > 1) then
    memIntro.Text := TestItem.IntroductionText.Text;

  if MHTest.Code = 'SAI' then
    memIntro.Clear;

  //Write question
 //if no question then...
  if Length(TestItem.QuestionText.Text) < 3 then
    lblQuestion.Caption := CRLF + IntToStr(TestItem.ItemNumber)
      + '.  Select one of the following statements:' + CRLF
  else
    lblQuestion.Caption := CRLF + ' ' + IntToStr(TestItem.ItemNumber)
      + '.  ' + TestItem.QuestionText.Strings[0] + CRLF;

  //Disable the backup button if ItemNumber = 1
  btnBackUp.Enabled := not (TestItem.ItemNumber = 1);

  //New question button enabled if current item is less than the
  //total number of questions answered and it's not the last item
  if (TestItem.ItemNumber <= Length(TestEngine.ChoicesMade))
    and not (MHTest.NumberOfItems = Length(TestEngine.ChoicesMade)) then
    btnNewQuest.Enabled := True
  else
    btnNewQuest.Enabled := False;

  //Write the response options in string grid
  for iChoiceNum := 0 to TestItem.ChoicesText.Count - 1 do
    grdChoices.Cells[0, iChoiceNum] := ' ' +
      TestItem.ChoicesText[iChoiceNum];

  //Need extra row for 'X' if staff entry
  if (TestList.TestMode = tmPatient) then
    grdChoices.RowCount := TestItem.NumberOfChoices - 1
  else
  begin
    grdChoices.RowCount := TestItem.NumberOfChoices;
    grdChoices.Cells[0, TestItem.ChoicesText.Count] :=
      ' X. Skip this question';
  end;

  //Set height of string grid panel
  grdChoices.Height := grdChoices.RowCount * grdChoices.RowHeights[0] + 10;

  if (TestList.TestMode = tmPatient) then
    Exit;

  //No row selected if new question; otherwise, select previous answer
  FChoicesRowClick := False;
  if (length(TestEngine.ChoicesMade) < TestItem.ItemNumber) then
  begin
    grdChoices.Row := 0;
    {DSC 2/7/02if TestItem.ItemNumber < 4 then
      grdResponses.TopRow := 1
    else
      grdResponses.TopRow := TestItem.ItemNumber - 3; }
  end
  else
  begin
    grdChoices.Row := pos(copy(TestEngine.ChoicesMade, TestItem.ItemNumber, 1),
      TestItem.ValidChoices) - 1;
    // DSC 2/7/02 grdResponses.TopRow := TestItem.ItemNumber;
    FResponsesRowClick := False;
    grdResponses.Row := TestItem.ItemNumber;
    FResponsesRowClick := True;
  end;
  FChoicesRowClick := True;

end;

procedure TfrmTestEntry.grdChoicesClick(Sender: TObject);
//******************************************************************************
// PURPOSE: Record answer
//******************************************************************************
var
  tmpValid: Char;
begin
  if not FChoicesRowClick then
    Exit;

  //Record answer
  tmpValid := TestItem.ValidChoices[grdChoices.Row + 1];
  TestEngine.AddToChoicesMade(tmpValid);

  UpdateFormDisplay;
end;

procedure TfrmTestEntry.LoadResults;
//******************************************************************************
// PURPOSE: Writes answers to string grid
//******************************************************************************
var
  //use to correct for the fact that tests without questions (e.g., BDI and RLOC)
  //do not always use zero based response options (e.g., BDI does, RLOC doesn't)
  tmpIndexCorrection: integer;
begin
  //Exit if patient entry because the string grid is not visible anyway
  if (TestList.TestMode = tmPatient) then
    Exit;

  //Add next response to string grid
  with grdResponses do
  begin
    SelectionColor := clInfoBk;
    RowCount := Length(TestEngine.ChoicesMade) + 1;
    FResponsesRowClick := False;
    Row := TestItem.ItemNumber;
    FResponsesRowClick := True;
    Cells[0, TestItem.ItemNumber] := IntToStr(TestItem.ItemNumber);
    Cells[1, TestItem.ItemNumber] :=
      CharAt(TestEngine.ChoicesMade[TestItem.ItemNumber], 1);
    if Length(TestItem.QuestionText.Text) < 3 then
    begin
      tmpIndexCorrection := StrToInt(copy(TestItem.ValidChoices, 1, 1));

      if (Cells[1, TestItem.ItemNumber] = 'X') then
        Cells[2, TestItem.ItemNumber] := 'Skipped'
      else
        Cells[2, TestItem.ItemNumber] :=
          TestItem.ChoicesText.Strings[StrToInt(Cells[1, TestItem.ItemNumber]) -
            tmpIndexCorrection];
    end
    else
      Cells[2, TestItem.ItemNumber] := TestItem.QuestionText.Text;
  end;

  Application.ProcessMessages;

end;

procedure TfrmTestEntry.FindSkipped;
//******************************************************************************
// PURPOSE: Post the number of skipped items on the status bar
//******************************************************************************
var
  ipos, Missing: integer;
begin

  if TestList.TestMode = tmPatient then
    exit;

  Missing := 0;

  for ipos := 1 to Length(TestEngine.ChoicesMade) do
    if TestEngine.ChoicesMade[ipos] = 'X' then
      inc(Missing);

  stsbrTestEntry.Panels[1].Text := 'No. of skipped items: ' + IntToStr(Missing);

end;

procedure TfrmTestEntry.StartNextTest;
//******************************************************************************
// PURPOSE: Cycle through tests in test list
//******************************************************************************
var
  i: integer;
begin

  FbtnCloseClick := False;

  TestEngine.InitNextTest;

  if (TestList.CurrentTestID = TestList.NumberOfTests) then
  begin
    btnNextTest.Hint := 'Exit test battery';
    btnNextTest.Caption := '&Close';
  end
  else
  begin
    btnNextTest.Hint := 'Go on to the next test in the battery';
    btnNextTest.Caption := '&Continue';
  end;

  //Reset components on form
  grdResponses.SelectionColor := clBtnFace;

  if TestEngine.IsRestartedTest then
  begin

    //Reload the responses into string grid
    for i := 1 to length(TestEngine.ChoicesMade) do
    begin
      TestItem.ItemNumber := i;
      TestItem.GetItemData;
      //Write introduction
      if not (MHTest.Code = 'SAI') and
        (Length(Trim(TestItem.IntroductionText.Text)) > 1) then
        memIntro.Text := TestItem.IntroductionText.Text;

      if MHTest.Code = 'SAI' then
        memIntro.Clear;
      LoadResults;
    end;
  end
  else
    //Clear the responses string grid
    with grdResponses do
    begin
      RowCount := 2;
      for i := 0 to 2 do
        Cells[i, 1] := '';
    end;

  //Change form caption
  if MHTest.Name = '' then
    frmTestEntry.Caption := MHTest.Code
  else
    frmTestEntry.Caption := MHTest.Name;

  //Display copyright info
  if MHTest.Publisher <> '' then
    stsbrTestEntry.Panels[0].Text := #169 + MHTest.Publisher
  else
    stsbrTestEntry.Panels[0].Text := ' ';

  FindSkipped;
  LoadQuestion;

end;

procedure TfrmTestEntry.FormKeyPress(Sender: TObject; var Key: Char);
//******************************************************************************
// PURPOSE: Transform key entries
//******************************************************************************
begin

  if FIsKeyHeldDown or FIsEscapeKey or not FKeyPressProcessingDone then
    exit;

  FKeyPressProcessingDone := False;
    //trying to prevent too fast data entry leading to index errors

  //convert key pressed to upper case
  Key := UpCase(Key);

  //Special handling for AUIR, Item 1
  if (MHTest.Code = 'AUIR') and (TestItem.ItemNumber = 1) then
  begin
    if Key = '1' then
      Key := 'Y';
    if Key = '2' then
      Key := 'N';
  end;

  //Special handling for BDI, Item 20
  if (MHTest.Code = 'BDI') and (TestItem.ItemNumber = 20) then
  begin
    if Key = '1' then
      Key := 'Y';
    if Key = '2' then
      Key := 'N';
  end;

  //if Key is a valid response, record it
  if pos(Key, TestItem.ValidChoices) > 0 then
  begin
    TestEngine.AddToChoicesMade(Key);
    UpdateFormDisplay;
  end;

  //go to previous item, emulates roll-and-scroll MHP behavior
  if (Key = '^') and not (TestItem.ItemNumber = 1) then
  begin
    TestItem.ItemNumber := TestItem.ItemNumber - 2;
    LoadQuestion;
  end;

  //end session, emulates roll-and-scroll MHP behavior
  if Key = '*' then
    // Close
    btnNextTestClick(Sender);

  FKeyPressedDone := true;

  if FKeyDownDone and FKeyPressedDone then
    FIsKeyHeldDown := true;

  Application.ProcessMessages;
  FKeyPressProcessingDone := True;

end;

procedure TfrmTestEntry.grdResponsesClick(Sender: TObject);
//******************************************************************************
// PURPOSE: Select previously answered question
//******************************************************************************
begin
  //Break up feedback loop with strgChoicesClick.
  if not FResponsesRowClick then
    Exit;

  //Set itemnumber to clicked-on row.
  TestItem.ItemNumber := grdResponses.Row - 1;
  //Load THAT question.
  LoadQuestion;
end;

procedure TfrmTestEntry.btnNewQuestClick(Sender: TObject);
//******************************************************************************
// PURPOSE: Jumps to next unanswered question if user has backed up
//******************************************************************************
begin
  TestItem.ItemNumber := Length(TestEngine.ChoicesMade);
  LoadQuestion;
end;

procedure TfrmTestEntry.FormClose(Sender: TObject;
  var Action: TCloseAction);
//******************************************************************************
// PURPOSE: Close the entire application if patient entry
//******************************************************************************
var
  TickCount: LongWord;
begin
  if UserSession.CPRSPatientChanged then
    exit;

  case TestList.TestMode of
    tmPatient:
      begin
        //Clear the remnants of the last question
        pnlButtons.Visible := False;
        stsbrTestEntry.Visible := False;
        memIntro.Clear;
        grdChoices.Visible := False;
        prgbrTestEntry.Visible := True;
        lblExit.Visible := True;
        lblExit.BringToFront;
        lblQuestion.Caption := CRLF +
          '  Thank you for answering these questions.  That is all for now.' +
          CRLF;
        if not (TestEngine.ChoicesMadeSaved) and
          (length(TestEngine.ChoicesMade) > 0) then
          TestEngine.SaveTestResults;
        TickCount := GetTickCount;
        while GetTickCount - TickCount < 5000 do
        begin
          prgbrTestEntry.Position := (GetTickCount - TickCount) div 50;
          Application.ProcessMessages;
        end;
        //Close the entire application
        frmMain.Close;
      end; //tmPatient
  else
    if not FbtnCloseClick then
      TestEngine.UnsavedTestWarning;
  end; //case
end;

procedure TfrmTestEntry.btnBackUpClick(Sender: TObject);
//******************************************************************************
// PURPOSE: Reload previous question
//******************************************************************************
begin
  //You have to subtract 2 because LoadQuestion increments ItemNumber
  TestItem.ItemNumber := TestItem.ItemNumber - 2;
  LoadQuestion;
end;

procedure TfrmTestEntry.FormCreate(Sender: TObject);
//******************************************************************************
// PURPOSE: Set components on form create
//******************************************************************************
var
  MHAPath: string;
  r: TRect;
begin
  //  FSecureDesktopActive := false; //initialize

    //Toggle button and results visibility, depending on staff, or patient-entry
  case TestList.TestMode of

    tmStaff:
      begin
        //set form top and height to work area (i.e., desktop size minus task bars)
        SystemParametersInfo(SPI_GetWorkArea, 0, @r, 0);
        BoundsRect := r;
        Top := r.Top;
        Height := r.Bottom - r.Top;

        //set form width to 638 and center it
        Width := 638;
        Left := (r.Right - Width) div 2;

        pnlResults.Visible := True;
        WindowState := wsNormal;
        BorderStyle := bsSizeable;
        BorderIcons := [biSystemMenu, biMaximize];
      end;

    tmPatient:
      begin
        //No hints for patients.
        Application.ShowHint := False;

        MsgDlg('USE ONLY ALPHA-NUMERIC KEYS ON THE KEYBOARD FROM '
          + 'THIS POINT ON.' + #13 + #10 + '' + CRLF
          + ' - Use of unauthorized keys may cause the test to abort.' + CRLF
          + ' - Instruct the client not to touch any other keys.' + CRLF
          + ' - All keystrokes will be monitored for possible security breach attempts.',
          DlgCaption, mtWarning, [mbOK], 0);

{$IFDEF RELEASE_VER}
        frmMain.Visible := False;

        pnlResults.Visible := False;
        WindowState := wsMaximized;
        BorderStyle := bsNone;
        FormStyle := fsStayOnTop;
        BorderIcons := [];

        //Check for SecureDestop application
        MHAPath := ExtractFilePath(Application.ExeName);

        //        if FileExists(MHAPath + 'YS_MHA_SD.exe') then
        //        begin
        //          FSecureDesktopActive := True;
                  //Initialize sescurity app parameters.
        with LMDStarterSecurity do
        begin
          DefaultDir := MHAPath;
          Command := 'YS_MHA_SD.exe';
          StartOption := soSW_MINIMIZE;
          AutoStart := true;
          Wait := False;
        end;
        //        end;
{$ENDIF}
{$IFDEF DEBUG_VER}
        pnlResults.Visible := False;
        WindowState := wsNormal;
        BorderStyle := bsSizeable;
        BorderIcons := [biSystemMenu, biMaximize];
{$ENDIF}
      end;
  end;

  //Set up string grid for questions
  with grdChoices do
  begin
    RowCount := 1;
    ColWidths[0] := pnlAskQuestion.Width;
  end;

  //Set up string grid that records patient responses
  with grdResponses do
  begin
    RowCount := 2;
    FixedRows := 1;
    ColWidths[0] := 30;
    ColWidths[1] := 30;
    Cells[0, 0] := ' No.';
    Cells[1, 0] := ' Ans.';

    if (TestList.TestMode = tmStaff) then
    begin
      ColWidths[2] := pnlResults.Width - 72;
      Cells[2, 0] := ' Question';
    end;
  end;

  TestList.CurrentTestID := 0;
  FResponsesRowClick := True;
  FChoicesRowClick := True;
  FIsKeyHeldDown := False;
  FKeyPressProcessingDone := True;
end;

procedure TfrmTestEntry.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
//******************************************************************************
// PURPOSE: What happens when a key is pressed. Needed this for the vk_* values.
//******************************************************************************
begin

  if (key = vk_Escape) and not (TestList.TestMode = tmPatient) then
  begin
    Close;
    key := $00;
    FIsKeyHeldDown := False;
    FIsEscapeKey := True;
    Exit;
  end
  else
    FIsEscapeKey := False;

  if FIsKeyHeldDown then
    exit;

  if ((key = vk_Return) or (key = vk_Up) or
    (key = vk_Down) or (key = vk_Left) or (key = vk_Right)
    or (key = vk_Prior) or (key = vk_Next)) or FIsKeyHeldDown then
    key := $00;

  if (key = vk_Back) and not (TestItem.ItemNumber = 1) then
  begin
    TestItem.ItemNumber := TestItem.ItemNumber - 2;
    LoadQuestion;
  end;

  FKeyDownDone := true;

end;

procedure TfrmTestEntry.UpdateFormDisplay;
//******************************************************************************
// PURPOSE: Cycle/refresh controls on the form.
//******************************************************************************
begin
  LoadResults;
  LoadQuestion;
  FindSkipped;
end;

procedure TfrmTestEntry.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
//******************************************************************************
// PURPOSE: Prevent repeating keystroke processing, when key is held down.
//******************************************************************************
begin
  FIsKeyHeldDown := False;
end;

procedure TfrmTestEntry.btnNextTestClick(Sender: TObject);
begin

  FbtnCloseClick := True;

  //save the test if complete or patient entry
  if (TestList.TestMode = tmPatient) then
    TestEngine.SaveTestResults
  else
  begin

    //Chance to finish the test if incomplete
    if TestEngine.WantToFinishTest then
      Exit;

    //ask if the user wants to save unsaved test data
    TestEngine.UnsavedTestWarning;

  end; //else

  //start next test if there is one, exit if not
  if (TestList.CurrentTestID < TestList.NumberOfTests) then
    StartNextTest
  else
    close;

end;

procedure TfrmTestEntry.FormDestroy(Sender: TObject);
begin
{$IFDEF RELEASE_VER}
  //Logout the current NT user, forcing all apps to close
  if TestList.TestMode = tmPatient then
    ExitWindowsEx(EWX_LOGOFF + EWX_FORCE, 0);
{$ENDIF}
end;

procedure TfrmTestEntry.FormShow(Sender: TObject);
begin
  Application.ProcessMessages;
  StartNextTest;
  grdChoices.SetFocus;

  //Invoke Security app
  if (TestList.TestMode = tmPatient) then
    //  if FSecureDesktopActive then
    LMDStarterSecurity.Execute;
  //    else
  //      frmMain.Close;

end;

procedure TfrmTestEntry.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if (UserSession.CPRSPatientChanged = true) then
  begin
    Canclose := True;
    exit;
  end;

  if not FbtnCloseClick then
    CanClose := not TestEngine.WantToFinishTest;
end;

end.

