{***************************************************************

 UNIT NAME: rMHA_VISTA
 PACKAGE: Mental Health
 WARNING:
 DATE CREATED: Sep, 2000
 SITE NAME: Salt Lake City Mental Health Informatics Section
 DEVELOPERS: CANNON/FELDMAN
 DESCRIPTION: Unit to interface with Vista data. Mainly Broker calls
              to load setters in objects.
 NOTE: This unit requires Mental Health patch 67, or greater, in
       order to run.

 HISTORY  :

 Required third-party Delphi components and libraries:

  CPRS
  Delphi FileMan Components

****************************************************************}
unit rMHA_VISTA;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls;

function GetListOfPreviousGAFs: TStrings;
function GetListOfPreviousTests: TStrings;
function GetOrderableTests: TStrings;
function GetListOfIncompleteTests: TStrings;
function GetListOfPreviousASIs: TStrings;
function GetScaleScores(tmpName: string): TStrings;
function GetASIReport(ReportType, ASIIens: string): TStrings;
function GetTestReport(RPCroutine, PtDFN, YStxt: string): TStrings;
procedure SaveGAFScore(PatientDFN, Score, ScoreDate, Staff: string);
procedure ChangeGAFScore(aIENS, aScore: String);
procedure DeleteGAF(aIENS: String);
procedure MarkGAFInError(aIENS: String);
function GetClerkEntryValidResponses(tmpName: string): TStrings;
function GetListOfBatteries: TStrings;
function GetEvaluationFactors(ASIIens: string): TStrings;
function GetSeverityRatings(ASIIens: string): TStrings;

implementation

uses
 fMHA_Main, //Main form
 TRPCB,
 MFUNStr,
 DiTypLib, // For DeleteRecord
 ESBDates, // For comparing dates
 ORFn,
 uCore,
 rCore, //for FMNow
 ORNet,
 uMHA_DM;
var
  //Temporary stringlist for parsing and other mundane chores
  TempStrLst: TStrings;

//******************************************************************************
// GetListOfPreviousGAFs [X]USED [X]DONE [ ]TESTED
// PURPOSE: Returns list of previous GAF scores along with date and rater
// AUTHOR: Feldman-Cannon // REVIEWER: Feldman-Cannon
// LAST MODIFICATION/REVIEW DATE: 02/16/01
//******************************************************************************

function GetListOfPreviousGAFs: TStrings;
begin

  with RPCBrokerV do
  begin
    ClearResults := True;
    ClearParameters := True;
    RemoteProcedure := 'YS GAF API';
    Param[0].Mult['"DFN"'] := Patient.DFN;
    Param[0].PType := List;
    Call;
  end;

  //delete the first record, which just says [DATA]
  RPCBrokerV.Results.Delete(0);

  Result := RPCBrokerV.Results;

end;

//******************************************************************************
// GetListOfPreviousTests [X]USED [X]DONE [ ]TESTED
// PURPOSE: Returns list of previously completed tests along with date
// AUTHOR: Feldman-Cannon // REVIEWER: Feldman-Cannon
// LAST MODIFICATION/REVIEW DATE: 02/16/01
//******************************************************************************

function GetListOfPreviousTests: TStrings;
begin

  with RPCBrokerV do begin
    ClearResults := True;
    ClearParameters := True;
    RemoteProcedure := 'YTAPI LISTALL';
    Param[0].Mult['"DFN"'] := Patient.DFN;
    Param[0].Mult['"END"'] := 'T';
    Param[0].PType := List;

    call;
  end;

  //delete the first record, which just says "[DATA]"
  RPCBrokerV.Results.Delete(0);

  Result := RPCBrokerV.Results;

end;

//******************************************************************************
// GetOrderableTests [X]USED [X]DONE [ ]TESTED
// PURPOSE: Returns list of tests that TestOrderer is permitted to order
// AUTHOR: Feldman-Cannon // REVIEWER: Feldman-Cannon
// LAST MODIFICATION/REVIEW DATE: 02/16/01
//******************************************************************************

function GetOrderableTests: TStrings;
var
  name, ien: string;
  x: integer;
begin

  TempStrLst.Clear;

  //Get list of available psych tests stored on the server
  //Screened by YSP key of test orderer, not user

  with MHA_DataModule.FMLister1 do begin
    RPCBroker := RPCBrokerv;
    FileNumber := '601';
    FieldNumbers.Clear;
    FieldNumbers.Add('.01');

    if TestOrderer.HasYSPKey then
      Screen := 'I $P(^(0),U,9)="T"&($P(^(0),U,13)' + #39 + '="N")'
    else
      Screen := 'I $P(^(0),U,9)="T"&($P(^(0),U,13)' + #39 + '="N"&($P(^(0),U,10)="Y"))';

    GetList(nil);
  end;

  //Converts FMLister contents to test name
  for x := 0 to MHA_DataModule.FMLister1.Results.Count - 1 do
  begin
    ien := MHA_DataModule.FMLister1.Results[x];
    name := MHA_DataModule.FMLister1.GetRecord(ien).GetField('.01').FMDBExternal;
    TempStrLst.Add(ien + U + name);
  end;

  Result := TempStrLst;

end;

//******************************************************************************
// GetListOfIncompleteTests [X]USED [X]DONE [X]TESTED
// PURPOSE: Retrieve list of incomplete tests from Vista
// AUTHOR: R. FELDMAN // REVIEWER:
// LAST MODIFICATION/REVIEW DATE: 02/16/01
//******************************************************************************

function GetListOfIncompleteTests: TStrings;
begin

  with RPCBrokerV do begin
    ClearResults := True;
    ClearParameters := True;
    RemoteProcedure := 'YTAPI LIST INCOMPLETES';
    Param[0].Mult['"DFN"'] := Patient.DFN;
    Param[0].Mult['"END"'] := 'T';
    Param[0].PType := List;
    call;
  end;
  Result := RPCBrokerV.Results;

end;

//******************************************************************************
// GetListOfPreviousASIs [X]USED [X]DONE []TESTED
// PURPOSE:
// AUTHOR: Cannon // REVIEWER:  Feldman
// LAST MODIFICATION/REVIEW DATE: 04/06/01
//******************************************************************************

function GetListOfPreviousASIs: TStrings;
begin
  //Load test names and dates of previous tests
  with RPCBrokerV do begin
    ClearResults := True;
    ClearParameters := True;
    RemoteProcedure := 'YTAPI ASI LISTER';
    Param[0].Mult['"DFN"'] := Patient.DFN;
    Param[0].PType := List;
    call;
  end;

  Result := RPCBrokerV.Results;

end;

//******************************************************************************
// GetListOfBatteries []USED []DONE []TESTED
// PURPOSE:
// AUTHOR: Feldman // REVIEWER: Cannon
// LAST MODIFICATION/REVIEW DATE: 08/17/01
//******************************************************************************

function GetListOfBatteries: TStrings;
begin

  with RPCBrokerv do
  begin
    ClearResults := True;
    ClearParameters := True;
    RemoteProcedure := 'YTAPI TEST BATTERY';
    Call;
  end;

  //delete the first record, which just says [DATA]
  RPCBrokerV.Results.Delete(0);

  Result := RPCBrokerV.Results;

end;

//******************************************************************************
// GetScaleScores [X]USED [X]DONE []TESTED
// PURPOSE: Retrieve from Vista listo of all scores for a particular test-type
//          and patient.
// AUTHOR: Cannon // REVIEWER:  Feldman
// LAST MODIFICATION/REVIEW DATE: 04/26/01
//******************************************************************************

function GetScaleScores(tmpName: string): TStrings;
begin
  with RPCBrokerv do
  begin
    ClearResults := True;
    ClearParameters := True;
    RemoteProcedure := 'YTAPI LISTONE';
    Param[0].Mult['"DFN"'] := Patient.DFN; //PtData.DFN;
    Param[0].Mult['"SCALE"'] := '1';
    Param[0].Mult['"CODE"'] := tmpName;
    Param[0].PType := List;
    Call;
    Result := RPCBrokerV.Results;
  end;

end;

//******************************************************************************
// GetClerkEntryValidResponses [X]USED []DONE []TESTED
// PURPOSE: Responses for Clerk Entry form
// AUTHOR: Cannon // REVIEWER:  Feldman
// LAST MODIFICATION/REVIEW DATE: 08/16/01
//******************************************************************************

function GetClerkEntryValidResponses(tmpName: string): TStrings;
var
  i: integer;
begin

  TempStrLst.Clear;

  with RPCBrokerv do
  begin
    ClearResults := True;
    ClearParameters := True;
    RemoteProcedure := 'YTAPI CLERK RESPONSES';
    Param[0].Mult['"CODE"'] := tmpName;
    Param[0].PType := List;
    Call;
    for i := 1 to MHTest.NumberOfItems do
      TempStrLst.Add(Piece(RPCBrokerV.Results[i], '^', 3));
  end;

  Result := TempStrLst;

end;

//******************************************************************************
// GetASIReport []USED []DONE []TESTED
// PURPOSE:
// AUTHOR: Cannon // REVIEWER:  Feldman
// LAST MODIFICATION/REVIEW DATE: 00/00/01
//******************************************************************************

function GetASIReport(ReportType, ASIIens: string): TStrings;
begin
  with RPCBrokerv do
  begin
    ClearResults := True;
    ClearParameters := True;
    RemoteProcedure := ReportType;
    Results.Clear;
    Param[0].Value := ASIIens;
    Param[0].PType := literal;
    Call;
    Result := Results;
  end;

end;

//******************************************************************************
// GetTestReport [X]USED [D]DONE []TESTED
// PURPOSE: Load test reports from Vista
// AUTHOR: Feldman// REVIEWER: Cannon
// LAST MODIFICATION/REVIEW DATE: 05/08/01
//******************************************************************************

function GetTestReport(RPCroutine, PtDFN, YStxt: string): TStrings;
begin
  with RPCBrokerV do
  begin
    ClearResults := True;
    ClearParameters := True;
    RemoteProcedure := RPCroutine;
    Results.Clear;
    Param[0].Value := PtDFN;
    Param[0].PType := literal;
    Param[1].Value := YStxt;
    Param[1].PType := literal;
    Call;
    Result := Results;
  end;
end;

//******************************************************************************
// SaveGAFScore [X]USED [X]DONE [X]TESTED
// PURPOSE: Save current GAF score to Vista.
// AUTHOR: Cannon // REVIEWER:  Feldman
// LAST MODIFICATION/REVIEW DATE: 04/30/01
//******************************************************************************

procedure SaveGAFScore(PatientDFN, Score, ScoreDate, Staff: string); //Save new GAF rating
begin
  with RPCBrokerV do begin
    ClearResults := True;
    ClearParameters := True;
    RemoteProcedure := 'YTAPI NEW GAF';
    Param[0].Mult['"DFN"'] := PatientDFN;
    Param[0].Mult['"GAF"'] := Score;
    Param[0].Mult['"DATE"'] := ScoreDate;
    Param[0].Mult['"STAFF"'] := Staff;
    Param[0].PType := List;
    Call;
  end;
end;

//******************************************************************************
// ChangeGAFScore [X]USED [X]DONE [X]TESTED
// PURPOSE: Change GAF score stored in Vista.
// AUTHOR: Cannon // REVIEWER:  Feldman
// LAST MODIFICATION/REVIEW DATE: 02/10/02
//******************************************************************************

procedure ChangeGAFScore(aIENS, aScore: String);
begin

  With MHA_DataModule do
  begin

  FMFiler1.RPCBroker := RPCBrokerV;
  FMValidator1.RPCBroker := RPCBrokerV;

  //Validate the value of the field
  FMValidator1.Setup('627.8', aIENS, '65', aScore);
  If not FMValidator1.Validate then
    FMValidator1.DisplayErrors
  else begin {file}
    //Put validated field for record on Filer's change list
    FMFiler1.AddFDA('627.8', aIENS, '65', aScore);
                 //(FileNumber, IENS, FieldNumber, NewScore)
    //File record
    if not FMFiler1.Update then FMFiler1.DisplayErrors;
  end;
  end;
end;

//******************************************************************************
// MarkGAFInError [X]USED [X]DONE [X]TESTED
// PURPOSE: Mark GAF score as error in Vista.
// AUTHOR: Cannon // REVIEWER:  Feldman
// LAST MODIFICATION/REVIEW DATE: 02/11/02
//******************************************************************************

procedure MarkGAFInError(aIENS: String);
begin
  With MHA_DataModule do
  begin
  FMFiler1.RPCBroker := RPCBrokerV;
  FMValidator1.RPCBroker := RPCBrokerV;

  //Save Memo Field
    with frmMain.FMMemoGAFInError do
    begin
      Clear;
      Lines.Add('Error: entered in error noted on ' +
      // ex. FEB 7,2002@09:45:03
      FormatFMDateTime('mmm dd,yyyy' + '@' + 'hh:mm:ss', FMToday)
      + ' by ' +
      User.Name); // ex. FINKELSTEIN,ALLAN
      // Set the field and file
      FMFile := '627.8';
      FMField := '80';
      IENS := aIENS;
      // Set the FMFiler property to the TFMFiler component to use for file changes.
      FMFiler := FMFiler1;
        //Add the memo control to the list of controls to be updated.
    end;
      FMFiler1.AddChgdControl(frmMain.FMMemoGAFInError);

    //File record
    if not FMFiler1.Update then FMFiler1.DisplayErrors;
  end;
end;

//******************************************************************************
// DeleteGAF [X]USED [X]DONE [X]TESTED
// PURPOSE: Delete GAF record in Vista.
// AUTHOR: Cannon // REVIEWER:  Feldman
// LAST MODIFICATION/REVIEW DATE: 02/10/02
//******************************************************************************

procedure DeleteGAF(aIENS: String);
begin
  DeleteRecord(RPCBrokerV, '627.8', aIENS + ',');
end;

//******************************************************************************
// GetEvaluationFactors [ ]USED [ ]DONE []TESTED
// PURPOSE:
// AUTHOR: Feldman // REVIEWER: Cannon
// LAST MODIFICATION/REVIEW DATE: 10/25/01
//******************************************************************************

function GetEvaluationFactors(ASIIens: string): TStrings;
begin
  //Load ASI Eval Factors
  with RPCBrokerV do begin
    ClearResults := True;
    ClearParameters := True;
    RemoteProcedure := 'YTAPI ASI FACTORS';
    Param[0].Mult['"IEN"'] := ASIIens;
    Param[0].PType := List;
    call;
  end;

  Result := RPCBrokerV.Results;

end;

//******************************************************************************
// GetSeverityRatings []USED []DONE []TESTED
// PURPOSE:
// AUTHOR: Feldman // REVIEWER: Cannon
// LAST MODIFICATION/REVIEW DATE: 10/29/01
//******************************************************************************

function GetSeverityRatings(ASIIens: string): TStrings;
Var
  i: Integer;
  strFieldCounter: String;
begin
  with MHA_DataModule.FMGets1 do
  begin
    FieldNumbers.Clear;
    RPCBroker := RPCBrokerv;
    FileNumber := '604';
    FieldNumbers.Add('.11');

    for i := 1 to 7 do
    begin
      case i of
        1: strFieldCounter := '8.12'; //Medical
        2: strFieldCounter := '9.34'; //Employment
        3: strFieldCounter := '11.18'; //Alcohol
        4: strFieldCounter := '11.185'; //Drug
        5: strFieldCounter := '14.34'; //Legal
        6: strFieldCounter := '18.29'; //Fam/Soc
        7: strFieldCounter := '19.33'; //Psych
      end;
      FieldNumbers.Add(strFieldCounter);
    end;

  end;

  //Get data
  MHA_DataModule.FMGets1.IENS := ASIIens;
  MHA_DataModule.FMGets1.GetData;
    {check if errors occurred during server call}
  if MHA_DataModule.FMGets1.ErrorList.Count > 0 then
    MHA_DataModule.FMGets1.DisplayErrors;

  Result := MHA_DataModule.FMGets1.Results;

end;


//******************************************************************************
// Unit Initialization [X]USED [X]DONE [ ]TESTED
// PURPOSE: Create the temporary stringlist objec
// AUTHOR: FELDMAN // REVIEWER:
// LAST MODIFICATION/REVIEW DATE: 02/16/01
//******************************************************************************
initialization
  TempStrLst := TStringList.Create;
  TempStrLst.Clear;

//******************************************************************************
// Unit Finalization [X]USED [X]DONE [ ]TESTED
// PURPOSE: Destroy the temporary stringlist object, if it exists.
// AUTHOR: FELDMAN // REVIEWER:
// LAST MODIFICATION/REVIEW DATE: 02/16/01
//******************************************************************************
finalization
  if TempStrLst <> nil then TempStrLst.Free;

end.

