unit fMHA_ASI_Main;
{- Mental Health Assistant, ASI Data Entry
   Patch One

   Designed by Roger Schultz, Ph.D.
   Mental Health Strategic Health Care Group, Informatics Section
   Revision Date: 8/2002

   Description:
   This unit provides a GUI for Addiction Severity Index. It requires:
     uMHA_ASI_Save,
     uMHA_ASI_Hints,
     uMHA_ASI_Build,
     uMHA_ASI_Vista,
     fMHA_ASI_Options,
     fMHA_ASI_Close,
     fMHA_ASI_Sign and
     uMHA_Registry.

   Compiled in Delphi 5.

   Each category of the ASI (e.g., Medical, Legal) has its own scroll box on the
   TabSheet.

   It is designed to allow rapid keyboard data-entry, although mouse data-entry is
   also supported. For keyboard entry, a response will score the current item and
   move automatically to the next item (speed tab). The speed tab can be disengaged
   by the user.

   This form presents three versions of the ASI: Full, Lite & Follow-up. In most
   aspects the DHCP presentation, business rules, and possible responses are used
   here.  There are exceptions. On the DHCP version most items had four possible
   responses: "0. No," "1. Yes," "X. Not Answered," & "N. Not Appliciable" but in
   this version, there are few items that have a "N" as a possible response. See
   page 92 of the ASI Manual and Question by Question Guide, VA Training 1997 for
   busines rules for the "N" response.

   The following questions do not appear on the Lite version but do appear on the
   followup version: M9, E8, E19, E20, E22, D14, D17, D18, D21, D22, D33, L22, L23,
   L30, F9, F10, F11, F27, F33, F35, F36, P14, P15, P16, P17, P18, P19, P20. Thus
   the Lite-Followup is longer than the Lite by 28 items.

   The position of questions, responses, lables, etc. are adjusted at run-time
   for top alignment. The left alignment is set at design time.

   Allowed values for income, items E12, E13, E14, E15, E16, E17, are whole numbers,
   no decimals or "$" sign. Range is 0..9999.

   Spin-edit components are a bit clunky since the letters "X" and "N" are required.
   To handle this the edit field is connected to a spin component.

   If this ASI is a partially completed ASI that the clinician (user) is
   finishing then set "Partial" to 'A' for Full ASI,
                                   'L' for Lite
                                   'F' for Followup
                                   ' ' for new ASI

-}


interface

uses
  Classes, ComCtrls, Controls, Dialogs,
  ExtCtrls, Forms, Graphics,
  Mask, Menus, Messages, fSignItem,
  RzCmboBx, RzEdit, RzPanel, RzRadGrp, RzSplit, RzSpnEdt,
  StdCtrls, Windows, StExpr, ORCtrls, Trpcb, Diaccess, Fmcmpnts, Fmcntrls,
  Grids, AdvGrid, SysUtils, Buttons;

type
  TfrmASI = class(TForm)
    MainMenu1: TMainMenu;
    mnuFile: TMenuItem;
    mnuClose: TMenuItem;
    mnuHelp: TMenuItem;
    mnuIndex: TMenuItem;
    N2: TMenuItem;
    mnuAbout: TMenuItem;
    pgctrlASI: TPageControl;
    TabGeneral: TTabSheet;
    TabMedical: TTabSheet;
    TabEmploy: TTabSheet;
    TabDrug: TTabSheet;
    TabLegal: TTabSheet;
    TabFamily: TTabSheet;
    TabSocial: TTabSheet;
    TabPsych: TTabSheet;
    TabSpirit: TTabSheet;
    TabLeisure: TTabSheet;
    spltEmploy: TRzSplitter;
    spltDrug: TRzSplitter;
    spltLegal: TRzSplitter;
    spltFamily: TRzSplitter;
    spltSocial: TRzSplitter;
    Panel2: TPanel;
    sbEmploy: TScrollBox;
    sbDrug: TScrollBox;
    sbLegal: TScrollBox;
    sbFamily: TScrollBox;
    sbSocial: TScrollBox;
    pnlGeneral: TPanel;
    pnlEmploy: TPanel;
    pnlDrug: TPanel;
    pnlLegal: TPanel;
    pnlFamily: TPanel;
    pnlSocial: TPanel;
    lblE1: TLabel;
    edtE1y: TEdit;
    SpnE1y: TRzSpinButtons;
    lblE1m: TLabel;
    edtE1m: TEdit;
    SpnE1m: TRzSpinButtons;
    SpnE2: TRzSpinButtons;
    edtE2: TEdit;
    lblE2: TLabel;
    rgrpE3: TRzRadioGroup;
    edtE3e: TRzEdit;
    rgrpE4: TRzRadioGroup;
    rgrpE5: TRzRadioGroup;
    lblE6: TLabel;
    edtE6y: TEdit;
    SpnE6y: TRzSpinButtons;
    lblE6m: TLabel;
    edtE6m: TEdit;
    spnE6m: TRzSpinButtons;
    rgrpE7: TRzRadioGroup;
    btnHollingshead: TButton;
    lblE7: TLabel;
    edtE7e: TRzEdit;
    rgrpE8: TRzRadioGroup;
    rgrpE9: TRzRadioGroup;
    rgrpE10: TRzRadioGroup;
    lblE11: TLabel;
    lblE12: TLabel;
    lblE13: TLabel;
    lblE14: TLabel;
    lblE15: TLabel;
    lblE16: TLabel;
    lblE17: TLabel;
    lblE18: TLabel;
    lblE19: TLabel;
    edtE11: TEdit;
    SpnE11: TRzSpinButtons;
    edtE12: TEdit;
    edtE13: TEdit;
    edtE14: TEdit;
    edtE15: TEdit;
    edtE16: TEdit;
    edtE17: TEdit;
    edtE18: TEdit;
    SpnE18: TRzSpinButtons;
    edtE19: TEdit;
    spnE19: TRzSpinButtons;
    rgrpE20: TRzRadioGroup;
    rgrpE21: TRzRadioGroup;
    rgrpE22: TRzRadioGroup;
    rgrpE23: TRzRadioGroup;
    rgrpE24: TRzRadioGroup;
    lbl30d: TLabel;
    lblly: TLabel;
    lblra: TLabel;
    cbxD1r: TRzComboBox;
    spnD1y: TRzSpinButtons;
    edtD1y: TEdit;
    spnD1d: TRzSpinButtons;
    edtD1d: TEdit;
    lblD1: TLabel;
    lblD2: TLabel;
    lblD3: TLabel;
    lblD4: TLabel;
    lblD5: TLabel;
    lblD6: TLabel;
    lblD7: TLabel;
    lblD8: TLabel;
    lblD9: TLabel;
    lblD10: TLabel;
    lblD11: TLabel;
    lblD12: TLabel;
    lblD13: TLabel;
    edtD2d: TEdit;
    edtD3d: TEdit;
    edtD4d: TEdit;
    edtD5d: TEdit;
    edtD6d: TEdit;
    edtD7d: TEdit;
    edtD8d: TEdit;
    edtD9d: TEdit;
    edtD10d: TEdit;
    edtD11d: TEdit;
    edtD12d: TEdit;
    edtD13d: TEdit;
    spnD13d: TRzSpinButtons;
    spnD12d: TRzSpinButtons;
    spnD11d: TRzSpinButtons;
    spnD10d: TRzSpinButtons;
    spnD9d: TRzSpinButtons;
    spnD8d: TRzSpinButtons;
    spnD7d: TRzSpinButtons;
    spnD6d: TRzSpinButtons;
    spnD5d: TRzSpinButtons;
    spnD4d: TRzSpinButtons;
    spnD3d: TRzSpinButtons;
    spnD2d: TRzSpinButtons;
    edtD2y: TEdit;
    edtD3y: TEdit;
    edtD4y: TEdit;
    edtD5y: TEdit;
    edtD6y: TEdit;
    edtD7y: TEdit;
    edtD8y: TEdit;
    edtD9y: TEdit;
    edtD10y: TEdit;
    edtD11y: TEdit;
    edtD12y: TEdit;
    edtD13y: TEdit;
    spnD13y: TRzSpinButtons;
    spnD12y: TRzSpinButtons;
    spnD11y: TRzSpinButtons;
    spnD10y: TRzSpinButtons;
    spnD9y: TRzSpinButtons;
    spnD8y: TRzSpinButtons;
    spnD7y: TRzSpinButtons;
    spnD6y: TRzSpinButtons;
    spnD5y: TRzSpinButtons;
    spnD4y: TRzSpinButtons;
    spnD3y: TRzSpinButtons;
    spnD2y: TRzSpinButtons;
    cbxD2r: TRzComboBox;
    cbxD3r: TRzComboBox;
    cbxD4r: TRzComboBox;
    cbxD5r: TRzComboBox;
    cbxD6r: TRzComboBox;
    cbxD7r: TRzComboBox;
    cbxD8r: TRzComboBox;
    cbxD9r: TRzComboBox;
    cbxD10r: TRzComboBox;
    cbxD11r: TRzComboBox;
    cbxD12r: TRzComboBox;
    rgrpD14: TRzRadioGroup;
    lblD15: TLabel;
    lblD16: TLabel;
    lblD17: TLabel;
    lblD18: TLabel;
    lblD19: TLabel;
    lblD20: TLabel;
    lblD21: TLabel;
    lblD22: TLabel;
    lblD23: TLabel;
    lblD24: TLabel;
    edtD15: TEdit;
    edtD16: TEdit;
    edtD17: TEdit;
    edtD18: TEdit;
    edtD19: TEdit;
    edtD20: TEdit;
    edtD21: TEdit;
    edtD22: TEdit;
    edtD23: TEdit;
    edtD24: TEdit;
    spnD24: TRzSpinButtons;
    spnD23: TRzSpinButtons;
    spnD22: TRzSpinButtons;
    spnD21: TRzSpinButtons;
    spnD20: TRzSpinButtons;
    spnD19: TRzSpinButtons;
    spnD18: TRzSpinButtons;
    spnD17: TRzSpinButtons;
    spnD16: TRzSpinButtons;
    lblD25: TLabel;
    edtD25: TEdit;
    spnD25: TRzSpinButtons;
    spnD26: TRzSpinButtons;
    edtD26: TEdit;
    lblD26: TLabel;
    lblD27: TLabel;
    edtD27: TEdit;
    spnD27: TRzSpinButtons;
    rgrpD28: TRzRadioGroup;
    rgrpD29: TRzRadioGroup;
    rgrpD30: TRzRadioGroup;
    rgrpD32: TRzRadioGroup;
    rgrpD33: TRzRadioGroup;
    rgrpD34: TRzRadioGroup;
    rgrpD35: TRzRadioGroup;
    rgrpD31: TRzRadioGroup;
    rgrpL1: TRzRadioGroup;
    rgrpL2: TRzRadioGroup;
    lblL3to16: TLabel;
    lblL3: TLabel;
    lblL4: TLabel;
    lblL5: TLabel;
    lblL6: TLabel;
    lblL7: TLabel;
    lblL8: TLabel;
    lblL9: TLabel;
    lblL17: TLabel;
    edtL9: TEdit;
    edtL8: TEdit;
    edtL7: TEdit;
    edtL6: TEdit;
    edtL5: TEdit;
    edtL4: TEdit;
    edtL3: TEdit;
    spnL3: TRzSpinButtons;
    spnL4: TRzSpinButtons;
    spnL5: TRzSpinButtons;
    spnL6: TRzSpinButtons;
    spnL7: TRzSpinButtons;
    spnL8: TRzSpinButtons;
    spnL9: TRzSpinButtons;
    lblL10: TLabel;
    lblL11: TLabel;
    lblL12: TLabel;
    lblL13: TLabel;
    lblL14: TLabel;
    lblL15: TLabel;
    lblL16: TLabel;
    edtL10: TEdit;
    edtL11: TEdit;
    edtL12: TEdit;
    edtL13: TEdit;
    edtL14: TEdit;
    edtL15: TEdit;
    edtL16: TEdit;
    edtL17: TEdit;
    spnL17: TRzSpinButtons;
    spnL16: TRzSpinButtons;
    spnL15: TRzSpinButtons;
    spnL14: TRzSpinButtons;
    spnL13: TRzSpinButtons;
    spnL12: TRzSpinButtons;
    spnL11: TRzSpinButtons;
    spnL10: TRzSpinButtons;
    lblL18toL21: TLabel;
    lblL18: TLabel;
    edtL18: TEdit;
    spnL18: TRzSpinButtons;
    spnL19: TRzSpinButtons;
    edtL19: TEdit;
    lblL19: TLabel;
    lblL20: TLabel;
    lblL21: TLabel;
    lblL22: TLabel;
    rgrpL23: TRzRadioGroup;
    edtL20: TEdit;
    edtL21: TEdit;
    edtL22: TEdit;
    spnL22: TRzSpinButtons;
    spnL21: TRzSpinButtons;
    spnL20: TRzSpinButtons;
    rgrpL24: TRzRadioGroup;
    rgrpL25: TRzRadioGroup;
    lblL26: TLabel;
    lblL27: TLabel;
    edtL26: TEdit;
    spnL26: TRzSpinButtons;
    spnL27: TRzSpinButtons;
    edtL27: TEdit;
    rgrpL28: TRzRadioGroup;
    rgrpL29: TRzRadioGroup;
    rgrpL30: TRzRadioGroup;
    rgrpL31: TRzRadioGroup;
    rgrpL32: TRzRadioGroup;
    lblHF1: TLabel;
    lblHF2: TLabel;
    lblH1: TLabel;
    lblH2: TLabel;
    LblH3: TLabel;
    lblH4: TLabel;
    lblH5: TLabel;
    lblH1a: TLabel;
    lblH2a: TLabel;
    lblH3a: TLabel;
    lblH4a: TLabel;
    lblH5a: TLabel;
    cbxH1a: TRzComboBox;
    cbxH2a: TRzComboBox;
    cbxH3a: TRzComboBox;
    cbxH4a: TRzComboBox;
    cbxH5a: TRzComboBox;
    cbxH1d: TRzComboBox;
    cbxH2d: TRzComboBox;
    cbxH3d: TRzComboBox;
    cbxH4d: TRzComboBox;
    cbxH5d: TRzComboBox;
    cbxH5p: TRzComboBox;
    cbxH4p: TRzComboBox;
    cbxH3p: TRzComboBox;
    cbxH2p: TRzComboBox;
    cbxH1p: TRzComboBox;
    lblH1p: TLabel;
    lblH2p: TLabel;
    lblH3p: TLabel;
    lblH4p: TLabel;
    lblH5p: TLabel;
    lblH5d: TLabel;
    lblH4d: TLabel;
    lblH3d: TLabel;
    lblH6a: TLabel;
    lblH6: TLabel;
    lblHF3: TLabel;
    lblH7: TLabel;
    lblH8: TLabel;
    lblH9: TLabel;
    lblH10: TLabel;
    lblH10a: TLabel;
    lblH9a: TLabel;
    lblH8a: TLabel;
    lblH7a: TLabel;
    lblH6d: TLabel;
    lblH7d: TLabel;
    lblH8d: TLabel;
    lblH9d: TLabel;
    lblH10d: TLabel;
    cbxH6a: TRzComboBox;
    cbxH7a: TRzComboBox;
    cbxH8a: TRzComboBox;
    cbxH9a: TRzComboBox;
    cbxH10a: TRzComboBox;
    cbxH10d: TRzComboBox;
    cbxH9d: TRzComboBox;
    cbxH8d: TRzComboBox;
    cbxH7d: TRzComboBox;
    cbxH6d: TRzComboBox;
    lblH6p: TLabel;
    lblH7p: TLabel;
    lblH8p: TLabel;
    lblH9p: TLabel;
    lblH10p: TLabel;
    cbxH10p: TRzComboBox;
    cbxH9p: TRzComboBox;
    cbxH8p: TRzComboBox;
    cbxH7p: TRzComboBox;
    cbxH6p: TRzComboBox;
    lblHF4: TLabel;
    lblH11: TLabel;
    lblH12: TLabel;
    lblH11a: TLabel;
    lblH12a: TLabel;
    cbxH11a: TRzComboBox;
    cbxH12a: TRzComboBox;
    lblH11d: TLabel;
    lblH12d: TLabel;
    cbxH11d: TRzComboBox;
    cbxH12d: TRzComboBox;
    lblH11p: TLabel;
    lblH12p: TLabel;
    cbxH11p: TRzComboBox;
    cbxH12p: TRzComboBox;
    lblH1d: TLabel;
    lblH2d: TLabel;
    rgrpF1: TRzRadioGroup;
    lblF2: TLabel;
    edtF2y: TEdit;
    spnF2y: TRzSpinButtons;
    lblF2m: TLabel;
    edtF2m: TEdit;
    spnF2m: TRzSpinButtons;
    rgrpF3: TRzRadioGroup;
    rgrpF4: TRzRadioGroup;
    lblF5: TLabel;
    lblF5y: TLabel;
    edtF5y: TEdit;
    spnF5y: TRzSpinButtons;
    lblF5m: TLabel;
    spnF5m: TRzSpinButtons;
    rgrpF6: TRzRadioGroup;
    rgrpF7: TRzRadioGroup;
    rgrpF8: TRzRadioGroup;
    rgrpF9: TRzRadioGroup;
    rgrpF10: TRzRadioGroup;
    lblF11: TLabel;
    edtF11: TEdit;
    spnF11: TRzSpinButtons;
    lblF12toF17: TLabel;
    lblF12: TLabel;
    cbxF12: TRzComboBox;
    lblF15: TLabel;
    cbxF15: TRzComboBox;
    cbxF16: TRzComboBox;
    lblF16: TLabel;
    cbxF13: TRzComboBox;
    lblF13: TLabel;
    lblF14: TLabel;
    cbxF14: TRzComboBox;
    lblF17: TLabel;
    cbxF17: TRzComboBox;
    lblF: TLabel;
    lbl30: TLabel;
    lbliyl: TLabel;
    rgrpF18: TRzRadioGroup;
    rgrpF19: TRzRadioGroup;
    rgrpF20: TRzRadioGroup;
    rgrpF21: TRzRadioGroup;
    rgrpF22: TRzRadioGroup;
    rgrpF23: TRzRadioGroup;
    rgrpF24: TRzRadioGroup;
    rgrpF25: TRzRadioGroup;
    rgrpF26: TRzRadioGroup;
    rgrpF26l: TRzRadioGroup;
    rgrpF25l: TRzRadioGroup;
    rgrpF24l: TRzRadioGroup;
    rgrpF23l: TRzRadioGroup;
    rgrpF22l: TRzRadioGroup;
    rgrpF21l: TRzRadioGroup;
    rgrpF20l: TRzRadioGroup;
    rgrpF19l: TRzRadioGroup;
    rgrpF18l: TRzRadioGroup;
    lbl37: TLabel;
    lbl33: TLabel;
    lbl35: TLabel;
    rgrpF27: TRzRadioGroup;
    rgrpF27l: TRzRadioGroup;
    rgrpF28: TRzRadioGroup;
    rgrpF28l: TRzRadioGroup;
    rgrpF29: TRzRadioGroup;
    rgrpF29l: TRzRadioGroup;
    lblF30: TLabel;
    edtF30: TEdit;
    spnF30: TRzSpinButtons;
    spnF31: TRzSpinButtons;
    edtF31: TEdit;
    lblF31: TLabel;
    rgrpF32: TRzRadioGroup;
    rgrpF33: TRzRadioGroup;
    rgrpF34: TRzRadioGroup;
    rgrpF35: TRzRadioGroup;
    rgrpF36: TRzRadioGroup;
    rgrpF37: TRzRadioGroup;
    rgrpF38: TRzRadioGroup;
    spnD15: TRzSpinButtons;
    Panel1: TPanel;
    mnuOptions: TMenuItem;
    edtF5m: TEdit;
    mnuHighlightColor: TMenuItem;
    EmployNextPage: TStExpressionEdit;
    drugNextPage: TStExpressionEdit;
    LegalNextpage: TStExpressionEdit;
    FamilyNextPage: TStExpressionEdit;
    SocialNextPage: TStExpressionEdit;
    mnuClinicalUse: TMenuItem;
    spltPsych: TRzSplitter;
    sbPsych: TScrollBox;
    lblP1: TLabel;
    lblP1a: TLabel;
    lblP1b: TLabel;
    lbl42: TLabel;
    lbl41: TLabel;
    lbl40: TLabel;
    lblP11: TLabel;
    lblEnd: TLabel;
    edtP1a: TEdit;
    edtP1b: TEdit;
    spnP1b: TRzSpinButtons;
    spnP1a: TRzSpinButtons;
    rgrpP2: TRzRadioGroup;
    rgrpP3: TRzRadioGroup;
    rgrpP3l: TRzRadioGroup;
    rgrpP4: TRzRadioGroup;
    rgrpP4l: TRzRadioGroup;
    rgrpP5: TRzRadioGroup;
    rgrpP5l: TRzRadioGroup;
    rgrpP6: TRzRadioGroup;
    rgrpP6l: TRzRadioGroup;
    rgrpP7: TRzRadioGroup;
    rgrpP8: TRzRadioGroup;
    rgrpP9: TRzRadioGroup;
    rgrpP10: TRzRadioGroup;
    edtP11: TEdit;
    spnP11: TRzSpinButtons;
    rgrpP12: TRzRadioGroup;
    rgrpP13: TRzRadioGroup;
    rgrpP14: TRzRadioGroup;
    rgrpP15: TRzRadioGroup;
    rgrpP16: TRzRadioGroup;
    rgrpP17: TRzRadioGroup;
    rgrpP18: TRzRadioGroup;
    rgrpP19: TRzRadioGroup;
    rgrpP20: TRzRadioGroup;
    rgrpP21: TRzRadioGroup;
    rgrpP22: TRzRadioGroup;
    rgrpP7l: TRzRadioGroup;
    rgrpP8l: TRzRadioGroup;
    rgrpP9l: TRzRadioGroup;
    rgrpP10l: TRzRadioGroup;
    pnlPsych: TPanel;
    spltMedical: TRzSplitter;
    pnlMedical: TPanel;
    sbMedical: TScrollBox;
    lblM1: TLabel;
    lblM2m: TLabel;
    lblM2: TLabel;
    lblM6: TLabel;
    edtM1: TEdit;
    SpnM1: TRzSpinButtons;
    SpnM2m: TRzSpinButtons;
    edtM2m: TEdit;
    SpnM2y: TRzSpinButtons;
    edtM2y: TEdit;
    rgrpM3: TRzRadioGroup;
    edtM3e: TRzEdit;
    rgrpM4: TRzRadioGroup;
    rgrpM5: TRzRadioGroup;
    edtM6: TEdit;
    SpnM6: TRzSpinButtons;
    rgrpM7: TRzRadioGroup;
    rgrpM8: TRzRadioGroup;
    rgrpM9: TRzRadioGroup;
    rgrpM10: TRzRadioGroup;
    rgrpM11: TRzRadioGroup;
    edtM5e: TRzEdit;
    MedicalNextPage: TStExpressionEdit;
    mnuSpeedTab: TMenuItem;
    pnlVerify: TPanel;
    pnlHint: TPanel;
    lblHint: TLabel;
    lblVerify: TLabel;
    lblEnd1: TLabel;
    memLeisure: TFMMemo;
    memSpirit: TFMMemo;
    memPsych: TFMMemo;
    memSocial: TFMMemo;
    memFamily: TFMMemo;
    memLegal: TFMMemo;
    memDrug: TFMMemo;
    memEmploy: TFMMemo;
    memMedical: TFMMemo;
    mnuShowHints: TMenuItem;
    mnuDefaultSize: TMenuItem;
    spltGeneral: TRzSplitter;
    sbGeneral: TScrollBox;
    lblG3: TLabel;
    lblG4: TLabel;
    lblG5: TLabel;
    lblG11: TLabel;
    lblG14: TLabel;
    lblG14m: TLabel;
    lblG20: TLabel;
    dtpG4: TDateTimePicker;
    dtpG5: TDateTimePicker;
    rgrpG8: TRzRadioGroup;
    rgrpG9: TRzRadioGroup;
    rgrpG12: TRzRadioGroup;
    edtG14y: TEdit;
    SpnG14y: TRzSpinButtons;
    edtG14m: TEdit;
    SpnG14m: TRzSpinButtons;
    rgrpG15: TRzRadioGroup;
    rgrpG17: TRzRadioGroup;
    rgrpG18: TRzRadioGroup;
    rgrpG19: TRzRadioGroup;
    edtG20: TEdit;
    SpnG20: TRzSpinButtons;
    edtG19e: TRzEdit;
    GeneralNextPage: TStExpressionEdit;
    cbxG11: TORComboBox;
    cbxG3: TORComboBox;
    memGeneral: TFMMemo;
    btnFinished: TBitBtn;
    ASI_FMGets1: TFMGets;
    ASI_FMFiler1: TFMFiler;
    mnuASIPopUp: TPopupMenu;
    mnuClinicalPU: TMenuItem;
    mnuIndexPU: TMenuItem;

    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure aHighlightOn(Sender: TObject);
    procedure rgrpG19Click(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure rgrpG19Exit(Sender: TObject);
    procedure rgrpM5Click(Sender: TObject);
    procedure rgrpM3Click(Sender: TObject);
    procedure rgrpE3Click(Sender: TObject);
    procedure mnuCloseClick(Sender: TObject);
    procedure edtG14yExit(Sender: TObject);
    procedure edtG20Exit(Sender: TObject);
    procedure edtM1Exit(Sender: TObject);
    procedure edtM2yExit(Sender: TObject);
    procedure edtM2mExit(Sender: TObject);
    procedure edtE1yExit(Sender: TObject);
    procedure edtE6yExit(Sender: TObject);
    procedure edtE12Exit(Sender: TObject);
    procedure edtE15Exit(Sender: TObject);
    procedure btnHollingsheadClick(Sender: TObject);
    procedure edtD2dExit(Sender: TObject);
    procedure edtD2yExit(Sender: TObject);
    procedure edtD16Exit(Sender: TObject);
    procedure edtD17Exit(Sender: TObject);
    procedure edtD21Exit(Sender: TObject);
    procedure edtD22Exit(Sender: TObject);
    procedure edtD23Exit(Sender: TObject);
    procedure edtD24Exit(Sender: TObject);
    procedure edtL17Exit(Sender: TObject);
    procedure edtL20Exit(Sender: TObject);
    procedure edtL22Exit(Sender: TObject);
    procedure edtL26Exit(Sender: TObject);
    procedure edtL27Exit(Sender: TObject);
    procedure edtF2yExit(Sender: TObject);
    procedure edtF5yExit(Sender: TObject);
    procedure edtP11Exit(Sender: TObject);
    procedure rgrpG8Click(Sender: TObject);
    procedure rgrpG12Click(Sender: TObject);
    procedure rgrpE4Click(Sender: TObject);
    procedure rgrpD14Click(Sender: TObject);
    procedure rgrpL24Click(Sender: TObject);
    procedure rgrpE8Click(Sender: TObject);
    procedure pgctrlASIChange(Sender: TObject);
    procedure sbSocialEnter(Sender: TObject);
    procedure edtD15Change(Sender: TObject);
    procedure edtD20Change(Sender: TObject);
    procedure edtD19Change(Sender: TObject);
    procedure edtD27Change(Sender: TObject);
    procedure edtD26Change(Sender: TObject);
    procedure edtL21Change(Sender: TObject);
    procedure MemLeisureExit(Sender: TObject);
    procedure MemSpiritExit(Sender: TObject);
    procedure MemPsychExit(Sender: TObject);
    procedure MemSocialExit(Sender: TObject);
    procedure MemFamilyExit(Sender: TObject);
    procedure MemLegalExit(Sender: TObject);
    procedure MemDrugExit(Sender: TObject);
    procedure MemEmployExit(Sender: TObject);
    procedure MemMedicalExit(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure edtF11Change(Sender: TObject);
    procedure mnuHighlightColorClick(Sender: TObject);
    procedure pnlEndClick(Sender: TObject);
    procedure GeneralNextPageEnter(Sender: TObject);
    procedure MedicalNextPageEnter(Sender: TObject);
    procedure EmployNextPageEnter(Sender: TObject);
    procedure drugNextPageEnter(Sender: TObject);
    procedure LegalNextpageEnter(Sender: TObject);
    procedure FamilyNextPageEnter(Sender: TObject);
    procedure SocialNextPageEnter(Sender: TObject);
    procedure spltDrugExit(Sender: TObject);
    procedure spltSocialExit(Sender: TObject);
    procedure spltPsychExit(Sender: TObject);
    procedure splitMedicalExit(Sender: TObject);
    procedure spltLegalExit(Sender: TObject);
    procedure spltFamilyExit(Sender: TObject);
    procedure spltEmployExit(Sender: TObject);
    procedure mnuClinicalUseClick(Sender: TObject);
    procedure mnuAboutClick(Sender: TObject);
    procedure mnuIndexClick(Sender: TObject);
    procedure cbxG11NeedData(Sender: TObject; const StartFrom: string;
      Direction, InsertAt: Integer);
    procedure cbxG3Exit(Sender: TObject);
    procedure mnuSpeedTabClick(Sender: TObject);
    procedure edtM1Change(Sender: TObject);
    procedure dtpG4Exit(Sender: TObject);
    procedure dtpG5Exit(Sender: TObject);
    procedure rgrpM7Exit(Sender: TObject);
    procedure rgrpD30Exit(Sender: TObject);
    procedure rgrpD31Exit(Sender: TObject);
    procedure rgrpL24Exit(Sender: TObject);
    procedure rgrpL25Exit(Sender: TObject);
    procedure rgrpF34Exit(Sender: TObject);
    procedure rgrpF35Exit(Sender: TObject);
    procedure rgrpP10lExit(Sender: TObject);
    procedure rgrpP9lExit(Sender: TObject);
    procedure rgrpP2Exit(Sender: TObject);
    procedure rgrpP9Exit(Sender: TObject);
    procedure rgrpP13Exit(Sender: TObject);
    procedure rgrpM8Exit(Sender: TObject);
    procedure rgrpE20Exit(Sender: TObject);
    procedure rgrpD14Exit(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure mnuShowHintsClick(Sender: TObject);
    procedure mnuDefaultSizeClick(Sender: TObject);
    procedure spltGeneralExit(Sender: TObject);
    procedure memGeneralExit(Sender: TObject);
    procedure btnFinishedClick(Sender: TObject);
    procedure lblVerifyClick(Sender: TObject);
    //procedure ARadioMouseClick(sender: TObject);
    procedure Check30(Sender: TObject);
    procedure CheckLife(Sender: TObject);
    procedure Check9(Sender: TObject);
    procedure Check12(Sender: TObject);
    procedure Check99(Sender: TObject);
    procedure Check9999(Sender: TObject);
    procedure CheckOutL17(Sender: TObject);
    procedure SpinEditKeyPress(Sender: TObject; var Key: Char);
    procedure SpinUp(Sender: TObject);
    procedure SpinDown(Sender: TObject);
    function CheckDeathDate(DateIn: TDateTime): Boolean;
    procedure FormActivate(Sender: TObject);
    procedure HighlightRadioGroup(Item: TRzRadioGroup);
    procedure rgrpG8Enter(Sender: TObject);
    procedure rgrpG9Enter(Sender: TObject);
    procedure rgrpG12Enter(Sender: TObject);
    procedure rgrpG15Enter(Sender: TObject);
    procedure rgrpG17Enter(Sender: TObject);
    procedure rgrpG18Enter(Sender: TObject);
    procedure rgrpG19Enter(Sender: TObject);
    procedure rgrpM3Enter(Sender: TObject);
    procedure rgrpM4Enter(Sender: TObject);
    procedure rgrpM5Enter(Sender: TObject);
    procedure rgrpM7Enter(Sender: TObject);
    procedure rgrpM8Enter(Sender: TObject);
    procedure rgrpM9Enter(Sender: TObject);
    procedure rgrpM10Enter(Sender: TObject);
    procedure rgrpM11Enter(Sender: TObject);
    procedure rgrpE3Enter(Sender: TObject);
    procedure rgrpE4Enter(Sender: TObject);
    procedure rgrpE5Enter(Sender: TObject);
    procedure rgrpE7Enter(Sender: TObject);
    procedure rgrpE8Enter(Sender: TObject);
    procedure rgrpE9Enter(Sender: TObject);
    procedure rgrpE10Enter(Sender: TObject);
    procedure rgrpE20Enter(Sender: TObject);
    procedure rgrpE21Enter(Sender: TObject);
    procedure rgrpE22Enter(Sender: TObject);
    procedure rgrpE23Enter(Sender: TObject);
    procedure rgrpE24Enter(Sender: TObject);
    procedure rgrpD14Enter(Sender: TObject);
    procedure rgrpD28Enter(Sender: TObject);
    procedure rgrpD29Enter(Sender: TObject);
    procedure rgrpD30Enter(Sender: TObject);
    procedure rgrpD31Enter(Sender: TObject);
    procedure rgrpD32Enter(Sender: TObject);
    procedure rgrpD33Enter(Sender: TObject);
    procedure rgrpD34Enter(Sender: TObject);
    procedure rgrpD35Enter(Sender: TObject);
    procedure rgrpL1Enter(Sender: TObject);
    procedure rgrpL2Enter(Sender: TObject);
    procedure rgrpL23Enter(Sender: TObject);
    procedure rgrpL24Enter(Sender: TObject);
    procedure rgrpL25Enter(Sender: TObject);
    procedure rgrpL28Enter(Sender: TObject);
    procedure rgrpL29Enter(Sender: TObject);
    procedure rgrpL30Enter(Sender: TObject);
    procedure rgrpL31Enter(Sender: TObject);
    procedure rgrpL32Enter(Sender: TObject);
    procedure rgrpF1Enter(Sender: TObject);
    procedure rgrpF3Enter(Sender: TObject);
    procedure rgrpF4Enter(Sender: TObject);
    procedure rgrpF6Enter(Sender: TObject);
    procedure rgrpF7Enter(Sender: TObject);
    procedure rgrpF8Enter(Sender: TObject);
    procedure rgrpF9Enter(Sender: TObject);
    procedure rgrpF10Enter(Sender: TObject);
    procedure rgrpF19Enter(Sender: TObject);
    procedure rgrpF20Enter(Sender: TObject);
    procedure rgrpF21Enter(Sender: TObject);
    procedure rgrpF18lEnter(Sender: TObject);
    procedure rgrpF19lEnter(Sender: TObject);
    procedure rgrpF20lEnter(Sender: TObject);
    procedure rgrpF21lEnter(Sender: TObject);
    procedure rgrpF22Enter(Sender: TObject);
    procedure rgrpF22lEnter(Sender: TObject);
    procedure rgrpF23Enter(Sender: TObject);
    procedure rgrpF23lEnter(Sender: TObject);
    procedure rgrpF24Enter(Sender: TObject);
    procedure rgrpF24lEnter(Sender: TObject);
    procedure rgrpF25Enter(Sender: TObject);
    procedure rgrpF25lEnter(Sender: TObject);
    procedure rgrpF26Enter(Sender: TObject);
    procedure rgrpF26lEnter(Sender: TObject);
    procedure rgrpF27Enter(Sender: TObject);
    procedure rgrpF27lEnter(Sender: TObject);
    procedure rgrpF28Enter(Sender: TObject);
    procedure rgrpF28lEnter(Sender: TObject);
    procedure rgrpF29Enter(Sender: TObject);
    procedure rgrpF29lEnter(Sender: TObject);
    procedure rgrpF32Enter(Sender: TObject);
    procedure rgrpF33Enter(Sender: TObject);
    procedure rgrpF34Enter(Sender: TObject);
    procedure rgrpF35Enter(Sender: TObject);
    procedure rgrpF36Enter(Sender: TObject);
    procedure rgrpF37Enter(Sender: TObject);
    procedure rgrpF38Enter(Sender: TObject);
    procedure rgrpP2Enter(Sender: TObject);
    procedure rgrpP3Enter(Sender: TObject);
    procedure rgrpP3lEnter(Sender: TObject);
    procedure rgrpP4Enter(Sender: TObject);
    procedure rgrpP4lEnter(Sender: TObject);
    procedure rgrpP5Enter(Sender: TObject);
    procedure rgrpP5lEnter(Sender: TObject);
    procedure rgrpP6Enter(Sender: TObject);
    procedure rgrpP6lEnter(Sender: TObject);
    procedure rgrpP7Enter(Sender: TObject);
    procedure rgrpP7lEnter(Sender: TObject);
    procedure rgrpP8Enter(Sender: TObject);
    procedure rgrpP8lEnter(Sender: TObject);
    procedure rgrpP9Enter(Sender: TObject);
    procedure rgrpP9lEnter(Sender: TObject);
    procedure rgrpP10Enter(Sender: TObject);
    procedure rgrpP10lEnter(Sender: TObject);
    procedure rgrpP12Enter(Sender: TObject);
    procedure rgrpP13Enter(Sender: TObject);
    procedure rgrpP14Enter(Sender: TObject);
    procedure rgrpP15Enter(Sender: TObject);
    procedure rgrpP16Enter(Sender: TObject);
    procedure rgrpP17Enter(Sender: TObject);
    procedure rgrpP18Enter(Sender: TObject);
    procedure rgrpP19Enter(Sender: TObject);
    procedure rgrpP20Enter(Sender: TObject);
    procedure rgrpP21Enter(Sender: TObject);
    procedure rgrpP22Enter(Sender: TObject);
    procedure rgrpF18Enter(Sender: TObject);

  private
    { Private declarations }
    function CheckRadioKey(K: Word): Boolean;
    procedure RecoverComments;
    function VerifyData(sender: TObject): Boolean;
    procedure SpinEditField(UpDown: Char; Sender: TObject);
    function CheckRange(StringIn: string; Max, Min: integer): Boolean;
    procedure RecoverCrashData;
    function SkippedItems: string;
    procedure PageScroll(Scroll: integer; Sender: TObject);
    procedure GetLastASI;
    function SaveSign: boolean;
    procedure CrossCheck(WhichOne: string; sender: TObject);
    function ConductCrossChecks(sender: TObject): Boolean;
    procedure CleanupTxtFiles;
    procedure CleanupComments;
    function SignAndExit: boolean;
    procedure SetupMainForm;
    procedure FixinToClose(Sender: TObject);
  public
    { Public declarations }
    OrderedByIEN,
      ProgramIEN, // blank for new records and found in frmMain for old records
      ResponseList: string;
    PartialList: TStringList;
    FASITimedOut,
      FUserTerminates,
      Initialize,
      NoFollowupAllowed: Boolean;
    LastASIDate: string[20];
    procedure ShowHint(Sender: TObject);
    procedure ASITimedOut;
  end;

var
  frmASI: TfrmASI;

implementation

{$R *.DFM}

uses
  fMHA_Main,
  uMHA_Registry, // Windows registry read\write functions
  uMHA_ASI_Build, // Display the ASI form
  uMHA_ASI_Hints, // Hints established for each item
  uMHA_ASI_Save, // Vista save operations
  uMHA_ASI_Vista, // Broker call operations
  fMHA_ASI_Options, // Speed tab, highlight colors, etc.
  fMHA_ASI_Close, // Form for user to select sign, save, ignore, etc.
  uMHA_MsgDlg, // Message dialog box with variable header
  fMHA_ASI_Sign, // For electronic signature
  fMHA_About, // Help Menu option
  ORNet, // for CPRS M connectivity stuff
  rCore, // for CPRS functions
  uCore, // for CPRS core objects: patient, user, etc.
  MFUNSTR, // For MUMPS functions. Piece, etc.
  Mumps, // Find procedure is used
  ORfn, // for "mixedcase"
  uMHA_DM; // For data module

var
  CrossCheckList: string;
  LastRadio: string[20];
  Fresh,
    CheckSkipItems,
    CrossCheckItems,
    CrossChecksCompleted: boolean;


//-------------------------------------------------------------------
//    Create
//-------------------------------------------------------------------


procedure TfrmASI.FormCreate(Sender: TObject);
begin
  ASI_Vista := TASI_Vista.Create; // uMHA_ASI_Vista
  ASI_Save  := TASI_Save.Create; // uMHA_ASI_Save
  ASI_Build := TASI_Build.Create; // uMHA_ASI_Build
  ASI_Hints := TASI_Hints.Create; // uMHA_ASI_Hints
  PartialList := TStringList.Create;

  FUserTerminates := False;
  FASITimedOut    := false;
  // Cosmetic: blank screen shown as form is created on slower PCs
  pgctrlASI.ActivePage := TabLeisure;

end;

procedure TfrmASI.FormActivate(Sender: TObject);
begin
  // Crash recovery file exists only when a non-standard exit occurs from the program.
  // While not showing the form, populate the items with data from ASI_Recovery.txt
  // then store on the dark-side.
  if frmMain.ASI_Crash then
  begin
    ASI_Build.SetupComboBoxItems; // uMHA_ASI_Build
    ASI_Build.DefaultValuesForItems; // uMHA_ASI_Build
    RecoverCrashData;
    
    Close;
  end;

  // read windows registry
  MHA_Registry.ReadRegistry; // uMHA_Registry

  //set this equal to option set on Main form
  mnuDefaultSize.Checked := frmMain.mnuDefaultFormSizes.Checked;
  mnuShowHints.Checked := frmMain.mnuShowHints.Checked;
  mnuSpeedTab.Checked := MHA_Registry.UseSpeedTab;

  // Add these items to the list that the user makes active.
  // This is a list of items that the user has answered during the
  // course of the ASI. This avoids saving default items (user did not
  // actually make a response) on the M-side.
  ResponseList := 'DTPG5 RGRPG8 RGRPG12 ';
  // first use of program: used for item G8, intake type
  Fresh := true;
  // Save the items without verification when user terminates before completion.
  // Also do not save items that the user has not entered (no defaults allowed).
  FUserTerminates := false;

  CheckSkipItems := False; // toggle for clearing of required items
  CrossCheckItems := false; // toggle for clearing of cross-cehck items
  CrossChecksCompleted := false; // toggle when cross check done
  ProgramIEN := '';
  CrossCheckList := 'G20,M1,M2,M7,M8,E1y,E12a,E12b,E15,E20,D2d,D2y,D12d,' +
    'D14,D16,D17,D21a,D21b,D22a,D22b,D23,D24a,D24b,D30a,D30b,D31a,D31b,L17,L20,L24,' +
    'L25,L26,L27,F34a,F34b,F35a,F35b,P2,P9a,P9b,P10,P11a,P11b,P13a,P13b,';

    // Avoid doing a follow-up ASI when there is no previous lite or full. This
    // occurs when frmMain.grdOldASI is "None"
  if frmMain.grdOldASI.Cells[0, 1] = 'None' then
    NoFollowupAllowed := true
  else
  begin
    NoFollowupAllowed := false;
    LastASIDate := frmMain.grdOldASI.Cells[0, 1];
  end;

  Initialize := True;
  // Setup combo box strings, reset scroll bars, place form on screen, etc.
  ASI_Build.BuildBasics; // uMHA_ASI_Build
  ASI_Build.BuildForm(frmMain.Partial_ASI); // uMHA_ASI_Build
  ASI_Hints.BuildHints; // uMHA_ASI_Hints
  CleanupComments;  // Removes M carriage returns & line feeds
  Initialize := False;
  ActiveControl := cbxG3;
   // ShowModal: call this from the main form, otherwise the SaveData
   // does not work when the form is timed out by Vista.
end;


//-------------------------------------------------------------------
//    Close, Clean-up, Destroy
//-------------------------------------------------------------------


procedure TfrmASI.FixinToClose(Sender: TObject);
var
  UserResponse: Char;
begin
  UserResponse := ' '; // initialize

  if UserSession.CPRSPatientChanged then
    Exit;

  // Show the Close Options
  try
    frmASI_Close := TfrmASI_Close.Create(self);
    UserResponse := frmASI_Close.CloseResponse;
    frmASI_Close.free;
  except
    on E: Exception do
    begin
      MsgDlg(E.Message, 'ASI Close',
        mtError, [mbOk], 0);
      Exit;
    end;
  end; {-try -}

  lblHint.Caption := '';

  if UserResponse = 'E' then // Exit: Close now
    Close;

  if UserResponse = 'C' then // Return to ASI: do not close
    Exit;

  if UserResponse = 'F' then // Finish later, don't sign
  // This is when the user decides to close the ASI, not the
  // vet. The items are not verified, it is not signed.
  begin
      // don't save ASI on dead vet
    if not CheckDeathDate(dtpG4.DateTime) then
      Exit;
    if not CheckDeathDate(dtpG5.DateTime) then
      Exit;
    FUserTerminates := true;

    ASI_Save.SaveASIData; // uMHA_ASI_Save
    lblHint.Caption := 'Data saved. Closing...';
    //Go back to ASI tab on main form, reload ASI data
    SetupMainForm;
    if ASI_Vista.ResponsesSaved then
      Close;
  end;

  if UserResponse = 'S' then
  // if G12 is answered "N" then verify, save, & sign,
  // otherwise, simply save and sign.
  begin
      // don't save ASI on dead vet
    if not CheckDeathDate(dtpG4.DateTime) then
      Exit;
    if not CheckDeathDate(dtpG5.DateTime) then
      Exit;

    if rgrpG12.ItemIndex = 3 then // "N" meaning "Completed"
    begin
      // Any blank items?
      if not VerifyData(sender) then
      begin
        lblHint.Caption := 'Verify not complete';
        exit;
      end;
      lblHint.Caption := 'Data verified...';
      // Cross checks
      if not CrossChecksCompleted then
      begin
        CrossCheckItems := true;
        lblHint.Caption := 'Data verified, conducting cross checks...';
        if not ConductCrossChecks(sender) then
        begin
          CrossChecksCompleted := false;
          exit;
        end;
      end;
      lblHint.Caption := 'Data verified, cross checks done...';
      // Save, get signature
      if not SignAndExit then
        Exit;
      // Update the main form
      SetupMainForm;
      Close;
    end
    else // G12 is not "N"
    begin
      // Save and sign the ASI. No checks conducted.
      if not SignAndExit then
        exit;
      // Update the main form
      SetupMainForm;
      Close;
    end;
  end;
end;

procedure TfrmASI.FormClose(Sender: TObject; var Action: TCloseAction);
begin
// This is case where form is not displayed. Data saved to ASI_Recovery_RPC Server.txt is
// transfered to this form and saved to dark-side.
  if (frmMain.ASI_Crash) then
  begin
    FUserTerminates := False;
    ASI_Save.SaveASIData; // uMHA_ASI_Save
    //close;
    exit;
  end;

  PartialList.Free;
  // Use main form dimensions & location to ensure overlap
  MHA_Registry.MainTop := Top;
  MHA_Registry.MainLeft := Left;
  MHA_Registry.MainHeight := Height;
  MHA_Registry.MainWidth := Width;
  if mnuSpeedTab.Checked then
    MHA_Registry.UseSpeedTab := true
  else
    MHA_Registry.UseSpeedTab := false;

  MHA_Registry.SaveRegistry;

  CleanupTxtFiles;

  // Pass on any options changes to main form
  frmMain.mnuShowHints.Checked := mnuShowHints.Checked;
  frmMain.mnuDefaultFormSizes.Checked := mnuDefaultSize.Checked;
end;

procedure TfrmASI.FormDestroy(Sender: TObject);
begin
  ASI_Vista.Free;
  ASI_Hints.Free;
  ASI_Save.Free;
  ASI_Build.Free;
end;

procedure TfrmASI.CleanupTxtFiles;
begin
  // remove the MHA_Recover.txt file; crash protection not needed now with normal
  // program termination.
  if FileExists(frmMain.ASI_RecoveryFile) then
    DeleteFile(frmMain.ASI_RecoveryFile);
  if FileExists(UserSession.TempFolderName + 'ASI_General_' + RPCBrokerV.Server + '.txt') then
    DeleteFile(UserSession.TempFolderName + 'ASI_General_' + RPCBrokerV.Server + '.txt');
  if FileExists(UserSession.TempFolderName + 'ASI_Medical_' + RPCBrokerV.Server + '.txt') then
    DeleteFile(UserSession.TempFolderName + 'ASI_Medical_' + RPCBrokerV.Server + '.txt');
  if FileExists(UserSession.TempFolderName + 'ASI_Employ_' + RPCBrokerV.Server + '.txt') then
    DeleteFile(UserSession.TempFolderName + 'ASI_Employ_' + RPCBrokerV.Server + '.txt');
  if FileExists(UserSession.TempFolderName + 'ASI_Drug_' + RPCBrokerV.Server + '.txt') then
    DeleteFile(UserSession.TempFolderName + 'ASI_Drug_' + RPCBrokerV.Server + '.txt');
  if FileExists(UserSession.TempFolderName + 'ASI_Legal_' + RPCBrokerV.Server + '.txt') then
    DeleteFile(UserSession.TempFolderName + 'ASI_Legal_' + RPCBrokerV.Server + '.txt');
  if FileExists(UserSession.TempFolderName + 'ASI_Family_' + RPCBrokerV.Server + '.txt') then
    DeleteFile(UserSession.TempFolderName + 'ASI_Family_' + RPCBrokerV.Server + '.txt');
  if FileExists(UserSession.TempFolderName + 'ASI_Social_' + RPCBrokerV.Server + '.txt') then
    DeleteFile(UserSession.TempFolderName + 'ASI_Social_' + RPCBrokerV.Server + '.txt');
  if FileExists(UserSession.TempFolderName + 'ASI_Psych_' + RPCBrokerV.Server + '.txt') then
    DeleteFile(UserSession.TempFolderName + 'ASI_Psych_' + RPCBrokerV.Server + '.txt');
  if FileExists(UserSession.TempFolderName + 'ASI_Spirit_' + RPCBrokerV.Server + '.txt') then
    DeleteFile(UserSession.TempFolderName + 'ASI_Spirit_' + RPCBrokerV.Server + '.txt');
  if FileExists(UserSession.TempFolderName + 'ASI_Leisure_' + RPCBrokerV.Server + '.txt') then
    DeleteFile(UserSession.TempFolderName + 'ASI_Leisure_' + RPCBrokerV.Server + '.txt');
end;


//-------------------------------------------------------------------
//        Buttons and panels
//-------------------------------------------------------------------


procedure TfrmASI.btnHollingsheadClick(Sender: TObject);
begin
  Application.HelpContext(1);
end;

procedure TfrmASI.btnFinishedClick(Sender: TObject);
begin
  FixinToClose(sender);
end;

procedure TfrmASI.pnlEndClick(Sender: TObject);
begin
  FixinToClose(sender);
end;

procedure TfrmASI.lblVerifyClick(Sender: TObject);
begin
  initialize := false;
  if CrossCheckItems then
    ConductCrossChecks(sender);
end;


//--------------------------------------------------------------------
//      Synchronize with Main Form
//--------------------------------------------------------------------


procedure TfrmASI.SetupMainForm;
begin
  //Go back to ASI tab on main form, reload ASI data
  with frmMain do
  begin
    FFirstASITab := True;
    pgctrlMain.activepage := tbshtASI;
    grdOldASI.Row := 1;
    pgctrlMainChange(Self);
  end;
end;

procedure TfrmASI.GetLastASI;
var
  i: integer;
  Done: boolean;
begin
  // This procedure will check that the last ASI is from the date the
  // user entered in item G5. The last ASI is frmMain.grdOldASI.Cells[0, 1]
  // but it may be the wrong date in rare circumstances.
  i := 1;
  Done := false;
  repeat
    LastASIDate := frmMain.grdOldASI.Cells[0, i];
    if LastASIDate = 'None' then exit;
    if (dtpG5.Date {Interview date} - StrToDate(LastASIDate)) < 0 then
    begin
        // try the next ASI
      Inc(i);
    end
    else
      Done := true;
  until ((i = frmMain.grdOldASI.RowCount) or (Done));
end;


//-------------------------------------------------------------------
//           Crash Recovery
//-------------------------------------------------------------------


// In order to recover from an unexpected crash (power goes off, user
// turns off PC without exiting program, etc.) the user's responses
// are placed in a overwritten text file. This list is deleted upon a normal
// close of the program. The existence of the text file when the program is
// launched indicates that a crash occurred during the last use.

procedure TfrmASI.RecoverCrashData;
// To recover the data, first the text file is read to the ASI form (full, lite
// or followup) then the form is saved in the normal way. The Form is not
// displayed during this process.
var
  i: integer;
  TempStr: string;
  f: TextFile;


  function GetNextItem: string;
  begin
    TempStr := '';
    // Item click are set off by this, reset Initialize a must
    Initialize := true;
    ReadLn(f, TempStr);
    Result := TempStr;
  end;

begin
  TempStr := '';
  Screen.Cursor := crHourglass;
  try
    // Crash-File setup
    AssignFile(f, frmMain.ASI_RecoveryFile);
    Reset(f);

    // First line stores the type of ASI in use, RecordIENS, Patient.DFN, etc.
    ReadLn(f, TempStr);
    frmMain.Partial_ASI := 'X'; // Default values are not placed in items initially.
    ASI_Vista.RecordIEN := Piece(TempStr, '^', 2); // do not put '`' here
    Patient.DFN := Piece(TempStr, '^', 3); // do not put '`' here
    OrderedByIEN := Piece(TempStr, '^', 4); // do not put '`' here
    ProgramIEN := Piece(TempStr, '^', 5); // do not put '`' here
    if Piece(TempStr, '^', 1) = 'Full' then
      ASI_Build.SetupASIFull;
    if Piece(TempStr, '^', 1) = 'Lite' then
      ASI_Build.SetupASILite;
    if Piece(TempStr, '^', 1) = 'Followup' then
      ASI_Build.SetupASIFollowup;
  except
    on E: Exception do
    begin
      showmessage(e.message);
    end;
  end;
  try
  // Run through all the components, place the value from the
  // crash file in the component.
    for i := 0 to ComponentCount - 1 do
    begin
      if Components[i] is TEdit then
        TEdit(Components[i]).Text := GetNextItem;
      if Components[i] is TRzEdit then
        TRzEdit(Components[i]).Text := GetNextItem;
      if Components[i] is TDateTimePicker then
        TDateTimePicker(Components[i]).Date := StrToDate(GetNextItem);
      if Components[i] is TRzRadioGroup then
        TRzRadioGroup(Components[i]).ItemIndex := StrToInt(GetNextItem);
      if Components[i] is TRzComboBox then
        TRzComboBox(Components[i]).Text := GetNextItem;
      // The combo box for interviewer G11 and program G3 are saved as IENs
    end;
  except
    on E: Exception do
    begin
      showmessage(e.message);
    end;
  end;

  CloseFile(f);
  RecoverComments;
end;


//-------------------------------------------------------------------
//           Menu Operations
//-------------------------------------------------------------------


procedure TfrmASI.mnuClinicalUseClick(Sender: TObject);
begin
  Application.HelpContext(10);
end;

procedure TfrmASI.mnuAboutClick(Sender: TObject);
begin
  frmMHA_About := TfrmMHA_About.Create(Self);
  frmMHA_About.ShowModal;
  frmMHA_About.Free;
end;

procedure TfrmASI.mnuIndexClick(Sender: TObject);
begin
  Application.HelpCommand(Help_PartialKey, 0);
end;

procedure TfrmASI.mnuShowHintsClick(Sender: TObject);
begin
  mnuShowHints.Checked := not mnuShowHints.Checked;
  Application.ShowHint := mnuShowHints.Checked;
end;

procedure TfrmASI.mnuDefaultSizeClick(Sender: TObject);
begin
  mnuDefaultSize.Checked := not mnuDefaultSize.Checked;
  ASI_Build.AdjustForm;
end;

procedure TfrmASI.mnuHighlightColorClick(Sender: TObject);
begin
  try
    frmASI_Options := TfrmASI_Options.Create(self);
    frmASI_Options.free;
  except
    on E: Exception do
    begin
      MsgDlg(E.Message, 'ASI Options',
        mtError, [mbOk], 0);
        //Left, Top, Height, Width, 0);
         {- Do something? -}
    end;
  end; {-try -}
  MHA_Registry.ReadRegistry;
end;

procedure TfrmASI.mnuSpeedTabClick(Sender: TObject);
begin
  if mnuSpeedTab.Checked then
  begin
    MHA_Registry.UseSpeedTab := false;
    mnuSpeedTab.Checked := false;
  end
  else
  begin
    MHA_Registry.UseSpeedTab := true;
    mnuSpeedTab.Checked := true;
  end;
  MHA_Registry.SaveRegistry;
end;

procedure TfrmASI.mnuCloseClick(Sender: TObject);
begin
  FixinToClose(sender);
end;

procedure TfrmASI.ASITimedOut;
begin
  FASITimedOut := true;
  ASI_Save.SaveASIData; // uMHA_ASI_Save
  Close;
end;


//-------------------------------------------------------------------
//       Key operations: Key Press, Down
//-------------------------------------------------------------------


procedure TfrmASI.SpinEditField(UpDown: Char; Sender: TObject);
var
  i: integer;
begin
  for i := 0 to ComponentCount - 1 do
  begin
    if Components[i] is TEdit then
      if TEdit(Components[i]).Name = ActiveControl.Name then
      begin
          // The range may be off but the TEdit exit function will catch it.
        if UpDown = 'U' then
          SpinUp(Sender);
        if UpDown = 'D' then
          SpinDown(Sender);
      end;
  end;
end;

function TfrmASI.CheckRadioKey(K: Word): Boolean;
// Radio group has to accept number keyboard input as well as, in some cases,
// the letters "x" and "n". In addition, some radio groups start with the
// number zero and some with 1.
var
  GotX,
    StartZero: Boolean;
  i, Start: integer;
  KeyState: TKeyboardState;
begin
  Result := False;
  Start := -1;
  // ignore these keys
  if k in [vk_Return, vk_Tab, vk_Escape, vk_NumLock, vk_Capital, vk_Shift,
    vk_Prior, vk_Next, vk_up, vk_down,
    vk_Menu, vk_Control, vk_F1, vk_F2, vk_F3, vk_F4, vk_F5,
    vk_F6, vk_F7, vk_F8, vk_F9, vk_F10, vk_F11, vk_F12] then
    Exit;
  // Input of number keys, not on keypad
  if k in [48..57 {number keys}] then
    Start := k - 48; // 0 to 9
  //Input from keypad number keys when numberlock is on
  GetKeyboardState(KeyState);
  if ((k in [96..105] {Key pad numbers}) and (KeyState[VK_NUMLOCK] = 1)) then
    Start := k - 96;
  // some items allow letter "X"
  if k in [88, 120] then // letters x and X
    Start := 100;
  // some items allow letter "N"
  if k in [78, 110] then //letters n & N
    Start := 200;
  // if Key not a number, "N" or "X", then bailout
  if Start = -1 then
  begin
    MessageBeep(MB_ICONASTERISK);
//    MsgDlg('This key is not recognized.',
//      'Addiction Severity Index',
//      mtInformation, [mbOk], Left, Top, Height, Width, 0);
    Exit;
  end;

  // Range check of number input
  StartZero := True;
  // Items that start with "1"; most start with zero
  if ((LastRadio = 'rgrpE7') or (LastRadio = 'rgrpF4') or
    (LastRadio = 'rgrpG17') or (LastRadio = 'rgrpG12') or
    (LastRadio = 'rgrpG19') or (LastRadio = 'rgrpG18') or
    (LastRadio = 'rgrpG8') or (LastRadio = 'rgrpG9') or
    (LastRadio = 'rgrpE10') or
    (LastRadio = 'rgrpF9') or (LastRadio = 'rgrpF1')) then
  begin
    if Start = 0 then // should start with "1", user picked wrong key
    begin
      MessageBeep(MB_ICONASTERISK);
//      MsgDlg('This key is out of range.',
//        'Addiction Severity Index',
//        mtInformation, [mbOk], Left, Top, Height, Width, 0);
      Exit;
    end;
    // Used below for index control
    StartZero := False;
  end;

  // Get the active component and check the key stroke
  for i := 0 to ComponentCount - 1 do
  begin
    if Components[i] is TRzRadioGroup then
      if TRzRadioGroup(Components[i]).Name = LastRadio then
      begin
        // Is the input out of range? Too small
        if ((Start < 100) and
          (TRzRadioGroup(Components[i]).Items.Count < Start)) then
        begin
          MessageBeep(MB_ICONASTERISK);
//          MsgDlg('This key is off the scale.',
//            'Addiction Severity Index',
//            mtInformation, [mbOk], Left, Top, Height, Width,  0);
          Exit;
        end;
        if Start < 100 then
        begin
          // Items where "X" or "N" has index of 2: disallow it
          if ((Start = 2) and
            ((LastRadio = 'rgrpG15') or (LastRadio = 'rgrpM3') or
            (LastRadio = 'rgrpM4') or (LastRadio = 'rgrpM5') or
            (LastRadio = 'rgrpM10') or
            (LastRadio = 'rgrpM11') or (LastRadio = 'rgrpE3') or
            (LastRadio = 'rgrpE4') or (LastRadio = 'rgrpE5') or
            (LastRadio = 'rgrpE8') or (LastRadio = 'rgrpE9') or
            (LastRadio = 'rgrpE23') or (LastRadio = 'rgrpE24') or
            (LastRadio = 'rgrpD34') or (LastRadio = 'rgrpL1') or
            (LastRadio = 'rgrpL2') or (LastRadio = 'rgrpL24') or
            (LastRadio = 'rgrpL31') or (LastRadio = 'rgrpL32') or
            (LastRadio = 'rgrpF7') or (LastRadio = 'rgrpF8') or
            (LastRadio = 'rgrpF18l') or
            (LastRadio = 'rgrpF19l') or (LastRadio = 'rgrpF20l') or
            (LastRadio = 'rgrpF21l') or (LastRadio = 'rgrpF22l') or
            (LastRadio = 'rgrpF23l') or (LastRadio = 'rgrpF24l') or
            (LastRadio = 'rgrpF25l') or (LastRadio = 'rgrpF26l') or
            (LastRadio = 'rgrpF27l') or (LastRadio = 'rgrpF28l') or
            (LastRadio = 'rgrpF29l') or
            (LastRadio = 'rgrpF18') or (LastRadio = 'rgrpF19') or
            (LastRadio = 'rgrpF20') or
            (LastRadio = 'rgrpF21') or (LastRadio = 'rgrpF22') or
            (LastRadio = 'rgrpF23') or (LastRadio = 'rgrpF24') or
            (LastRadio = 'rgrpF25') or (LastRadio = 'rgrpF26') or
            (LastRadio = 'rgrpF27') or (LastRadio = 'rgrpF28') or
            (LastRadio = 'rgrpF29') or (LastRadio = 'rgrpF37') or
            (LastRadio = 'rgrpF38') or (LastRadio = 'rgrpP2') or
            (LastRadio = 'rgrpP3') or (LastRadio = 'rgrpP3l') or
            (LastRadio = 'rgrpP4') or (LastRadio = 'rgrpP4l') or
            (LastRadio = 'rgrpP5') or (LastRadio = 'rgrpP5l') or
            (LastRadio = 'rgrpP6') or (LastRadio = 'rgrpP6l') or
            (LastRadio = 'rgrpP7') or (LastRadio = 'rgrpP7l') or
            (LastRadio = 'rgrpP8') or (LastRadio = 'rgrpP8l') or
            (LastRadio = 'rgrpP9') or (LastRadio = 'rgrpP9l') or
            (LastRadio = 'rgrpP10') or (LastRadio = 'rgrpP10l') or
            (LastRadio = 'rgrpP14') or (LastRadio = 'rgrpP15') or
            (LastRadio = 'rgrpP16') or (LastRadio = 'rgrpP17') or
            (LastRadio = 'rgrpP18') or (LastRadio = 'rgrpP19') or
            (LastRadio = 'rgrpP21') or
            (LastRadio = 'rgrpP22'))) then
          begin
            MessageBeep(MB_ICONASTERISK);
//            MsgDlg('This is out of range. Try again.',
//              'Addiction Severity Index',
//              mtInformation, [mbOk], Left, Top, Height, Width, 0);
            Exit;
          end;
          // Items where "X" or "N" has index of 3: disallow it
          if ((Start = 3) and
            ((LastRadio = 'rgrpF6') or (LastRadio = 'rgrpF10') or
            (LastRadio = 'rgrpF18') or (LastRadio = 'rgrpF3') or
            (LastRadio = 'rgrpF19') or (LastRadio = 'rgrpF20') or
            (LastRadio = 'rgrpF21') or (LastRadio = 'rgrpF22') or
            (LastRadio = 'rgrpF23') or (LastRadio = 'rgrpF24') or
            (LastRadio = 'rgrpF25') or (LastRadio = 'rgrpF26') or
            (LastRadio = 'rgrpF27') or (LastRadio = 'rgrpF28') or
            (LastRadio = 'rgrpF29') or (LastRadio = 'rgrpF18l') or
            (LastRadio = 'rgrpF19l') or (LastRadio = 'rgrpF20l') or
            (LastRadio = 'rgrpF21l') or (LastRadio = 'rgrpF22l') or
            (LastRadio = 'rgrpF23l') or (LastRadio = 'rgrpF24l') or
            (LastRadio = 'rgrpF25l') or (LastRadio = 'rgrpF26l') or
            (LastRadio = 'rgrpF27l') or (LastRadio = 'rgrpF28l') or
            (LastRadio = 'rgrpF29l'))) then
          begin
            MessageBeep(MB_ICONASTERISK);
//            MsgDlg('This is out of range. Try again.',
//              'Addiction Severity Index',
//              mtInformation, [mbOk], Left, Top, Height, Width, 0);
            Exit;
          end;
          // Items where "X" or "N" has index of 4: disallow it
          if ((Start = 4) and
            ((LastRadio = 'rgrpE9') or (LastRadio = 'rgrpG12') or
            (LastRadio = 'rgrpF6') or
            (LastRadio = 'rgrpF10') or (LastRadio = 'rgrpF9'))) then
          begin
            MessageBeep(MB_ICONASTERISK);
//            MsgDlg('This is out of range. Try again.',
//              'Addiction Severity Index',
//              mtInformation, [mbOk], Left, Top, Height, Width, 0);
            Exit;
          end;
          // Items with "X" or "N" at index = 5
          if ((Start = 5) and
            ((LastRadio = 'rgrpM7') or (LastRadio = 'rgrpM8') or
            (LastRadio = 'rgrpE20') or (LastRadio = 'rgrpE21') or
            (LastRadio = 'rgrpD28') or (LastRadio = 'rgrpD29') or
            (LastRadio = 'rgrpF9') or (LastRadio = 'rgrpD30') or
            (LastRadio = 'rgrpD31') or (LastRadio = 'rgrpL28') or
            (LastRadio = 'rgrpL29') or (LastRadio = 'rgrpF32') or
            (LastRadio = 'rgrpF33') or (LastRadio = 'rgrpF34') or
            (LastRadio = 'rgrpF35') or (LastRadio = 'rgrpP12') or
            (LastRadio = 'rgrpP13'))) then
          begin
            MessageBeep(MB_ICONASTERISK);
//            MsgDlg('This is out of range. Try again.',
//              'Addiction Severity Index',
//              mtInformation, [mbOk], Left, Top, Height, Width, 0);
            Exit;
          end;
          // Items where "X" or "N" has index of 6: disallow it
          if ((Start = 6) and
            ((LastRadio = 'rgrpE20') or
            (LastRadio = 'rgrpE21') or (LastRadio = 'rgrpD28') or
            (LastRadio = 'rgrpD29') or (LastRadio = 'rgrpD30') or
            (LastRadio = 'rgrpD31') or (LastRadio = 'rgrpL28') or
            (LastRadio = 'rgrpL29') or
            (LastRadio = 'rgrpF32') or (LastRadio = 'rgrpF33') or
            (LastRadio = 'rgrpF34') or (LastRadio = 'rgrpF35'))) then
          begin
            MessageBeep(MB_ICONASTERISK);
//            MsgDlg('This is out of range. Try again.',
//              'Addiction Severity Index',
//              mtInformation, [mbOk], Left, Top, Height, Width, 0);
            Exit;
          end;
          // Items where "X" or "N" has index of 7: disallow it
          if ((Start = 7) and
            ((LastRadio = 'rgrpF1') or (LastRadio = 'rgrpG19'))) then
          begin
            MessageBeep(MB_ICONASTERISK);
//            MsgDlg('This is out of range. Try again.',
//              'Addiction Severity Index',
//              mtInformation, [mbOk], Left, Top, Height, Width, 0);
            Exit;
          end;
          // Items where "X" or "N" has index of 8: disallow it
          if ((Start = 8) and
            ((LastRadio = 'rgrpG19'))) then
          begin
            MessageBeep(MB_ICONASTERISK);
//            MsgDlg('This is out of range. Try again.',
//              'Addiction Severity Index',
//              mtInformation, [mbOk], Left, Top, Height, Width, 0);
            Exit;
          end;
          // Items where "X" or "N" has index of 9: disallow it
          if ((Start = 9) and
            ((LastRadio = 'rgrpE10'))) then
          begin
            MessageBeep(MB_ICONASTERISK);
//            MsgDlg('This is out of range. Try again.',
//              'Addiction Severity Index',
//              mtInformation, [mbOk], Left, Top, Height, Width, 0);
            Exit;
          end;
           // Adjust the number items for "1" input
          if not StartZero then // for those that start with "1"
            Start := Start - 1;
          if Start > TRzRadioGroup(Components[i]).Items.Count - 1 then
          begin
            MessageBeep(MB_ICONASTERISK);
//            MsgDlg('This is out of range. Try again.',
//              'Addiction Severity Index',
//              mtInformation, [mbOk], Left, Top, Height, Width,  0);
            Exit;
          end;
          // This acts like clicking on the item
          TRzRadioGroup(Components[i]).ItemIndex := Start;
        end;

            // Questions without X or N
        if ((LastRadio = 'rgrpG8') or (LastRadio = 'rgrpG9') or
          (LastRadio = 'rgrpG17') or (LastRadio = 'rgrpG18') or
          (LastRadio = 'rgrpD14')) then
        begin
          if start >= 100 then // "X" or "N" keys -- not used
          begin
            MessageBeep(MB_ICONASTERISK);
//            MsgDlg('This key is not recognized.',
//              'Addiction Severity Index',
//              mtInformation, [mbOk], Left, Top, Height, Width, 0);
            Exit;
          end
          else
          begin // only appropriate keys
            Perform(WM_NEXTDLGCTL, 0, 0);
            Result := True;
            exit;
          end;
        end
        else
        begin
          if Start = 100 then // 'X' key
          begin
            // these items have X as last item
            GotX := false;
            if ((LastRadio = 'rgrpG15') or (LastRadio = 'rgrpM3') or
              (LastRadio = 'rgrpM4') or (LastRadio = 'rgrpM5') or
              (LastRadio = 'rgrpM7') or (LastRadio = 'rgrpM8') or
              (LastRadio = 'rgrpM9') or (LastRadio = 'rgrpM10') or
              (LastRadio = 'rgrpM11') or (LastRadio = 'rgrpE3') or
              (LastRadio = 'rgrpE4') or (LastRadio = 'rgrpE5') or
              (LastRadio = 'rgrpE8') or (LastRadio = 'rgrpE22') or
              (LastRadio = 'rgrpE23') or (LastRadio = 'rgrpE24') or
              (LastRadio = 'rgrpD33') or (LastRadio = 'rgrpD34') or
              (LastRadio = 'rgrpL1') or
              (LastRadio = 'rgrpL2') or (LastRadio = 'rgrpL24') or
              (LastRadio = 'rgrpL30') or (LastRadio = 'rgrpL31') or
              (LastRadio = 'rgrpL32') or (LastRadio = 'rgrpF1') or
              (LastRadio = 'rgrpF7') or (LastRadio = 'rgrpF8') or
              (LastRadio = 'rgrpF36') or (LastRadio = 'rgrpF37') or
              (LastRadio = 'rgrpF38') or (LastRadio = 'rgrpP2') or
              (LastRadio = 'rgrpP3') or (LastRadio = 'rgrpP3l') or
              (LastRadio = 'rgrpP4') or (LastRadio = 'rgrpP4l') or
              (LastRadio = 'rgrpP5') or (LastRadio = 'rgrpP5l') or
              (LastRadio = 'rgrpP6') or (LastRadio = 'rgrpP6l') or
              (LastRadio = 'rgrpP7') or (LastRadio = 'rgrpP7l') or
              (LastRadio = 'rgrpP8') or (LastRadio = 'rgrpP8l') or
              (LastRadio = 'rgrpP9') or (LastRadio = 'rgrpP9l') or
              (LastRadio = 'rgrpP10') or (LastRadio = 'rgrpP10l') or
              (LastRadio = 'rgrpP12') or (LastRadio = 'rgrpP13') or
              (LastRadio = 'rgrpP14') or (LastRadio = 'rgrpP15') or
              (LastRadio = 'rgrpP16') or (LastRadio = 'rgrpP17') or
              (LastRadio = 'rgrpP18') or (LastRadio = 'rgrpP19') or
              (LastRadio = 'rgrpP20') or (LastRadio = 'rgrpP21') or
              (LastRadio = 'rgrpP22')) then
            begin
              GotX := true;
              // This acts like clicking on the item
              TRzRadioGroup(Components[i]).ItemIndex :=
                TRzRadioGroup(Components[i]).Items.Count - 1;
            end;
            // these items have "X" as second to last item
            if ((LastRadio = 'rgrpG19') or (LastRadio = 'rgrpE9') or
              (LastRadio = 'rgrpE10') or (LastRadio = 'rgrpE20') or
              (LastRadio = 'rgrpE21') or (LastRadio = 'rgrpD28') or
              (LastRadio = 'rgrpD29') or (LastRadio = 'rgrpD30') or
              (LastRadio = 'rgrpD31') or (LastRadio = 'rgrpD32') or
              (LastRadio = 'rgrpL23') or (LastRadio = 'rgrpL25') or
              (LastRadio = 'rgrpL28') or (LastRadio = 'rgrpL29') or
              (LastRadio = 'rgrpF3') or (LastRadio = 'rgrpF4') or
              (LastRadio = 'rgrpF6') or (LastRadio = 'rgrpF9') or
              (LastRadio = 'rgrpF10') or (LastRadio = 'rgrpF18l') or
              (LastRadio = 'rgrpF18') or (LastRadio = 'rgrpF19l') or
              (LastRadio = 'rgrpF19') or (LastRadio = 'rgrpF20l') or
              (LastRadio = 'rgrpF20') or (LastRadio = 'rgrpF21l') or
              (LastRadio = 'rgrpF21') or (LastRadio = 'rgrpF22l') or
              (LastRadio = 'rgrpF22') or (LastRadio = 'rgrpF23l') or
              (LastRadio = 'rgrpF23') or (LastRadio = 'rgrpF24l') or
              (LastRadio = 'rgrpF24') or (LastRadio = 'rgrpF25l') or
              (LastRadio = 'rgrpF25') or (LastRadio = 'rgrpF26l') or
              (LastRadio = 'rgrpF26') or (LastRadio = 'rgrpF27l') or
              (LastRadio = 'rgrpF27') or (LastRadio = 'rgrpF28l') or
              (LastRadio = 'rgrpF28') or (LastRadio = 'rgrpF29l') or
              (LastRadio = 'rgrpF29') or (LastRadio = 'rgrpF32') or
              (LastRadio = 'rgrpF33') or (LastRadio = 'rgrpF34') or
              (LastRadio = 'rgrpF35')) then
            begin
              GotX := true;
              // This acts like clicking on the item
              TRzRadioGroup(Components[i]).ItemIndex :=
                TRzRadioGroup(Components[i]).Items.Count - 2;
            end;
            if not GotX then
            begin
              MessageBeep(MB_ICONASTERISK);
//              MsgDlg('This key is not recognized.',
//                'Addiction Severity Index',
//                mtInformation, [mbOk], Left, Top, Height, Width, 0);
              Exit;
            end;
          end; // start = 100

          if Start = 200 then // 'N' Always the last item when used
          begin
            // only these items use the "N" key
            if ((LastRadio = 'rgrpG19') or (LastRadio = 'rgrpG12') or
              (LastRadio = 'rgrpE7') or (LastRadio = 'rgrpE9') or
              (LastRadio = 'rgrpE10') or (LastRadio = 'rgrpE20') or
              (LastRadio = 'rgrpE21') or (LastRadio = 'rgrpD28') or
              (LastRadio = 'rgrpD29') or (LastRadio = 'rgrpD30') or
              (LastRadio = 'rgrpD31') or
              (LastRadio = 'rgrpD32') or (LastRadio = 'rgrpL23') or
              (LastRadio = 'rgrpL25') or (LastRadio = 'rgrpL28') or
              (LastRadio = 'rgrpL29') or (LastRadio = 'rgrpL30') or
              (LastRadio = 'rgrpF3') or (LastRadio = 'rgrpF4') or
              (LastRadio = 'rgrpF6') or (LastRadio = 'rgrpF9') or
              (LastRadio = 'rgrpF10') or
              (LastRadio = 'rgrpF18') or (LastRadio = 'rgrpF18l') or
              (LastRadio = 'rgrpF19') or (LastRadio = 'rgrpF19l') or
              (LastRadio = 'rgrpF20') or (LastRadio = 'rgrpF20l') or
              (LastRadio = 'rgrpF21') or (LastRadio = 'rgrpF21l') or
              (LastRadio = 'rgrpF22') or (LastRadio = 'rgrpF22l') or
              (LastRadio = 'rgrpF23') or (LastRadio = 'rgrpF23l') or
              (LastRadio = 'rgrpF24') or (LastRadio = 'rgrpF24l') or  
              (LastRadio = 'rgrpF25') or (LastRadio = 'rgrpF25l') or
              (LastRadio = 'rgrpF26') or (LastRadio = 'rgrpF26l') or
              (LastRadio = 'rgrpF27') or (LastRadio = 'rgrpF27l') or
              (LastRadio = 'rgrpF28') or (LastRadio = 'rgrpF28l') or
              (LastRadio = 'rgrpF29') or (LastRadio = 'rgrpF29l') or
              (LastRadio = 'rgrpF32') or (LastRadio = 'rgrpF33') or
              (LastRadio = 'rgrpF34') or (LastRadio = 'rgrpF35')) then
              // This acts like clicking on the item
              TRzRadioGroup(Components[i]).ItemIndex :=
                TRzRadioGroup(Components[i]).Items.Count - 1
            else
            begin
              MessageBeep(MB_ICONASTERISK);
//              MsgDlg('This key is not recognized.',
//                'Addiction Severity Index',
//                mtInformation, [mbOk], Left, Top, Height, Width, 0);
              Exit;
            end;
          end;
        end;
            // go to next question
        Perform(WM_NEXTDLGCTL, 0, 0);
      end;
  end;
  Result := True;
end;

procedure TfrmASI.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if ActiveControl.ClassName = 'TFMMemo' then
    exit;

 // Note: use this here to avoid a beep
  if Key = #13 then {- Go to next field -}
  begin
    Perform(WM_NEXTDLGCTL, 0, 0);
    Key := #0; {- requires "Message" in "uses" -}
    Exit;
  end;
end;

procedure TfrmASI.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
// This is the case where a Radio Group has a button clicked and the user
// presses the Tab or Return Keys.  Go to next item without changing last item.
  if ((ActiveControl.ClassName = 'TRzGroupButton') and ((Key = vk_Tab) or
    (Key = vk_Return))) then
  begin
    Perform(WM_NEXTDLGCTL, 0, 0);
    Key := 0; {- requires "Message" in "uses" -}
    Exit;
  end;
end;

procedure TfrmASI.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  Fresh := false;
  if Key = vk_Escape then
  begin
    Key := $00;
    FixinToClose(sender);
    Exit;
  end;

  // Spin up
  if ((Key = vk_Up) and (ActiveControl.ClassName = 'TEdit')) then
    SpinEditField('U', sender);

  // Spin Down
  if ((Key = vk_Down) and (ActiveControl.ClassName = 'TEdit')) then
    SpinEditField('D', sender);

  // Go to next question with down arrow
  if ((Key = vk_Down) and not ((ActiveControl.ClassName = 'TEdit') or
    (ActiveControl.ClassName = 'TORComboEdit') or // Real name for TORComboBox
    (ActiveControl.ClassName = 'TFMMemo'))) then
  begin
    Perform(WM_NEXTDLGCTL, 0, 0);
    Key := $00; {- requires "Message" in "uses" -}
    Exit;
  end;

  // Go to last item with up arrow
  if ((Key = vk_Up) and not ((ActiveControl.ClassName = 'TEdit') or
    (ActiveControl.ClassName = 'TORComboEdit') or // Real name for TORComboBox
    (ActiveControl.ClassName = 'TFMMemo'))) then
  begin
    Perform(WM_INITDIALOG, 0, 0);
    Key := $00; {- requires "Message" in "uses" -}
    Exit;
  end;

  //Scroll with PgDn
  if ((Key = vk_Next) and not (ActiveControl.ClassName = 'TFMMemo')) then
    PageScroll(200, Sender);

  // Scroll with PgUp
  if ((Key = vk_Prior) and not (ActiveControl.ClassName = 'TFMMemo')) then
    PageScroll(-200, sender);

  if ActiveControl.ClassName = 'TRzRadioGroup' then
  begin
    LastRadio := ActiveControl.Name; // A mouse control also sets this. Both needed.
    if CheckRadioKey(Key) then
    begin
      Key := $00; {- requires "Message" in "uses" -}
      if MHA_Registry.UseSpeedTab then
        Perform(WM_NEXTDLGCTL, 0, 0);
    end;
  end;
end;

//---------------------------------------------------------------------
//       Highlight On and Off
//---------------------------------------------------------------------

procedure TfrmASI.aHighLightOn(Sender: TObject);
// Change the foreground and background on the active item. Also add the item
// to the ResponseList (used when saving "Finish Later"--no default values are
// included.
// For Radio Groups, this is activated when the user tabs into the component,
// mouse clicks do not change the ActiveControl.Name used in this procedure.
var
  i: Integer;
  LabelToHighlight: string;

  procedure HighlightLabel(Name: string);
  // Some components have labels that get also highlighted;
  // this routine will remove the "edt" or "cbx" and place "lbl"
  // on the component's name.
  begin
    LabelToHighlight := Name;
    Delete(LabelToHighlight, 1, 3);
    LabelToHighlight := 'lbl' + LabelToHighlight;
    // Drug and history combo boxes have extra letter at end of name: remove it
    if ((LabelToHighlight[Length(LabelToHighlight)] = 'a') or
      (LabelToHighlight[Length(LabelToHighlight)] = 'b') or
      (LabelToHighlight[Length(LabelToHighlight)] = 'd') or
      (LabelToHighlight[Length(LabelToHighlight)] = 'p') or
      (LabelToHighlight[Length(LabelToHighlight)] = 'r') or
      (LabelToHighlight[Length(LabelToHighlight)] = 'y')) then
      Delete(LabelToHighlight, Length(LabelToHighlight), 1);
  end;

begin
  if CheckSkipItems then
    if Length(SkippedItems) = 0 then
    begin
      CheckSkipItems := False;
      lblVerify.caption := '';
      pnlVerify.Height := 0;
      CrossCheckItems := True;
      ConductCrossChecks(Sender);
      Exit;
    end;

  if not Initialize then
    ASI_Save.SaveASIOnTheRun; // uMHA_ASI_Save

// Place the item in the middle of the scroll box.
  if pgctrlASI.ActivePage = TabGeneral then
    if ActiveControl.Top > sbGeneral.Height div 2 then
      sbGeneral.VertScrollBar.Position := sbGeneral.VertScrollBar.Position +
        ActiveControl.Top - (sbGeneral.Height div 2);
  if pgctrlASI.ActivePage = TabMedical then
    if ActiveControl.Top > sbMedical.Height div 2 then
      sbMedical.VertScrollBar.Position := sbMedical.VertScrollBar.Position +
        ActiveControl.Top - (sbMedical.Height div 2);
  if pgctrlASI.ActivePage = TabEmploy then
    if ActiveControl.Top > sbEmploy.Height div 2 then
      sbEmploy.VertScrollBar.Position := sbEmploy.VertScrollBar.Position +
        ActiveControl.Top - (sbEmploy.Height div 2);
  if pgctrlASI.ActivePage = TabDrug then
    if ActiveControl.Top > sbDrug.Height div 2 then
      sbDrug.VertScrollBar.Position := sbDrug.VertScrollBar.Position +
        ActiveControl.Top - (sbDrug.Height div 2);
  if pgctrlASI.ActivePage = TabLegal then
    if ActiveControl.Top > sbLegal.Height div 2 then
      sbLegal.VertScrollBar.Position := sbLegal.VertScrollBar.Position +
        ActiveControl.Top - (sbLegal.Height div 2);
  if pgctrlASI.ActivePage = TabFamily then
    if ActiveControl.Top > sbFamily.Height div 2 then
      sbFamily.VertScrollBar.Position := sbFamily.VertScrollBar.Position +
        ActiveControl.Top - (sbFamily.Height div 2);
  if pgctrlASI.ActivePage = TabSocial then
    if ActiveControl.Top > sbSocial.Height div 2 then
      sbSocial.VertScrollBar.Position := sbSocial.VertScrollBar.Position +
        ActiveControl.Top - (sbSocial.Height div 2);
  if pgctrlASI.ActivePage = TabPsych then
    if ActiveControl.Top > sbPsych.Height div 2 then
      sbPsych.VertScrollBar.Position := sbPsych.VertScrollBar.Position +
        ActiveControl.Top - (sbPsych.Height div 2);
 
  lblHint.Caption  := '';
  LabelToHighlight := '';

  // scan the components, find the active one, highlight it and, if needed, its
  // label. Add the component to the list of items the user has visited.
  // Change the hint bar as well.
  with Sender do
  begin
    for i := 0 to ComponentCount - 1 do
    begin
      if Components[i] is TRzRadioGroup then
        with TRzRadioGroup(Components[i]) do
          begin
            if Name = ActiveControl.Name then
              else
              begin
                if Enabled then
                  begin
                    Color := clBtnFace;
                    Font.Color := clBlack;
                    ItemFont.Color := clBlack;
                  end;
              end;
          end;

      if Components[i] is TEdit then
        with TEdit(Components[i]) do
          begin
          // Item has focus, highlight it
          if Name = ActiveControl.Name then
            begin
              Color := MHA_Registry.HighlightBackground;
              Font.Color := MHA_Registry.HighlightForeground;
              lblHint.Caption := Hint;
              HighLightLabel(ActiveControl.Name);
                 // Add to list of items the user clicked on
              ResponseList := ResponseList + UpperCase(Name) + ' ';
            end
            else
            begin
              if Enabled then
                begin
                  Color := clWindow;
                  Font.Color := clBlack;
                end
                else
                begin
                  Color := clSilver;
                  Font.Color := clBlack;
                end;
            end;
        end;

      if Components[i] is TRzEdit then
        with TRzEdit(Components[i]) do
        begin
          if Name = ActiveControl.Name then
            begin
              Color := MHA_Registry.HighlightBackground;
              Font.Color := MHA_Registry.HighlightForeground;
              lblHint.Caption := Hint;
              HighLightLabel(ActiveControl.Name);
                 // Add to list of items the user clicked on
              ResponseList := ResponseList + UpperCase(Name) + ' ';
            end
            else
            begin
              if Enabled then
                begin
                  Color := clWindow;
                  Font.Color := clBlack;
                end
                else
                begin
                  Color := clSilver;
                  Font.Color := clBlack;
                end;
            end;
        end;

      if Components[i] is TRzComboBox then
        with TRzComboBox(Components[i]) do
          begin
          if Name = ActiveControl.Name then
            begin
              Color := MHA_Registry.HighlightBackground;
              Font.Color := MHA_Registry.HighlightForeground;
              HighLightLabel(ActiveControl.Name);
              lblHint.Caption := Hint;
                // Add to list of items the user clicked on
              ResponseList := ResponseList + UpperCase(Name) + ' ';
            end
            else
            begin
              if Enabled then
                begin
                  Color := clWindow;
                  Font.Color := clBlack;
                end;
            end;
        end;

      if Components[i] is TORComboBox then
        with TORComboBox(Components[i]) do
          begin
          if Name = ActiveControl.Name then
            begin
              Color := MHA_Registry.HighlightBackground;
              Font.Color := MHA_Registry.HighlightForeground;
              HighLightLabel(ActiveControl.Name);
              lblHint.Caption := Hint;
                // Add to list of items the user clicked on
              ResponseList := ResponseList + UpperCase(Name) + ' ';
            end
            else
            begin
              Color := clWindow;
              Font.Color := clBlack;
            end;
          end;

      if Components[i] is TDateTimePicker then
        with TDateTimePicker(Components[i]) do
        begin
          if Name = ActiveControl.Name then
            begin
              Color := MHA_Registry.HighlightBackground;
              Font.Color := MHA_Registry.HighlightForeground;
              HighLightLabel(ActiveControl.Name);
              lblHint.Caption := Hint;
                // Add to list of items the user clicked on
              ResponseList := ResponseList + UpperCase(Name) + ' ';
            end
            else
            begin
              Color := clWindow;
              Font.Color := clBlack;
            end;
        end;

      if Components[i] is TFMMemo then
        with TFMMemo(Components[i]) do
          begin
          if Name = ActiveControl.Name then
            begin
              lblHint.Caption := TFMMemo(Components[i]).Hint;
              TFMMemo(Components[i]).Color := MHA_Registry.HighlightBackground;
            end
            else
            begin
             if Enabled then
               begin
                 Color := clWindow;
                 Font.Color := clBlack;
              end;
            end;
         end;

      if Components[i] is TLabel then
        with TLabel(Components[i]) do
          begin
            if Name <> 'lblVerify' then
              begin
                Color :=  clBtnFace;
                Font.Color := clBlack;
              end;  
          end;

      if Components[i] is TStExpressionEdit then
        TStExpressionEdit(Components[i]).Color := clBtnFace;
    end; // for i
  end; // with sender
  

  // Find the proper label and highlight it
  if LabelToHighlight = '' then
      Exit;
  for i := 0 to ComponentCount - 1 do
    begin
      if Components[i] is TLabel then
        if TLabel(Components[i]).Name = LabelToHighlight then
          begin
            TLabel(Components[i]).Color := MHA_Registry.HighlightBackground;
            TLabel(Components[i]).Font.Color := MHA_Registry.HighlightForeground;
            Break;
          end;
    end; // for i
end;


//-------------------------------------------------------------------
//      Memo field comments: Saved to file for recovery purposes
//-------------------------------------------------------------------


procedure TfrmASI.RecoverComments;
begin
  if FileExists(UserSession.TempFolderName + 'ASI_General_' + RPCBrokerV.Server + '.txt') then
    memGeneral.Lines.LoadFromFile(UserSession.TempFolderName + 'ASI_General_' + RPCBrokerV.Server + '.txt');
  if FileExists(UserSession.TempFolderName + 'ASI_Medical_' + RPCBrokerV.Server + '.txt') then
    memMedical.Lines.LoadFromFile(UserSession.TempFolderName + 'ASI_Medical_' + RPCBrokerV.Server + '.txt');
  if FileExists(UserSession.TempFolderName + 'ASI_Employ_' + RPCBrokerV.Server + '.txt') then
    memEmploy.Lines.LoadFromFile(UserSession.TempFolderName + 'ASI_Employ_' + RPCBrokerV.Server + '.txt');
  if FileExists(UserSession.TempFolderName + 'ASI_Drug_' + RPCBrokerV.Server + '.txt') then
    memDrug.Lines.LoadFromFile(UserSession.TempFolderName + 'ASI_Drug_' + RPCBrokerV.Server + '.txt');
  if FileExists(UserSession.TempFolderName + 'ASI_Legal_' + RPCBrokerV.Server + '.txt') then
    memLegal.Lines.LoadFromFile(UserSession.TempFolderName + 'ASI_Legal_' + RPCBrokerV.Server + '.txt');
  if FileExists(UserSession.TempFolderName + 'ASI_Family_' + RPCBrokerV.Server + '.txt') then
    memFamily.Lines.LoadFromFile(UserSession.TempFolderName + 'ASI_Family_' + RPCBrokerV.Server + '.txt');
  if FileExists(UserSession.TempFolderName + 'ASI_Social_' + RPCBrokerV.Server + '.txt') then
    memSocial.Lines.LoadFromFile(UserSession.TempFolderName + 'ASI_Social_' + RPCBrokerV.Server + '.txt');
  if FileExists(UserSession.TempFolderName + 'ASI_Psych_' + RPCBrokerV.Server + '.txt') then
    memPsych.Lines.LoadFromFile(UserSession.TempFolderName + 'ASI_Psych_' + RPCBrokerV.Server + '.txt');
  if FileExists(UserSession.TempFolderName + 'ASI_Spirit_' + RPCBrokerV.Server + '.txt') then
    memSpirit.Lines.LoadFromFile(UserSession.TempFolderName + 'ASI_Spirit_' + RPCBrokerV.Server + '.txt');
  if FileExists(UserSession.TempFolderName + 'ASI_Leisure_' + RPCBrokerV.Server + '.txt') then
    memLeisure.Lines.LoadFromFile(UserSession.TempFolderName + 'ASI_Leisure_' + RPCBrokerV.Server + '.txt');
end;

procedure TfrmASI.MemLeisureExit(Sender: TObject);
begin
  MemLeisure.Lines.SaveToFile(UserSession.TempFolderName + 'ASI_Leisure_' + RPCBrokerV.Server + '.txt');
end;

procedure TfrmASI.MemSpiritExit(Sender: TObject);
begin
  MemSpirit.Lines.SaveToFile(UserSession.TempFolderName + 'ASI_Spirit_' + RPCBrokerV.Server + '.txt');
end;

procedure TfrmASI.MemPsychExit(Sender: TObject);
begin
  MemPsych.Lines.SaveToFile(UserSession.TempFolderName + 'ASI_Psych_' + RPCBrokerV.Server + '.txt');
end;

procedure TfrmASI.MemSocialExit(Sender: TObject);
begin
  MemSocial.Lines.SaveToFile(UserSession.TempFolderName + 'ASI_Social_' + RPCBrokerV.Server + '.txt');
end;

procedure TfrmASI.MemFamilyExit(Sender: TObject);
begin
  MemFamily.Lines.SaveToFile(UserSession.TempFolderName + 'ASI_Family_' + RPCBrokerV.Server + '.txt');
end;

procedure TfrmASI.MemLegalExit(Sender: TObject);
begin
  MemLegal.Lines.SaveToFile(UserSession.TempFolderName + 'ASI_Legal_' + RPCBrokerV.Server + '.txt');
end;

procedure TfrmASI.MemDrugExit(Sender: TObject);
begin
  MemDrug.Lines.SaveToFile(UserSession.TempFolderName + 'ASI_Drug_' + RPCBrokerV.Server + '.txt');
end;

procedure TfrmASI.MemEmployExit(Sender: TObject);
begin
  MemEmploy.Lines.SaveToFile(UserSession.TempFolderName + 'ASI_Employ_' + RPCBrokerV.Server + '.txt');
end;

procedure TfrmASI.MemMedicalExit(Sender: TObject);
begin
  MemMedical.Lines.SaveToFile(UserSession.TempFolderName + 'ASI_Medical_' + RPCBrokerV.Server + '.txt');
end;

procedure TfrmASI.memGeneralExit(Sender: TObject);
begin
  MemGeneral.Lines.SaveToFile(UserSession.TempFolderName + 'ASI_General_' + RPCBrokerV.Server + '.txt');
end;


//-------------------------------------------------------------------
//            Verify, Save data, Sign it
//-------------------------------------------------------------------


function TfrmASI.SkippedItems: string;
const
  RequiredComboBoxs: string =
    // General
  'cbxG3,cbxG11,' +
    // Drug
  'cbxD1r,cbxD2r,cbxD3r,cbxD4r,cbxD5r,cbxD6r,cbxD7r,cbxD8r,cbxD9r,cbxD10r,cbxD11r,cbxD12r';
  RequiredRadioGroups: string =
    // General
  'rgrpG9,rgrpG12,rgrpG19,' +
    // Medical
  'rgrpM3,rgrpM4,rgrpM5,rgrpM7,rgrpM8,rgrpM10,rgrpM11,' +
    // Employment
  'rgrpE4,rgrpE5,rgrpE7,rgrpE9,rgrpE10,rgrpE21,rgrpE23,rgrpE24,' +
     // Drug
  'rgrpD28,rgrpD29,rgrpD30,rgrpD31,' +
    // Legal
  'rgrpL1,rgrpL2,rgrpL24,rgrpL28,rgrpL29,rgrpL31,rgrpL32,' +
    // Social
  'rgrpF1,rgrpF3,rgrpF4,rgrpF6,rgrpF7,rgrpF8,rgrpF18,rgrpF18l,rgrpF19,rgrpF19l,rgrpF20,rgrpF20l,rgrpF21,rgrpF21l,' +
    'rgrpF22,rgrpF22l,rgrpF23,rgrpF23l,rgrpF24,rgrpF24l,rgrpF25,rgrpF25l,rgrpF26,rgrpF26l,rgrpF28,rgrpF28l,' +
    'rgrpF29,rgrpF29l,rgrpF32,rgrpF34,rgrpF37,rgrpF38,' +
    // Psych
  'rgrpP2,rgrpP3,rgrpP3l,rgrpP4,rgrpP4l,rgrpP5,rgrpP5l,rgrpP6,rgrpP6l,rgrpP7,rgrpP7l,' +
    'rgrpP8,rgrpP8l,rgrpP9,rgrpP9l,rgrpP10,rgrpP10l,edtP11,rgrpP12,rgrpP13,rgrpP21,rgrpP22,';
  RequiredEdits: string =
    // General
  'edtG14y,edtG14m,edtG20,' +
    // Medical
  'edtM1,edtM6,' +
    // Employment
  'edtE1y,edtE1m,edtE2,edtE6y,edtE6m,edtE11,edtE12,' +
    'edtE13,edtE14,edtE15,edtE16,edtE17,edtE18,' +
     // Drug
  'edtD1a,edtD1y,edtD4a,edtD4y,edtD7a,edtD7y,edtD10a,edtD10y,' +
    'edtD2a,edtD2y,edtD5a,edtD5y,edtD8a,edtD8y,edtD11a,edtD11y,' +
    'edtD3a,edtD3y,edtD6a,edtD6y,edtD9a,edtD9y,edtD12a,edtD12y,' +
    'edtD13a,edtD13y,edtD19,edtD20,edtD23,edtD24,edtD25,edtD26,edtD27,' +
    // Legal
  'edtL3,edtL4,edtL5,edtL6,edtL7,edtL8,edtL9,edtL10,edtL11,edtL12,edtL13,edtL14,' +
    'edtL15,edtL16,edtL17,edtL18,edtL19,edtL20,edtL21,edtL26,edtL27,' +
    // Social
  'edtF30,edtF31,' +
    // Psych
  'edtP1a,edtP1b,edtP11';

var
  TempStr,
    BlankItem: string;
  i, j, k: integer;
  GotOne: Boolean;

  function RequiredComboBox(Name: string): Boolean;
  begin
    result := false;
    if Find(RequiredComboBoxs, Name) > 0 then
      result := true;
  end;

  function RequiredRadioGroup(Name: string): Boolean;
  begin
    result := false;
    if Find(RequiredRadioGroups, Name) > 0 then
      result := true;
  end;

  function RequiredEdit(Name: string): Boolean;
  begin
    result := false;
    if Find(RequiredEdits, Name) > 0 then
      result := true;
  end;

  procedure MoveToPage(P: Char);
  begin
    case P of
      'G': begin
          pgctrlASI.ActivePage := TabGeneral;
        end;
      'M': begin
          pgctrlASI.ActivePage := TabMedical;
        end;
      'E': begin
          pgctrlASI.ActivePage := TabEmploy;
        end;
      'D': begin
          pgctrlASI.ActivePage := TabDrug;
        end;
      'L': begin
          pgctrlASI.ActivePage := TabLegal;
        end;
      'F': begin
          pgctrlASI.ActivePage := TabSocial;
        end;
      'P': begin
          pgctrlASI.ActivePage := TabPsych;
        end;
    end;
  end;

begin
  Result := '';
  if not CheckSkipItems then // Don't continue
    Exit;
  BlankItem := '';
  GotOne := False; // used to place user at required item

  for i := 0 to ComponentCount - 1 do
  begin
    TempStr := '';
    k := 0;
    if Components[i] is TRzComboBox then
      if ((TRzComboBox(Components[i]).Visible) and
        (TRzComboBox(Components[i]).Text = '') and
        (RequiredComboBox(TRzComboBox(Components[i]).Name))) then
      begin
        TempStr := TRzComboBox(Components[i]).Name;
        if TempStr[Length(TempStr)] in ['a'..'z', 'A'..'Z'] then
          k := 1;
        for j := 4 to Length(TempStr) - k do
          BlankItem := BlankItem + TempStr[j];
        BlankItem := BlankItem + ' ';
        if not GotOne then
        begin
          MoveToPage(TempStr[5]);
                //ActiveControl := TRzComboBox(Components[i]);
          GotOne := True;
        end;
      end;
    if Components[i] is TRzRadioGroup then
      if ((TRzRadioGroup(Components[i]).Visible) and
        (TRzRadioGroup(Components[i]).ItemIndex = -1) and
        (RequiredRadioGroup(TRzRadioGroup(Components[i]).Name))) then
      begin
        TempStr := TRzRadioGroup(Components[i]).Name;
        if TempStr[Length(TempStr)] in ['a'..'z', 'A'..'Z'] then
          k := 1;
        for j := 5 to Length(TempStr) - k do
          BlankItem := BlankItem + TempStr[j];
        BlankItem := BlankItem + ' ';
        if not GotOne then
        begin
          MoveToPage(TempStr[6]);
                // ActiveControl := TRzRadioGroup(Components[i]);
          GotOne := True;
        end;
      end;
    if Components[i] is TEdit then
      if ((TEdit(Components[i]).Visible) and
        (TEdit(Components[i]).Text = '') and
        (RequiredEdit(TEdit(Components[i]).Name))) then
      begin
        if TEdit(Components[i]).Name = 'edtG19e' then
          if rgrpG19.ItemIndex <> 5 then
            exit;
        if TEdit(Components[i]).Name = 'edtM3e' then
          if rgrpM3.ItemIndex <> 1 then
            exit;
        if TEdit(Components[i]).Name = 'edtM5e' then
          if rgrpM5.ItemIndex <> 1 then
            exit;

        TempStr := TEdit(Components[i]).Name;
        if TempStr[Length(TempStr)] in ['a'..'z', 'A'..'Z'] then
          k := 1;
        for j := 4 to Length(TempStr) - k do
          BlankItem := BlankItem + TempStr[j];
        BlankItem := BlankItem + ' ';
        if not GotOne then
        begin
          MoveToPage(TempStr[5]);
                //ActiveControl := TEdit(Components[i]);
          GotOne := True;
        end;
      end;
  end;

  if Length(BlankItem) <> 0 then
  begin
    pnlVerify.Height := 39;
    pnlVerify.BevelWidth := 1;
    pnlVerify.BorderStyle := bsNone;
    lblVerify.Caption := 'Required items not answered: ' + BlankItem;
    lblVerify.Color := clRed;
    lblVerify.Font.Color := clWhite;
  end;
  Result := BlankItem;
end;

function TfrmASI.VerifyData(sender: TObject): boolean;
begin
  Result := false;
  FUserTerminates := false;
  CheckSkipItems := True;

  // "SkippedItems" is the above function that lists the required items
  // that have not been completed. An empty list means all the items are
  // done. Consistency checks are done at the item level when the user
  // is working on that item.
  if Length(SkippedItems) <> 0 then
    Exit;

  lblVerify.caption := '';
  pnlVerify.Height := 0;
  CheckSkipItems := False;

  Result := True;
end;


//-------------------------------------------------------------------
//          Spin-Edit Up\Down Arrows
//-------------------------------------------------------------------

//  Note: 'X' & 'N' make typical spin components inoperative

procedure TfrmASI.SpinUp(Sender: TObject);
// Max value is stored in TEdit.Tag
var
  Max,
    i, N: Integer;
begin
  for i := 0 to ComponentCount - 1 do
  begin
    if Components[i] is TEdit then
      if TEdit(Components[i]).Name = ActiveControl.Name then
      begin
        if ((Length(TEdit(Components[i]).text) = 0) or
          (UpperCase(TEdit(Components[i]).text) = 'X') or
          (UpperCase(TEdit(Components[i]).text) = 'N')) then
        begin
          TEdit(Components[i]).text := '0';
          Exit;
        end;
        if TEdit(Components[i]).Tag = 0 then
          Max := Patient.Age
        else
          Max := TEdit(Components[i]).Tag;
        N := StrToInt(TEdit(Components[i]).text);

        if N >= Max then
          N := Max - 1;
        Inc(N);
        TEdit(Components[i]).text := IntToStr(N);
        Break;
      end;
  end;
end;

procedure TfrmASI.SpinDown(Sender: TObject);
// The minimum value is always zero.
var
  i, N: Integer;
begin
  for i := 0 to ComponentCount - 1 do
  begin
    if Components[i] is TEdit then
      if TEdit(Components[i]).Name = ActiveControl.Name then
      begin
        if ((Length(TEdit(Components[i]).text) = 0) or
          (UpperCase(TEdit(Components[i]).text) = 'X') or
          (UpperCase(TEdit(Components[i]).text) = 'N')) then
        begin
          TEdit(Components[i]).Text := '0';
          Exit;
        end;
        N := StrToInt(TEdit(Components[i]).text);
        if N <= 0 then
          N := 1;
        Dec(N);
        TEdit(Components[i]).text := IntToStr(N);
        Break;
      end;
  end;
end;

procedure TfrmASI.SpinEditKeyPress(Sender: TObject; var Key: Char);
// Spin and edit components are linked so that 'X' and 'N' are legal
// keystrokes for spin-edit fields
var
  i: Integer;
  KeyState: TKeyboardState;
begin
  for i := 0 to ComponentCount - 1 do
  begin
      // get the active spin-edit component
    if TEdit(Components[i]).Name = ActiveControl.Name then
    begin
          // remove the default zero
      if TEdit(Components[i]).Text = '0' then
        TEdit(Components[i]).Text := '';
          // remove numbers or letter from edit field, insert the 'X' or 'N'
      if key in [#78, #110 {N,n}] then
      begin
        key := #0;
        TEdit(Components[i]).Text := 'N';
        Exit;
      end;
      if Key in [#88, #120 {X,x}] then
      begin
        key := #0;
        TEdit(Components[i]).Text := 'X';
        Exit;
      end;
      GetKeyboardState(KeyState);
      if ((key in [#9, #8, #16]) or
        (key in [#48..#57 {Numbers}]) or
        ((key in [#96..#105] {Key pad numbers}) and (KeyState[VK_NUMLOCK] = 1))) then
      begin
        if ((TEdit(Components[i]).Text = 'X') or
          (TEdit(Components[i]).Text = 'N')) then
          (TEdit(Components[i]).Text := '');
        Exit;
      end
      else
      begin
        key := #0;
        MessageBeep(MB_ICONASTERISK);
//            MsgDlg('Only numbers and "X" or "N" are allowed.',
//             'Addiction Severilty Index', mtInformation, [mbOk],
//             Left, Top, Height, Width, 0);
      end;
    end;
  end;
end;


//--------------------------------------------------------------------------
//       Range Checks for Spin Edit Items
//--------------------------------------------------------------------------


function TfrmASI.CheckRange(StringIn: string; Max, Min: integer): Boolean;
var
  n: integer;
begin
  Result := True;
  // Allow "X" and "N" to be okay
  if Length(StringIn) = 1 then
    if ((UpperCase(StringIn) = 'X') or (UpperCase(StringIn) = 'N')) then
      Exit;

  // Allow a blank to slip thru
  if Length(StringIn) = 0 then
    Exit;

  Result := False;

  try
    n := StrToInt(StringIn);
  except
    on E: Exception do
    begin
      MsgDlg('This number does not work. ' + e.message,
        'Addiction Severity Index', mtInformation, [mbOK], 0);
        //Left, Top, Height, Width,  0);
        //WhichEdit.Text := ' ';
        //ActiveControl := WhichEdit;
      Exit;
    end;
  end; {- try-}

  // Range checks
  if n > Max then
  begin
    MsgDlg('This number is too large. The maximum is ' + IntToStr(Max) + '.',
      'Addiction Severity Index', mtInformation, [mbOK], 0);
      //Left, Top, Height, Width, 0);
      //WhichEdit.Text := ' ';
      //ActiveControl := WhichEdit;
    Exit;
  end;
  if n < Min then
  begin
    MsgDlg('This number is too small. The minimum is ' + IntToStr(Min) + '.',
      'Addiction Severity Index', mtInformation, [mbOK], 0);
      //Left, Top, Height, Width, 0);
      //WhichEdit.Text := ' ';
      //ActiveControl := WhichEdit;
    Exit;
  end;

  Result := True;
end;

procedure TfrmASI.Check12(Sender: TObject);
var
  i: Integer;
begin
  if Initialize then
    Exit;
  for i := 0 to ComponentCount - 1 do
  begin
    if Components[i] is TEdit then
      if TEdit(Components[i]).Name = ActiveControl.Name then
      begin
        if not CheckRange(TEdit(Components[i]).text, 11, 0) then
          TEdit(Components[i]).Text := '';
        Break;
      end;
  end;
end;

procedure TfrmASI.Check9(Sender: TObject);
var
  i: Integer;
begin
  if Initialize then
    Exit;
  for i := 0 to ComponentCount - 1 do
  begin
    if Components[i] is TEdit then
      if TEdit(Components[i]).Name = ActiveControl.Name then
      begin
        if not CheckRange(TEdit(Components[i]).text, 9, 0) then
          TEdit(Components[i]).Text := '';
        Break;
      end;
  end;
end;

procedure TfrmASI.Check99(Sender: TObject);
var
  i: Integer;
begin
  if Initialize then
    Exit;
  for i := 0 to ComponentCount - 1 do
  begin
    if Components[i] is TEdit then
      if TEdit(Components[i]).Name = ActiveControl.Name then
      begin
        if not CheckRange(TEdit(Components[i]).text, 99, 0) then
          TEdit(Components[i]).Text := '';
        Break;
      end;
  end;
end;

procedure TfrmASI.Check9999(Sender: TObject);
var
  i: Integer;
begin
  if Initialize then
    Exit;
  for i := 0 to ComponentCount - 1 do
  begin
    if Components[i] is TEdit then
      if TEdit(Components[i]).Name = ActiveControl.Name then
      begin
        if not CheckRange(TEdit(Components[i]).text, 9999, 0) then
          TEdit(Components[i]).Text := '';
        Break;
      end;
  end;
end;

procedure TfrmASI.Check30(Sender: TObject);
var
  i: Integer;
begin
  if Initialize then
    Exit;
  for i := 0 to ComponentCount - 1 do
  begin
    if Components[i] is TEdit then
      if TEdit(Components[i]).Name = ActiveControl.Name then
      begin
        if not CheckRange(TEdit(Components[i]).text, 30, 0) then
          TEdit(Components[i]).Text := '';
        Break;
      end;
  end;
end;

procedure TfrmASI.CheckLife(Sender: TObject);
var
  i: Integer;
begin
  if Initialize then
    Exit;
  for i := 0 to ComponentCount - 1 do
  begin
    if Components[i] is TEdit then
      if TEdit(Components[i]).Name = ActiveControl.Name then
      begin
        if not CheckRange(TEdit(Components[i]).text, Patient.Age, 0) then
          TEdit(Components[i]).Text := '';
        Break;
      end;
  end;
end;


//--------------------------------------------------------------------------
//              Spin-Edit Items: Exit
//--------------------------------------------------------------------------


procedure TfrmASI.edtG14yExit(Sender: TObject);
begin

  if ((edtG14y.Text = 'N') or (edtG14y.Text = 'X')) then
    edtG14m.Text := edtG14y.Text;
end;

procedure TfrmASI.edtG20Exit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('G20', sender);
end;

procedure TfrmASI.edtM1Exit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('M1', sender);
end;

procedure TfrmASI.edtM2yExit(Sender: TObject);
begin
  if ((edtM2y.Text = 'N') or (edtM2y.Text = 'X')) then
    edtM2m.Text := edtM2y.Text;
end;

procedure TfrmASI.edtM2mExit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('M2', sender);
end;

procedure TfrmASI.edtE1yExit(Sender: TObject);
begin
  if ((edtE1y.Text = 'N') or (edtE1y.Text = 'X')) then
    edtE1m.Text := edtE1y.Text;
  if not CrossCheckItems then
    CrossCheck('E1y', sender);
end;

procedure TfrmASI.edtE6yExit(Sender: TObject);
begin
  if ((edtE6y.Text = 'N') or (edtE6y.Text = 'X')) then
    edtE6m.Text := edtE6y.Text;
end;

procedure TfrmASI.edtE12Exit(Sender: TObject);
begin
  if not CrossCheckItems then
  begin
    CrossCheck('E12a', sender);
    CrossCheck('E12b', sender);
  end;
end;

procedure TfrmASI.edtE15Exit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('E15', sender);
end;

procedure TfrmASI.edtD2dExit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('D2d', sender);
end;

procedure TfrmASI.edtD2yExit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('D2y', sender);
end;

procedure TfrmASI.edtD16Exit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('D16', sender);
end;

procedure TfrmASI.edtD17Exit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('D17', sender);
end;

procedure TfrmASI.edtD21Exit(Sender: TObject);
begin
  if not CrossCheckItems then
  begin
    CrossCheck('D21a', sender);
    CrossCheck('D21b', sender);
  end;
end;

procedure TfrmASI.edtD22Exit(Sender: TObject);
begin
  if not CrossCheckItems then
  begin
    CrossCheck('D22a', sender);
    CrossCheck('D22b', sender);
  end;
end;

procedure TfrmASI.edtD23Exit(Sender: TObject);
begin
  if not CrossCheckItems then
  begin
    CrossCheck('D23a', sender);
    CrossCheck('D23b', sender);
  end;
end;

procedure TfrmASI.edtD24Exit(Sender: TObject);
begin
  if not CrossCheckItems then
  begin
    CrossCheck('D24a', sender);
    CrossCheck('D24b', sender);
  end;
end;

procedure TfrmASI.CheckOutL17(Sender: TObject);
var
  i: Integer;
begin
  for i := 0 to ComponentCount - 1 do
  begin
    if Components[i] is TEdit then
      if TEdit(Components[i]).Name = ActiveControl.Name then
      begin
        CheckRange(TEdit(Components[i]).text, 99, 0);
        Break;
      end;
  end;
  if ((edtL3.text = '0') and (edtL4.text = '0') and
    (edtL5.text = '0') and (edtL6.text = '0') and
    (edtL7.text = '0') and (edtL8.text = '0') and
    (edtL9.text = '0') and (edtL10.text = '0') and
    (edtL11.text = '0') and (edtL12.text = '0') and
    (edtL13.text = '0') and (edtL14.text = '0') and
    (edtL15.text = '0') and (edtL16.text = '0')) then
  begin
    edtL17.text := 'N';
    edtL17.Color := clSilver;
    edtL17.Enabled := false;
    spnL17.Enabled := false;
  end
  else
  begin
    //edtL17.text := '0';
    edtL17.Color := clWindow;
    edtL17.Enabled := true;
    spnL17.Enabled := true;
  end;
end;

procedure TfrmASI.edtL17Exit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('L17', sender);
end;

procedure TfrmASI.edtL20Exit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('L20', sender);
end;

procedure TfrmASI.edtL22Exit(Sender: TObject);
begin
  if ((edtL21.text = 'N') or (edtL21.text = 'X') or
    (edtL21.text = '') or
    (edtL22.text = 'N') or (edtL22.text = 'X') or
    (edtL22.text = '')) then // do nothing
  else
  begin
    if StrToInt(edtL21.text) < StrToInt(edtL22.text) then
    begin
      MsgDlg('The months of lifelong incarceration is less than the ' +
        'months of the last incarceration. Please correct.',
        'Addiction Severity Index', mtInformation, [mbOK], 0);
          //Left, Top, Height, Width, 0);
      ActiveControl := edtL22;
    end;
  end;
end;

procedure TfrmASI.edtL26Exit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('L26', sender);
end;

procedure TfrmASI.edtL27Exit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('L27', sender);
end;

procedure TfrmASI.edtF2yExit(Sender: TObject);
begin
 if ((edtF2y.Text = 'N') or (edtF2y.Text = 'X')) then
  begin
    edtF2m.Text := edtF2y.Text;
  end;
end;

procedure TfrmASI.edtF5yExit(Sender: TObject);
begin
  if ((edtF5y.Text = 'N') or (edtF5y.Text = 'X')) then
    edtF5m.Text := edtF5y.Text;
end;

procedure TfrmASI.edtP11Exit(Sender: TObject);
begin
  if not CrossCheckItems then
  begin
    CrossCheck('P11a', sender);
    CrossCheck('P11b', sender);
  end;
end;

//-------------------------------------------------------------------
//         Radio Group: Click Controls
//-------------------------------------------------------------------

procedure TfrmASI.rgrpG8Click(Sender: TObject);
begin
  if Initialize then Exit;

  if ((NoFollowupAllowed) and (rgrpG8.ItemIndex = 2)) then
  begin
    MsgDlg('This is the veteran''s first ASI and it must be either a Full or Lite Intake.',
      'Addiction Severity Index', mtConfirmation, [mbOk], 0);
      //Left, Top, Height, Width, 0);
    rgrpG8.ItemIndex := -1;
    ActiveControl := rgrpG8;
    exit;
  end;

  if not Fresh then // no responses made yet
  begin
    if MsgDlg('Do you wish to set the items that follow to their default responses? ' +
      '(This may change responses you have already entered.)',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrYes then
        //Left, Top, Height, Width, 0) = mrYes then
      frmMain.Partial_ASI := ' '
    else
      frmMain.Partial_ASI := 'X';
  end;

  if rgrpG8.ItemIndex = 0 then // Full
    ASI_Build.SetupASIFull;
  if rgrpG8.ItemIndex = 1 then // Lite
    ASI_Build.SetupASILite;
  if rgrpG8.ItemIndex = 2 then // Followup
    ASI_Build.SetupASIFollowup;
  rgrpG8.color := MHA_Registry.HighlightBackground;
  Fresh := false;
end;


procedure TfrmASI.rgrpG19Click(Sender: TObject);
begin
  if rgrpG19.ItemIndex = 5 then
  begin
    edtG19e.Color := clWindow;
    edtG19e.Enabled := True;
  end
  else
  begin
    edtG19e.Text := '';
    edtG19e.Color := clSilver;
    edtG19e.Enabled := false;
  end;
  if ((rgrpG19.ItemIndex = 0) or (rgrpG19.ItemIndex = 6 {X}) or
    (rgrpG19.ItemIndex = 7 {N})) then
  begin
    edtG20.Text := 'N';
    edtG20.Color := clSilver;
    edtG20.Enabled := false;
    edtG19e.Text := '';
    edtG19e.Color := clSilver;
    edtG19e.Enabled := false;
    SpnG20.Enabled := false;
    lblG20.Enabled := false;
  end;
  if ((rgrpG19.ItemIndex = 1) or (rgrpG19.ItemIndex = 2) or
    (rgrpG19.ItemIndex = 3) or (rgrpG19.ItemIndex = 4) or
    (rgrpG19.ItemIndex = 5)) then
  begin
    edtG20.Enabled := true;
    edtG20.Color := clWindow;
    edtG20.Text := '0';
    SpnG20.Enabled := true;
    lblG20.Enabled := true;
  end;
end;

procedure TfrmASI.rgrpG12Click(Sender: TObject);
begin
  if Initialize then exit;
  if rgrpG12.ItemIndex <> 3 then
  begin
    MsgDlg('The many default answers provided on this form will not be honored when ' +
      'an ASI is only partially completed. Only the items that you actually answer will ' +
      'be saved. ',
      'Addiction Severity Index', mtInformation, [mbYes], 0);
      //Left, Top, Height, Width, 0);
  end;
end;

procedure TfrmASI.rgrpM3Click(Sender: TObject);
begin
  if rgrpM3.ItemIndex = 1 then
  begin
    edtM3e.Enabled := True;
    edtM3e.Color := clWindow;
  end
  else
  begin
    edtM3e.Text := '';
    edtM3e.Color := clSilver;
    edtM3e.Enabled := false;
  end;
end;

procedure TfrmASI.rgrpM5Click(Sender: TObject);
begin
  if rgrpM5.ItemIndex = 1 then
  begin
    edtM5e.Enabled := True;
    edtM5e.Color := clWindow;
  end
  else
  begin
    edtM5e.Text := '';
    edtM5e.Color := clSilver;
    edtM5e.Enabled := false;
  end;
end;

procedure TfrmASI.rgrpE3Click(Sender: TObject);
begin
  if rgrpE3.ItemIndex = 1 then
  begin
    edtE3e.Enabled := True;
    edtE3e.Color := clWindow;
  end
  else
  begin
    edtE3e.Text := '';
    edtE3e.Color := clSilver;
    edtE3e.Enabled := false;
  end;
end;

procedure TfrmASI.rgrpE4Click(Sender: TObject);
begin
  if Initialize then exit;
  if rgrpE4.ItemIndex <> 0 then
    rgrpE5.Enabled := True;
  if rgrpE4.ItemIndex = 0 then
  begin
    rgrpE5.ItemIndex := 0;
    rgrpE5.Color := clBtnFace;
    rgrpE5.Enabled := false;
  end;
end;

procedure TfrmASI.rgrpE8Click(Sender: TObject);
begin
  if Initialize then exit;
  if rgrpE8.ItemIndex = 0 then
  begin
    rgrpE9.ItemIndex := 3;
    rgrpE9.Color := clSilver;
    rgrpE9.Enabled := False;
  end
  else
    rgrpE9.Enabled := True;
end;

procedure TfrmASI.rgrpD14Click(Sender: TObject);
begin
  if Initialize then exit;
  if rgrpD14.ItemIndex = 0 then
  begin
    edtD15.Text := 'N';
    edtD15.Color := clSilver;
    edtD15.Enabled := false;
    spnD15.Enabled := false;
    edtD16.Text := 'N';
    edtD16.Color := clSilver;
    edtD16.Enabled := false;
    spnD16.Enabled := false;
  end
  else
  begin
    edtD15.Enabled := true;
    spnD15.Enabled := true;
    if edtD15.text = 'N' then
      edtD15.Text := '0';
    edtD15.Color := clWindow;
    edtD16.Enabled := true;
    spnD16.Enabled := true;
    if edtD16.text = 'N' then
      edtD16.Text := '0';
    edtD16.Color := clWindow;
  end;
end;

procedure TfrmASI.rgrpL24Click(Sender: TObject);
begin
  if Initialize then exit;
  if rgrpL24.ItemIndex = 1 then
  begin
    if not rgrpL25.Enabled then
      rgrpL25.Enabled := True;
  end
  else
  begin
    rgrpL25.ItemIndex := 18;
    rgrpL25.Color := clBtnFace;
    if rgrpL25.Enabled then
      rgrpL25.Enabled := False;
  end;
end;


//-------------------------------------------------------------------
//               Page Control
//-------------------------------------------------------------------


procedure TfrmASI.pgctrlASIChange(Sender: TObject);
begin
  if pgctrlASI.ActivePage = TabGeneral then
  begin
    if not CrossCheckItems then
      ActiveControl := cbxG3;
  end;
  if pgctrlASI.ActivePage = TabMedical then
  begin
    if not Initialize then
      if not CrossCheckItems then
        ActiveControl := edtM1;
  end;
  if pgctrlASI.ActivePage = TabEmploy then
  begin
    if not Initialize then
      if not CrossCheckItems then
        ActiveControl := edtE1y;
  end;
  if pgctrlASI.ActivePage = TabDrug then
  begin
    sbDrug.VertScrollBar.Position := 0;
    if not CrossCheckItems then
      ActiveControl := edtD1d;
  end;
  if pgctrlASI.ActivePage = TabLegal then
  begin
    if CrossCheckItems then exit;
    if rgrpG8.ItemIndex <> 2 then
    begin
      ActiveControl := rgrpL1;
      rgrpL1.Color := MHA_Registry.HighlightBackground;
    end
    else
    begin
      ActiveControl := rgrpL2;
      rgrpL2.Color := MHA_Registry.HighlightBackground;
    end;
  end;
  if pgctrlASI.ActivePage = TabFamily then
  begin
    if rgrpG8.ItemIndex = 0 then
      ActiveControl := cbxH1a;
  end;
  if pgctrlASI.ActivePage = TabSocial then
  begin
    if not CrossCheckItems then
    begin
      ActiveControl := rgrpF1;
      rgrpF1.Color := MHA_Registry.HighlightBackground;
    end;
  end;
  if pgctrlASI.ActivePage = TabPsych then
  begin
    if not Initialize then
      if not CrossCheckItems then
        ActiveControl := edtP1a;
  end;
  if pgctrlASI.ActivePage = TabSpirit then
  begin
    ActiveControl := memSpirit;
  end;
  if pgctrlASI.ActivePage = TabLeisure then
  begin
    ActiveControl := memLeisure;
  end;
end;

procedure TfrmASI.PageScroll(Scroll: integer; Sender: TObject);
begin
  if pgctrlASI.ActivePage = TabGeneral then
    sbGeneral.VertScrollBar.Position := sbGeneral.VertScrollBar.Position + Scroll;
  if pgctrlASI.ActivePage = TabMedical then
    sbMedical.VertScrollBar.Position := sbMedical.VertScrollBar.Position + Scroll;
  if pgctrlASI.ActivePage = TabEmploy then
    sbEmploy.VertScrollBar.Position := sbEmploy.VertScrollBar.Position + Scroll;
  if pgctrlASI.ActivePage = TabDrug then
    sbDrug.VertScrollBar.Position := sbDrug.VertScrollBar.Position + Scroll;
  if pgctrlASI.ActivePage = TabLegal then
    sbLegal.VertScrollBar.Position := sbLegal.VertScrollBar.Position + Scroll;
  if pgctrlASI.ActivePage = TabFamily then
    sbFamily.VertScrollBar.Position := sbFamily.VertScrollBar.Position + Scroll;
  if pgctrlASI.ActivePage = TabSocial then
    sbSocial.VertScrollBar.Position := sbSocial.VertScrollBar.Position + Scroll;
  if pgctrlASI.ActivePage = TabPsych then
    sbPsych.VertScrollBar.Position := sbPsych.VertScrollBar.Position + Scroll;
  if pgctrlASI.ActivePage = TabSpirit then
    ActiveControl := memSpirit;
  if pgctrlASI.ActivePage = TabLeisure then
    ActiveControl := memLeisure;
end;

procedure TfrmASI.sbSocialEnter(Sender: TObject);
begin
  if ((edtE11.text = '0') or (edtE11.text = '00') or
    (rgrpE10.ItemIndex = 6)) then
  begin
    rgrpF26.ItemIndex := 3;
    rgrpF26.Color := clBtnFace;
    rgrpF26.Enabled := false;
  end;
end;

//-------------------------------------------------------------------
//  These are dummy items that are used to move to next tab when the
//  user moves from one item to the next.
//-------------------------------------------------------------------

procedure TfrmASI.GeneralNextPageEnter(Sender: TObject);
begin
  if CrossCheckItems then Exit;
  pgctrlASI.ActivePage := TabMedical;
  ActiveControl := edtM1;
end;

procedure TfrmASI.MedicalNextPageEnter(Sender: TObject);
begin
  if CrossCheckItems then Exit;
  pgctrlASI.ActivePage := TabEmploy;
  ActiveControl := edtE1y;
end;

procedure TfrmASI.EmployNextPageEnter(Sender: TObject);
begin
  if CrossCheckItems then Exit;
  pgctrlASI.ActivePage := TabDrug;
  ActiveControl := edtD1d;
end;

procedure TfrmASI.DrugNextPageEnter(Sender: TObject);
begin
  if CrossCheckItems then Exit;
  pgctrlASI.ActivePage := TabLegal;
  if rgrpG8.ItemIndex = 2 then
    ActiveControl := rgrpL2
  else
    ActiveControl := rgrpL1;
end;

procedure TfrmASI.LegalNextpageEnter(Sender: TObject);
begin
  if CrossCheckItems then Exit;
  if rgrpG8.ItemIndex = 0 then // Full ASI
  begin
    pgctrlASI.ActivePage := TabFamily;
    ActiveControl := cbxH1a;
  end
  else
  begin // Followup or Lite
    pgctrlASI.ActivePage := TabSocial;
    ActiveControl := rgrpF1;
  end;
end;

procedure TfrmASI.FamilyNextPageEnter(Sender: TObject);
begin
  if CrossCheckItems then Exit;
  pgctrlASI.ActivePage := TabSocial;
  ActiveControl := rgrpF1;
end;

procedure TfrmASI.SocialNextPageEnter(Sender: TObject);
begin
  if CrossCheckItems then Exit;
  pgctrlASI.ActivePage := TabPsych;
  ActiveControl := edtP1a;
end;


//-------------------------------------------------------------------
//  Comment Fields: Save the size of the comment memo field
//-------------------------------------------------------------------


procedure TfrmASI.spltGeneralExit(Sender: TObject);
begin
  MHA_Registry.ASIGeneral := spltGeneral.percent;
end;

procedure TfrmASI.spltDrugExit(Sender: TObject);
begin
  MHA_Registry.ASIDrug := spltDrug.percent;
end;

procedure TfrmASI.spltSocialExit(Sender: TObject);
begin
  MHA_Registry.ASISocial := spltSocial.percent;
end;

procedure TfrmASI.spltPsychExit(Sender: TObject);
begin
  MHA_Registry.ASIPsych := spltPsych.percent;
end;

procedure TfrmASI.splitMedicalExit(Sender: TObject);
begin
  MHA_Registry.ASIMedical := spltMedical.percent;
end;

procedure TfrmASI.spltLegalExit(Sender: TObject);
begin
  MHA_Registry.ASILegal := spltLegal.percent;
end;

procedure TfrmASI.spltFamilyExit(Sender: TObject);
begin
  MHA_Registry.ASIFamily := spltFamily.percent;
end;

procedure TfrmASI.spltEmployExit(Sender: TObject);
begin
  MHA_Registry.ASIEmploy := spltEmploy.percent;
end;


//-------------------------------------------------------------------
//             Spin-Edit Fields: Change
//-------------------------------------------------------------------


procedure TfrmASI.edtM1Change(Sender: TObject);
begin
  if Initialize then exit;
  if CheckRange(edtM1.text, 99, 0) then
  else
    Exit;
  if rgrpG8.ItemIndex = 0 then // Full
  begin
    if ((edtM1.Text = '0') or (edtM1.Text = '00')) then
      edtM1.Text := 'N';
    if ((edtM1.Text = 'N') or (edtM1.Text = 'X')) then
    begin
      edtM2m.Text := edtM1.Text;
      edtM2y.Text := edtM1.Text;
      edtM2y.Enabled := false;
      edtM2m.Enabled := false;
      spnM2y.Enabled := false;
      spnM2m.Enabled := false;
    end
    else
    begin
      edtM2y.Enabled := true;
      edtM2m.Enabled := true;
      spnM2y.Enabled := true;
      spnM2m.Enabled := true;
      edtM2y.text := '';
      edtM2m.text := '';
    end;
  end;
end;

procedure TfrmASI.edtD15Change(Sender: TObject);
begin
  if Initialize then exit;
  CheckRange(edtD15.text, 99, 0);

  if ((edtD15.Text = '0') or (edtD15.text = '00')) then
  begin
    edtD16.text := 'N';
    edtD16.Color := clSilver;
    edtD16.Enabled := false;
    spnD16.Enabled := false;
  end
  else
  begin
    edtD16.Enabled := true;
    spnD16.Enabled := true;
    //edtD16.text := '';
    edtD16.Color := clWindow;
  end;
end;

procedure TfrmASI.edtD20Change(Sender: TObject);
begin
  if Initialize then exit;
  CheckRange(edtD20.text, 99, 0);
  if ((edtD20.text = '00') or (edtD20.text = '0')) then
  begin
    edtD22.text := 'N';
    edtD22.color := clSilver;
    edtD22.Enabled := false;
    spnD22.Enabled := false;
  end
  else
  begin
    edtD22.Enabled := true;
    spnD22.Enabled := true;
    //edtD22.text := '0';
    edtD22.color := clWindow;
  end
end;

procedure TfrmASI.edtD19Change(Sender: TObject);
begin
  if Initialize then exit;
  CheckRange(edtD19.text, 99, 0);
  if ((edtD19.text = '00') or (edtD19.text = '0')) then
  begin
    edtD21.text := 'N';
    edtD21.color := clSilver;
    edtD21.Enabled := false;
    spnD21.Enabled := false;
  end
  else
  begin
    edtD21.Enabled := true;
    spnD21.Enabled := true;
    //edtD21.text := '0';
    edtD21.color := clWindow;
  end
end;

procedure TfrmASI.edtD27Change(Sender: TObject);
begin
  if Initialize then exit;
  CheckRange(edtD27.text, 30, 0);
  if ((edtD27.text = '0') or (edtD27.text = '00')) then
  begin
    rgrpD29.ItemIndex := 0;
    rgrpD29.Color := clBtnFace;
    rgrpD29.Enabled := false;
    rgrpD31.ItemIndex := 0;
    rgrpD31.Color := clBtnFace;
    rgrpD31.Enabled := false;
    ActiveControl := edtD27;
  end
  else
  begin
    rgrpD29.Enabled := true;
    rgrpD31.Enabled := true;
    rgrpD29.ItemIndex := -1;
    rgrpD31.ItemIndex := -1;
  end;
end;

procedure TfrmASI.edtD26Change(Sender: TObject);
begin
  if Initialize then exit;
  CheckRange(edtD26.text, 30, 0);
  if ((edtD26.text = '0') or (edtD26.text = '00')) then
  begin
    rgrpD28.ItemIndex := 0;
    rgrpD28.Color := clBtnFace;
    rgrpD28.Enabled := false;
    rgrpD30.ItemIndex := 0;
    rgrpD30.Color := clBtnFace;
    rgrpD30.Enabled := false;
    ActiveControl := edtD26;
  end
  else
  begin
    rgrpD28.Enabled := true;
    rgrpD30.Enabled := true;
    rgrpD28.ItemIndex := -1;
    rgrpD30.ItemIndex := -1;
  end;
end;

procedure TfrmASI.edtF11Change(Sender: TObject);
begin
  if Initialize then exit;
  CheckRange(edtF11.text, 9, 0);
  if ((edtF11.text = '0') or (edtF11.text = '00')) then
  begin
    Initialize := true;
    rgrpF24.itemindex := 3;
    Initialize := false;
    rgrpF24.Enabled := false;
  end
  else
    rgrpF24.Enabled := True;
end;

procedure TfrmASI.edtL21Change(Sender: TObject);
begin
  if Initialize then exit;
  CheckRange(edtL21.text, 99, 0);
  if ((edtL21.text = '0') or (edtL21.text = '00')) then
  begin
    if rgrpG8.itemIndex <> 1 then
    begin
      edtL22.text := 'N';
      edtL22.Color := clSilver;
      edtL22.Enabled := false;
      spnL22.Enabled := false;
      rgrpL23.ItemIndex := 18; // 'Not applicable'
      rgrpL23.Color := clBtnFace;
      rgrpL23.Enabled := false;
      ActiveControl := edtL21;
    end;
  end
  else
  begin
    if rgrpG8.itemIndex <> 1 then
    begin
      edtL22.Enabled := true;
      spnL22.Enabled := true;
      rgrpL23.Enabled := true;
      //edtL22.text := '0';
      edtL22.Color := clWindow;
    end;
  end;
end;



//---------------------------------------------------------------------
//     Combo Box Items
//--------------------------------------------------------------------



procedure TfrmASI.cbxG11NeedData(Sender: TObject; const StartFrom: string;
  Direction, InsertAt: Integer);
begin
  inherited;
  (Sender as TORComboBox).ForDataUse(SubSetOfPersons(StartFrom, Direction));
end;


procedure TfrmASI.cbxG3Exit(Sender: TObject);
begin
  fresh := true;
  if cbxG3.Items.IndexOf(cbxG3.Text) = -1 then
  begin
    MsgDlg('That is not a proper program type. Please choose again.',
      'Mental Health Assistant', mtWarning, [mbOK], 0);
      //Left, Top, Height, Width, 0);
    cbxG3.ItemIndex := 0;
    ActiveCOntrol := cbxG3;
  end;
end;


//---------------------------------------------------------------------
//     Date functions
//--------------------------------------------------------------------


function TfrmASI.CheckDeathDate(DateIn: TDateTime): boolean;
begin
  Result := True;
  with PatientSubSession do
    if PatientIsDead and (DOD < DateIn) then
      begin
        Result := False;
        if MsgDlg('According to Vista records, the veteran was deceased before the ' +
          'date of admission and/or interview. Veteran died: ' + DateToStr(DOD) +
          '. ASI cannot be saved with this date.'#13 +
          'Do you wish to cancel this ASI?',
          'Mental Health Assistant', mtError, [mbYes, mbNo], 0) = mrYes then
        begin
          Close;
          Exit;
        end; //if MsgDlg
      end; //if PatientIs Dead and (DOD < DateIn)
end;

procedure TfrmASI.dtpG4Exit(Sender: TObject);
begin
  Fresh := True;
  // if Vet is dead, make sure ASI is not completed after the death
  if not CheckDeathDate(dtpG4.DateTime) then
    ActiveControl := dtpG4;
end;

procedure TfrmASI.dtpG5Exit(Sender: TObject);
begin
  Fresh := True;
  GetLastASI;
  // if Vet is dead, make sure ASI is not completed after the death
  if not CheckDeathDate(dtpG5.DateTime) then
    ActiveControl := dtpG5;
end;


//---------------------------------------------------------------------
//         Radio Group: Exit
//---------------------------------------------------------------------


procedure TfrmASI.rgrpG19Exit(Sender: TObject);
begin
  if ((rgrpG19.ItemIndex = 0) or (rgrpG19.ItemIndex = 6 {X}) or
    (rgrpG19.ItemIndex = 7 {N})) then
  begin
    edtG20.Text := 'N';
    edtG20.Enabled := false;
    edtG19e.Text := '';
    edtG19e.Enabled := false;
    SpnG20.Enabled := false;
    lblG20.Enabled := false;
    pgctrlASI.ActivePage := TabMedical;
    ActiveControl := edtM1;
  end;
  if rgrpG19.ItemIndex = 5 then
    ActiveControl := edtG19e;
  if ((rgrpG19.ItemIndex = 1) or (rgrpG19.ItemIndex = 2) or
    (rgrpG19.ItemIndex = 3) or (rgrpG19.ItemIndex = 4) or
    (rgrpG19.ItemIndex = 5)) then
  begin
    edtG20.Enabled := true;
    SpnG20.Enabled := true;
    lblG20.Enabled := true;
  end;
end;

procedure TfrmASI.rgrpM7Exit(Sender: TObject);
begin
//  bHighLightOff(sender);
  if not CrossCheckItems then
    CrossCheck('M7', sender);
end;

procedure TfrmASI.rgrpM8Exit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('M8', sender);
end;

procedure TfrmASI.rgrpE20Exit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('E20', sender);
end;

procedure TfrmASI.rgrpD14Exit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('D14', sender);
end;


procedure TfrmASI.rgrpD30Exit(Sender: TObject);
begin
  if not CrossCheckItems then
  begin
    CrossCheck('D30a', sender);
    CrossCheck('D30b', sender);
  end;
end;

procedure TfrmASI.rgrpD31Exit(Sender: TObject);
begin
  if not CrossCheckItems then
  begin
    CrossCheck('D31a', sender);
    CrossCheck('D31b', sender);
  end;
end;

procedure TfrmASI.rgrpL24Exit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('L24', sender);
end;

procedure TfrmASI.rgrpL25Exit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('L25', sender);
end;

procedure TfrmASI.rgrpF34Exit(Sender: TObject);
begin
  if not CrossCheckItems then
  begin
    CrossCheck('F34a', sender);
    CrossCheck('F34b', sender);
  end;
end;

procedure TfrmASI.rgrpF35Exit(Sender: TObject);
begin
  if not CrossCheckItems then
  begin
    CrossCheck('F35a', sender);
    CrossCheck('F35b', sender);
  end;
end;

procedure TfrmASI.rgrpP10lExit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('P10', sender);
end;

procedure TfrmASI.rgrpP9lExit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('P9b', sender);
end;

procedure TfrmASI.rgrpP2Exit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('P2', sender);
end;

procedure TfrmASI.rgrpP9Exit(Sender: TObject);
begin
  if not CrossCheckItems then
    CrossCheck('P9a', sender);
end;

procedure TfrmASI.rgrpP13Exit(Sender: TObject);
begin
  if not CrossCheckItems then
  begin
    CrossCheck('P13a', sender);
    CrossCheck('P13b', sender);
  end;
end;

function TfrmASI.ConductCrossChecks(sender: TObject): boolean;
// This will loop thru the items in the CrossCheckList.
// Some may have been checked already while the user inputed data.
begin
  result := false;

  if initialize then Exit;

  // Bail out if the list is empty
  if ((CrossCheckItems) and (Length(Trim(CrossCheckList)) = 0)) then
  begin
    result := true;
    CrossCheckItems := false;
    CrossChecksCompleted := true;
    lblHint.Caption := 'Data verified, cross checks done...';
    lblVerify.Caption := '';
    pnlVerify.Height := 0;
       // Save, get signature
    if not SignAndExit then
      Exit;
      // Update the main form
    SetupMainForm;
    Close;
    exit;
  end;

  // Check next item on list (list maybe reduced by prior checks)
  CrossCheck(GetPiece(CrossCheckList, ',', 1), sender);
end;

procedure TfrmASI.CrossCheck(WhichOne: string; sender: TObject);
// There are 48 cross checks. Most users will trip-the-wire as go through
// the ASI, taking each item in order. The exit test for critial items can be defeated
// by non-linear users, so a second test is done before the ASI is saved. Only items
// that have not been reviewed will be tested in the second review.
var
  arrests,
    i: integer;
begin
  if WhichOne = '' then
    begin
      CrossCheckItems := false;
      exit;
    end;

  i := Pos(WhichOne, CrossCheckList);
  initialize := true;

  if CrossCheckItems then
    begin
    // Make the Verify Panel look like a button.
    with pnlVerify do
      begin
        Height := 60;
        BevelWidth := 3;
        BorderStyle := bsSingle;
      end;
      lblVerify.Color := clMaroon;
      lblVerify.Font.Color := clWhite;
    end
    else
    begin
      lblVerify.Caption := '';
      pnlVerify.Height := 0;
    end;  

  // 1. G20
  if (WhichOne = 'G20') and
    ((rgrpG19.ItemIndex = 1) or (rgrpG19.ItemIndex = 2) or
    (rgrpG19.ItemIndex = 3) or (rgrpG19.ItemIndex = 4) or
    (rgrpG19.ItemIndex = 5)) and ((edtG20.Text = '') or
    (edtG20.text = '0') or (edtG20.text = 'X') or (edtG20.text = 'N')) then
  begin
    // remove this item from the list
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'In question G19 you recorded that the veteran has ' +
      'been in a controlled environment in the past 30 days, thus question ' +
      'G20 should be greater than 0.'
      + #13 + #10 + 'Click here to continue cross checks.';
    MsgDlg('In question G19 you recorded that the veteran has ' +
      'been in a controlled environment in the past 30 days, thus question ' +
      'G20 should be greater than 0.',
      'Addiction Severity Index', mtConfirmation, [mbOK], 0);
      //Left, Top, Height, Width, 0);
    pgctrlASI.ActivePage := TabGeneral;
    edtG20.Text := '';
    ActiveControl := edtG20;
    initialize := False;
    exit;
  end;

  // 2. M1
  if (WhichOne = 'M1') and
    ((Length(edtM1.text) > 0) and (edtM1.text[1] in ['0', 'X', 'N'])) and
    (rgrpG19.ItemIndex = 3) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'You recorded in the general information section in G19, that ' +
      'the veteran had been hospitialized for medical problems in the past ' +
      '30 days. This hospitialization would usually be coded in question M1. ' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('You recorded in the general information section in G19, that ' +
      'the veteran had been hospitialized for medical problems in the past ' +
      '30 days. This hospitialization would usually be coded in question M1. ' +
      'Do you wish to change M1?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabMedical;
      edtM1.Text := '';
      ActiveControl := edtM1;
      initialize := False;
      exit;
    end;
  end;

  // 3. M2
  if (WhichOne = 'M2') and (rgrpG8.ItemIndex = 0 { Full}) and
    ((Length(edtM2y.text) > 0) and (edtM2y.text[1] in ['1'..'9'])) and
    ((Length(edtM2m.text) > 0) and (edtM2m.text[1] in ['1'..'9'])) and
    (rgrpG19.ItemIndex = 3) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'You recorded in the general information section (G19), that ' +
      'the veteran had been hospitialized this month for medical problems. ' +
      'The correct coding in M2 is usually "0" "0" in this case.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('You recorded in the general information section (G19), that ' +
      'the veteran had been hospitialized this month for medical problems. ' +
      'The correct coding in M2 is usually "0" "0" in this case. ' +
      'Do you want to recode M2?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabMedical;
      edtM2y.Text := '';
      edtM2m.Text := '';
      ActiveControl := edtM2y;
      initialize := False;
      exit;
    end;
  end;

  // 4. M7
  if (WhichOne = 'M7') and
    ((length(EdtM6.text) > 0) and (edtM6.text[1] in ['1'..'9'])) and
    (rgrpM7.ItemIndex = 0) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'In question M6, you recorded that the veteran experienced ' +
      'some medical problems in the past 30 days. If this were true, then we ' +
      'would expect that the veteran would be at least slightly bothered by these ' +
      'problems.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('In question M6, you recorded that the veteran experienced ' +
      'some medical problems in the past 30 days. If this were true, then we ' +
      'would expect that the veteran would be at least slightly bothered by these ' +
      'problems. Do you want to change M7?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabMedical;
      rgrpM7.itemIndex := -1;
      ActiveControl := rgrpM7;
      initialize := False;
      exit;
    end;
  end;

  // 5. M8
  if (WhichOne = 'M8') and
    ((Length(edtM6.text) > 0) and (edtM6.text[1] in ['0', 'N'])) and
    ((rgrpM7.ItemIndex <> 0) or (rgrpM8.ItemIndex <> 0)) then
  begin
    lblVerify.caption := 'In question M6, you recorded that the veteran experienced no ' +
      'medical problems in the past 30 days, since they report being troubled ' +
      'or wanting treatment (M7 or M8), it''s fair to expect that they had some ' +
      'problem days.' +
      #13 + #10 + 'Click here to continue cross checks.';
    MsgDlg('In question M6, you recorded that the veteran experienced no ' +
      'medical problems in the past 30 days, since they report being troubled ' +
      'or wanting treatment (M7 or M8), it''s fair to expect that they had some ' +
      'problem days. Go back to M6 and identify the number of days the problem ' +
      'has bothered them.',
      'Addiction Severity Index', mtConfirmation, [mbOK],
      0);
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    pgctrlASI.ActivePage := TabMedical;
    edtM6.Text := '';
    ActiveControl := edtM6;
    initialize := False;
    exit;
  end;

  // 6. E1y
  if (WhichOne = 'E1y') and (rgrpG8.ItemIndex <> 2) {Followup can be zero} and
    ((length(EdtE1y.text) > 0) and (edtE1y.text[1] in ['0'..'9'])) then
  begin
    if (StrToInt(edtE1y.text) < 4) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption := 'You are reporting that the veteran has had less than 4 years ' +
        'of education, this is rare. Please review this, did you include home ' +
        'schooling, grade school, etc.?' +
        #13 + #10 + 'Click here to continue cross checks.';
      if MsgDlg('You are reporting that the veteran has had less than 4 years ' +
        'of education, this is rare. Please review this, did you include home ' +
        'schooling, grade school, etc.? Do you want to change E1?',
        'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
      begin
        Initialize := False;
        if not CrossCheckItems then
          exit;
      end
      else
      begin
        pgctrlASI.ActivePage := TabEmploy;
        edtE1y.Text := '';
        ActiveControl := edtE1y;
        initialize := False;
        exit;
      end;
    end;
  end;

  // 7. E5 is a forced change. set in code

  // 8. E9 is hard coded

  // 9. E12a
  if (WhichOne = 'E12a') and
    ((Length(edtE11.text) > 0) and (Length(edtE12.text) > 0)) and
    (edtE11.text[1] in ['0', 'N']) and (edtE12.text[1] in ['1'..'9']) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'In the last question, E11, you recorded that the veteran ' +
      'was not paid for working at all in the past month. If this is the case, ' +
      'E12 is generally $0.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('In the last question, E11, you recorded that the veteran ' +
      'was not paid for working at all in the past month. If this is the case, ' +
      'E12 is generally $0. Do you wish to change E12?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabEmploy;
      edtE12.Text := '0';
      ActiveControl := edtE12;
      initialize := False;
      exit;
    end;
  end;

  // 10. E12b
  if (WhichOne = 'E12b') and
    ((Length(edtE11.text) > 0) and (edtE11.text[1] in ['1'..'9'])) and
    ((Length(edtE12.text) > 0) and (edtE12.text[1] in ['0', 'N'])) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'In the question E11, you recorded that the veteran ' +
      'was paid for working this month. If this is the case, ' +
      'E12 is generally not $0.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('In the question E11, you recorded that the veteran ' +
      'was paid for working this month. If this is the case, ' +
      'E12 is generally not $0. Do you wish to change E12?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabEmploy;
      edtE12.Text := '0';
      ActiveControl := edtE12;
      initialize := False;
      exit;
    end;
  end;

  // 11. E15
  if (WhichOne = 'E15') and
    ((Length(edtE15.text) > 0) and (edtE15.text[1] in ['0', 'N'])) and
    (rgrpM5.ItemIndex = 1) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'You recorded in question M5, that the veteran receives a pension ' +
      'for a medical problem. This income is generally recorded in E15 unless ' +
      'they did not receive any cash this month. ' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('You recorded in question M5, that the veteran receives a pension ' +
      'for a medical problem. This income is generally recorded in E15 unless ' +
      'they did not receive any cash this month. Do you wish to change E15?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabEmploy;
      edtE15.Text := '';
      ActiveControl := edtE15;
      initialize := False;
      exit;
    end;
  end;

  // 12. E20
  if (WhichOne = 'E20') and (rgrpG8.ItemIndex <> 1 {Full & Followup}) and
    ((Length(edtE19.text) > 0) and (edtE19.text[1] in ['1'..'9'])) and
    (rgrpE20.ItemIndex = 0) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'In question E19, you recorded that the veteran experienced some employment ' +
      'problems in the past 30 days. If this were true, then we would expect that the ' +
      'veteran would be at least slightly bothered by these problems.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('In question E19, you recorded that the veteran experienced some employment ' +
      'problems in the past 30 days. If this were true, then we would expect that the ' +
      'veteran would be at least slightly bothered by these problems. ' +
      'Do you wish to change E20?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabEmploy;
      rgrpE20.ItemIndex := -1;
      ActiveControl := rgrpE20;
      initialize := False;
      exit;
    end;
  end;

  // 13. D2d
  if (WhichOne = 'D2d') and
    ((Length(edtD1d.text) > 0) and (edtD1d.text[1] in ['0'..'9'])) and
    ((length(edtD2d.text) > 0) and (edtD2d.text[1] in ['0'..'9'])) then
  begin
    if StrToInt(edtD1d.text) < StrToInt(edtD2d.text) then
    begin
      lblVerify.caption := 'You recorded that the veteran drank "to intoxication" ' +
        '(D2) more days than the total days drinking any alcohol at all ' +
        '(D1). Probe and recode D1. Remember D2 is a sub-set of D1.' +
        #13 + #10 + 'Click here to continue cross checks.';
      MsgDlg('You recorded that the veteran drank "to intoxication" ' +
        '(D2) more days than the total days drinking any alcohol at all ' +
        '(D1). Probe and recode D1. Remember D2 is a sub-set of D1.',
        'Addiction Severity Index', mtConfirmation, [mbOK],
        0);
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      pgctrlASI.ActivePage := TabDrug;
      edtD1d.Text := '';
      ActiveControl := edtD1d;
      initialize := False;
      exit;
    end;
  end;

  // 14. D2y
  if (WhichOne = 'D2y') and (rgrpG8.ItemIndex <> 2 {Full & Lite}) and
    ((Length(edtD1y.text) > 0) and (edtD1y.text[1] in ['0'..'9'])) and
    ((Length(edtD2y.text) > 0) and (edtD2y.text[1] in ['0'..'9'])) then
  begin
    if StrToInt(edtD1y.text) < StrToInt(edtD2y.text) then
    begin
      lblVerify.caption := 'You recorded that the veteran drank "to intoxication" ' +
        '(D2) more years than the total number of years drinking any alcohol at all ' +
        '(D1). Remember D2 is a sub-set of D1.' +
        #13 + #10 + 'Click here to continue cross checks.';
      MsgDlg('You recorded that the veteran drank "to intoxication" ' +
        '(D2) more years than the total number of years drinking any alcohol at all ' +
        '(D1). Probe and recode D1. Remember D2 is a sub-set of D1.',
        'Addiction Severity Index', mtConfirmation, [mbOK],
        0);
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      pgctrlASI.ActivePage := TabDrug;
      edtD1y.Text := '';
      ActiveControl := edtD1y;
      initialize := False;
      exit;
    end;
  end;

  // 15. D12d
  if (WhichOne = 'D12d') and
    ((Length(edtG20.text) > 0) and (edtG20.Text[1] in ['1'..'9'])) and
    ((edtD1d.Text = '30') or
    (edtD2d.Text = '30') or (edtD6d.Text = '30') or (edtD10d.Text = '30') or
    (edtD3d.Text = '30') or (edtD7d.Text = '30') or (edtD11d.Text = '30') or
    (edtD4d.Text = '30') or (edtD8d.Text = '30') or (edtD12d.Text = '30') or
    (edtD5d.Text = '30') or (edtD9d.Text = '30')) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'You recorded in the general information section that the veteran ' +
      'had been in a controlled environment in the past month, yet they used ' +
      'either drugs or alcohol every day. Please review this.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('You recorded in the general information section that the veteran ' +
      'had been in a controlled environment in the past month, yet they used ' +
      'either drugs or alcohol every day. Please review this. Do you wish to ' +
      'change any information in the drug/alcohol grid?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabDrug;
      //ActiveControl := edtD1d;
      Delete(CrossCheckList, Pos('1,', CrossCheckList), 2);
      initialize := false;
      exit;
    end;
  end;

  // 16. D14
  if (WhichOne = 'D14') and (rgrpG8.ItemIndex <> 1 {Full & Followup}) and
    ((length(edtD1d.text) > 0) and (edtD1d.text[1] in ['1'..'9'])) and
    ((rgrpD14.ItemIndex = 3) or
    (rgrpD14.ItemIndex = 4) or (rgrpD14.ItemIndex = 5) or (rgrpD14.ItemIndex = 6) or
    (rgrpD14.ItemIndex = 7) or (rgrpD14.ItemIndex = 8) or (rgrpD14.ItemIndex = 9) or
    (rgrpD14.ItemIndex = 10) or (rgrpD14.ItemIndex = 11) or (rgrpD14.ItemIndex = 12) or
    (rgrpD14.ItemIndex = 14)) then
    if StrToInt(edtD1d.text) > 15 then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption := 'You report that the veteran''s problem does not include alcohol, ' +
        'however, the veteran used alcohol at least 15 days in the past month. Please ' +
        'review this and consider option "Alcohol + drug(s)" for question D14.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if MsgDlg('You report that the veteran''s problem does not include alcohol, ' +
        'however, the veteran used alcohol at least 15 days in the past month. Please ' +
        'review this and consider option "Alcohol + drug(s)" for question D14. Do you ' +
        'want to change D14?',
        'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
      begin
        Initialize := False;
        if not CrossCheckItems then
          exit;
      end
      else
      begin
        pgctrlASI.ActivePage := TabDrug;
        rgrpD14.ItemIndex := -1;
        ActiveControl := rgrpD14;
        initialize := False;
        exit;
      end;
    end;

  // 17. D16
  if ((WhichOne = 'D16') and (rgrpG8.ItemIndex = 0 {Full}) and
    (edtD16.text = '0') and
    (((Length(edtD1d.text) > 0) and (edtD1d.Text[1] in ['1'..'9'])) or
    ((Length(edtD2d.text) > 0) and (edtD2d.Text[1] in ['1'..'9'])) or
    ((Length(edtD3d.text) > 0) and (edtD3d.Text[1] in ['1'..'9'])) or
    ((Length(edtD4d.text) > 0) and (edtD4d.Text[1] in ['1'..'9'])) or
    ((Length(edtD5d.text) > 0) and (edtD5d.Text[1] in ['1'..'9'])) or
    ((Length(edtD6d.text) > 0) and (edtD6d.Text[1] in ['1'..'9'])) or
    ((Length(edtD7d.text) > 0) and (edtD7d.Text[1] in ['1'..'9'])) or
    ((Length(edtD8d.text) > 0) and (edtD8d.Text[1] in ['1'..'9'])) or
    ((Length(edtD9d.text) > 0) and (edtD9d.Text[1] in ['1'..'9'])) or
    ((Length(edtD10d.text) > 0) and (edtD10d.Text[1] in ['1'..'9'])) or
    ((Length(edtD11d.text) > 0) and (edtD11d.Text[1] in ['1'..'9'])) or
    ((Length(edtD12d.text) > 0) and (edtD12d.Text[1] in ['1'..'9'])))) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'You recorded that the veteran is "still sober," however ' +
      'drug or alcohol use in the past 30 days is documented in the drug and ' +
      'alcohol grid. Please review this.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('You recorded that the veteran is "still sober," however ' +
      'drug or alcohol use in the past 30 days is documented in the drug and ' +
      'alcohol grid. Please review this. Do you want to change D16?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabDrug;
      edtD16.Text := '';
      ActiveControl := edtD16;
      initialize := False;
      exit;
    end;
  end;

  // 18. D17
  if (WhichOne = 'D17') and (rgrpG8.ItemIndex <> 1 {Full & Followup}) and
    ((length(edtD17.text) > 0) and (edtD17.Text[1] in ['1'..'9'])) then
    if StrToInt(edtD17.text) > 5 then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption := 'You recorded more than 5 episodes of DT''s for the veteran. ' +
        'This is extremely rare, please review the definition of DT''s if you ' +
        'are unsure.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if MsgDlg('You recorded more than 5 episodes of DT''s for the veteran. ' +
        'This is extremely rare, please review the definition of DT''s if you ' +
        'are unsure. Do you want to change D17?',
        'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
      begin
        Initialize := False;
        if not CrossCheckItems then
          exit;
      end
      else
      begin
        pgctrlASI.ActivePage := TabDrug;
        edtD17.Text := '';
        ActiveControl := edtD17;
        initialize := False;
        exit;
      end;
    end;

  // 19. D21a
  if (WhichOne = 'D21a') and (rgrpG8.ItemIndex <> 1 {Full & Followup}) and
    ((Length(edtD19.text) > 0) and (edtD19.text[1] in ['0', 'N'])) and
    ((Length(edtD21.text) > 0) and (edtD21.text[1] in ['1'..'9'])) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'You recorded that the veteran never had any treatment for ' +
      'alcohol abuse, so the number of detox treatments is not applicable. ' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('You recorded that the veteran never had any treatment for ' +
      'alcohol abuse, so the number of detox treatments is not applicable. ' +
      'Do you want to recode D19, the total number of treatments received?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabDrug;
      edtD19.Text := '';
      ActiveControl := edtD19;
      initialize := False;
      exit;
    end;
  end;

  // 20.  D21b
  if (WhichOne = 'D21b') and (rgrpG8.ItemIndex <> 1 {Full & Followup}) and
    ((Length(edtD19.text) > 0) and (edtD19.text[1] in ['0'..'9'])) and
    ((Length(edtD21.text) > 0) and (edtD21.text[1] in ['0'..'9'])) then
    if StrToInt(edtD19.text) < StrToInt(edtD21.text) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption := 'You recorded that the veteran had more detox treatments than ' +
        'the total number of treatments received for alcohol abuse. Remember ' +
        'D21 is a sub-set of D19.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if MsgDlg('You recorded that the veteran had more detox treatments than ' +
        'the total number of treatments received for alcohol abuse. Remember ' +
        'D21 is a sub-set of D19. Do you want to recode D19?',
        'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
      begin
        Initialize := False;
        if not CrossCheckItems then
          exit;
      end
      else
      begin
        pgctrlASI.ActivePage := TabDrug;
        edtD19.Text := '';
        ActiveControl := edtD19;
        initialize := False;
        exit;
      end;
    end;

  // 21.   D22a
  if (WhichOne = 'D22a') and (rgrpG8.ItemIndex <> 1 {Full & Followup}) and
    ((Length(edtD20.text) > 0) and (edtD20.text[1] in ['1'..'9'])) and
    ((Length(edtD22.text) > 0) and (edtD22.text[1] in ['1'..'9'])) then
    if StrToInt(edtD22.text) > StrToInt(edtD20.text) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption := 'You recorded that the veteran had more detox treatments than the ' +
        'total number of treatments received for drug abuse. Remember D22 is a sub-set ' +
        'of D20.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if MsgDlg('You recorded that the veteran had more detox treatments than the ' +
        'total number of treatments received for drug abuse. Remember D22 is a sub-set ' +
        'of D20. Do you want to recode D20?',
        'Addiction Severity Index', mtConfirmation, [mbYes, mbNo], 0) = mrNo then
      begin
        Initialize := False;
        if not CrossCheckItems then
          exit;
      end
      else
      begin
        pgctrlASI.ActivePage := TabDrug;
        edtD20.Text := '';
        ActiveControl := edtD20;
        initialize := False;
        exit;
      end;
    end;

  // 22. D22b
  if (WhichOne = 'D22b') and (rgrpG8.ItemIndex <> 1 {Full & Followup}) and
    ((Length(edtD20.text) > 0) and (edtD20.text[1] in ['0', 'N'])) and
    ((Length(edtD22.text) > 0) and (edtD22.text[1] in ['1'..'9'])) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'You recorded that the veteran never had any treatments for drug ' +
      'abuse, so the number of detox treatments is not applicable.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('You recorded that the veteran never had any treatments for drug ' +
      'abuse, so the number of detox treatments is not applicable. Do you want ' +
      'to recode D20, the total number of treatments received?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabDrug;
      edtD20.Text := '';
      ActiveControl := edtD20;
      initialize := False;
      exit;
    end;
  end;

  // 23. D23a
  if (WhichOne = 'D23a') and
    ((Length(edtD1d.text) > 0) and (edtD1d.text[1] in ['0', 'N'])) and
    ((Length(edtD23.text) > 0) and (edtD23.text[1] in ['1'..'9'])) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'You recorded no days of drinking in the past 30, but recorded the ' +
      'veteran has spent money on alcohol. This is unlikely. Please review. ' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('You recorded no days of drinking in the past 30, but recorded the ' +
      'veteran has spent money on alcohol. This is unlikely. Please review. Do you ' +
      'want to change D23?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabDrug;
      edtD23.Text := '';
      ActiveControl := edtD23;
      initialize := False;
      exit;
    end;
  end;

  // 24. D23b
  if (WhichOne = 'D23b') and
    ((Length(edtD1d.text) > 0) and (edtD1d.text[1] in ['1'..'9'])) and
    ((Length(edtD2d.text) > 0) and (edtD2d.text[1] in ['1'..'9'])) and
    ((Length(edtD23.text) > 0) and (edtD23.text[1] in ['0', 'X', 'N'])) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'You recorded the veteran as drinking in the past 30 days (D1 and ' +
      'D2), but no money was spent on alcohol. Please review this.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('You recorded the veteran as drinking in the past 30 days (D1 and ' +
      'D2), but no money was spent on alcohol. Please review this, do you wish ' +
      'to change D23?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabDrug;
      edtD23.Text := '';
      ActiveControl := edtD23;
      initialize := False;
      exit;
    end;
  end;

  // 25. D24a
  if ((WhichOne = 'D24a') and
    ((Length(edtD24.text) > 0) and (edtD24.text[1] in ['1'..'9'])) and
    (((Length(edtD3d.text) > 0) and (edtD3d.text[1] in ['0', 'X', 'N'])) and
    ((Length(edtD4d.text) > 0) and (edtD4d.text[1] in ['0', 'X', 'N'])) and
    ((Length(edtD5d.text) > 0) and (edtD5d.text[1] in ['0', 'X', 'N'])) and
    ((Length(edtD6d.text) > 0) and (edtD6d.text[1] in ['0', 'X', 'N'])) and
    ((Length(edtD7d.text) > 0) and (edtD7d.text[1] in ['0', 'X', 'N'])) and
    ((Length(edtD8d.text) > 0) and (edtD8d.text[1] in ['0', 'X', 'N'])) and
    ((Length(edtD9d.text) > 0) and (edtD9d.text[1] in ['0', 'X', 'N'])) and
    ((Length(edtD10d.text) > 0) and (edtD10d.text[1] in ['0', 'X', 'N'])) and
    ((Length(edtD11d.text) > 0) and (edtD11d.text[1] in ['0', 'X', 'N'])) and
    ((Length(edtD12d.text) > 0) and (edtD12d.text[1] in ['0', 'X', 'N'])))) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'You recorded zero days of drug use in the past 30, but the veteran ' +
      'acknowledges spending money on drugs this month. Please review, and code ' +
      'those drugs used in the past 30 days.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('You recorded zero days of drug use in the past 30, but the veteran ' +
      'acknowledges spending money on drugs this month. Please review, and code ' +
      'those drugs used in the past 30 days. Do you wish to change number of days ' +
      'of drug use in the drug grid?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabDrug;
      //ActiveControl := edtD3d;
      initialize := false;
      exit;
    end;
  end;

  // 26. D24b
  if ((WhichOne = 'D24b') and
    ((Length(edtD24.text) > 0) and (edtD24.text[1] in ['0', 'X', 'N'])) and
    (((Length(edtD3d.text) > 0) and (edtD3d.text[1] in ['1'..'9'])) or
    ((Length(edtD4d.text) > 0) and (edtD4d.text[1] in ['1'..'9'])) or
    ((Length(edtD5d.text) > 0) and (edtD5d.text[1] in ['1'..'9'])) or
    ((Length(edtD6d.text) > 0) and (edtD6d.text[1] in ['1'..'9'])) or
    ((Length(edtD7d.text) > 0) and (edtD7d.text[1] in ['1'..'9'])) or
    ((Length(edtD8d.text) > 0) and (edtD8d.text[1] in ['1'..'9'])) or
    ((Length(edtD9d.text) > 0) and (edtD9d.text[1] in ['1'..'9'])) or
    ((Length(edtD10d.text) > 0) and (edtD10d.text[1] in ['1'..'9'])) or
    ((Length(edtD11d.text) > 0) and (edtD11d.text[1] in ['1'..'9'])) or
    ((Length(edtD12d.text) > 0) and (edtD12d.text[1] in ['1'..'9'])))) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'You recorded some days of drug use in the past 30 days, but no ' +
      'money was spent on drugs. Please review this.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('You recorded some days of drug use in the past 30 days, but no ' +
      'money was spent on drugs. Please review this. Do you want to change D24?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabDrug;
      edtD24.text := '0';
      ActiveControl := edtD24;
      initialize := False;
      exit;
    end;
  end;

  // 27. D30a
  if (WhichOne = 'D30a') and
    ((length(edtD26.text) > 0) and (edtD26.text[1] in ['1'..'9'])) and
    ((rgrpD28.ItemIndex = 0) or (rgrpD30.ItemIndex = 0)) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'In question D26, you recorded that the veteran had ' +
      'experienced some days with alcohol problems in the past 30 days. If this ' +
      'is true, then we would expect the veteran would be at least slightly ' +
      'bothered or slightly in need of treatment for these problems.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('In question D26, you recorded that the veteran had ' +
      'experienced some days with alcohol problems in the past 30 days. If this ' +
      'is true, then we would expect the veteran would be at least slightly ' +
      'bothered or slightly in need of treatment for these problems. Do you want ' +
      'to recode D28 and/or D30?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabDrug;
      rgrpD28.ItemIndex := -1;
      //ActiveControl := rgrpD28;
      initialize := false;
      exit;
    end;
  end;

  // 28. D30b
  if (WhichOne = 'D30b') and
    ((length(edtD26.text) > 0) and (edtD26.text[1] in ['0', 'N'])) and
    ((rgrpD28.ItemIndex <> 0) or (rgrpD30.ItemIndex <> 0)) then
  begin
    lblVerify.caption := 'In question D26, you recorded that the veteran had ' +
      'experienced no alcohol problems in the past 30 days. Since they report ' +
      'being troubled or wanting treatment, it''s fair to expect that they ' +
      'had some problem days.' +
      #13 + #10 + 'Click here to continue cross checks.';
    MsgDlg('In question D26, you recorded that the veteran had ' +
      'experienced no alcohol problems in the past 30 days. Since they report ' +
      'being troubled or wanting treatment, it''s fair to expect that they ' +
      'had some problem days. Go back to D26 and identify the number of days ' +
      'the problem has bothered them.',
      'Addiction Severity Index', mtConfirmation, [mbOK],
      0);
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    pgctrlASI.ActivePage := TabDrug;
    edtD26.text := '';
    ActiveControl := edtD26;
    initialize := False;
    exit;
  end;

  // 29. D31a
  if (WhichOne = 'D31a') and
    ((Length(edtD27.text) > 0) and (edtD27.text[1] in ['1'..'9'])) and
    ((rgrpD29.ItemIndex = 0) or (rgrpD31.ItemIndex = 0)) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'In question D27, you recorded that the veteran had ' +
      'experienced some days with drug problems in the past 30 days. If this ' +
      'is true, then we would expect the veteran would be at least slightly ' +
      'bothered or slightly in need of treatment for these problems.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('In question D27, you recorded that the veteran had ' +
      'experienced some days with drug problems in the past 30 days. If this ' +
      'is true, then we would expect the veteran would be at least slightly ' +
      'bothered or slightly in need of treatment for these problems. Do you want ' +
      'to recode D31?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabDrug;
      rgrpD31.ItemIndex := -1;
      ActiveControl := rgrpD31;
      initialize := False;
      exit;
    end;
  end;

  // 30. D31b
  if (WhichOne = 'D31b') and
    ((Length(edtD27.text) > 0) and (edtD27.text[1] in ['0', 'N'])) and
    ((rgrpD29.ItemIndex <> 0) or (rgrpD31.ItemIndex <> 0)) then
  begin
    lblVerify.caption := 'In question D27, you recorded that the veteran had ' +
      'experienced no drug problems in the past 30 days. Since they report ' +
      'being troubled or wanting treatment, it''s fair to expect that they ' +
      'had some problem days.' +
      #13 + #10 + 'Click here to continue cross checks.';
    MsgDlg('In question D27, you recorded that the veteran had ' +
      'experienced no drug problems in the past 30 days. Since they report ' +
      'being troubled or wanting treatment, it''s fair to expect that they ' +
      'had some problem days. Go back to D27 and identify the number of days ' +
      'the problem has bothered them.',
      'Addiction Severity Index', mtConfirmation, [mbOK],
      0);
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    pgctrlASI.ActivePage := TabDrug;
    edtD27.text := '';
    ActiveControl := edtD27;
    initialize := False;
    exit;
  end;

  // 31. L17
  if (WhichOne = 'L17') then
  begin
    Arrests := 0;
    if ((length(edtL3.text) > 0) and (edtL3.text[1] in ['0'..'9'])) then
        // use zero in range in case the user types "04"
      Arrests := arrests + StrToInt(edtL3.text);
    if ((length(edtL4.text) > 0) and (edtL4.text[1] in ['0'..'9'])) then
      Arrests := arrests + StrToInt(edtL4.text);
    if ((length(edtL5.text) > 0) and (edtL5.text[1] in ['0'..'9'])) then
      Arrests := arrests + StrToInt(edtL5.text);
    if ((length(edtL6.text) > 0) and (edtL6.text[1] in ['0'..'9'])) then
      Arrests := arrests + StrToInt(edtL6.text);
    if ((length(edtL7.text) > 0) and (edtL7.text[1] in ['0'..'9'])) then
      Arrests := arrests + StrToInt(edtL7.text);
    if ((length(edtL8.text) > 0) and (edtL8.text[1] in ['0'..'9'])) then
      Arrests := arrests + StrToInt(edtL8.text);
    if ((length(edtL9.text) > 0) and (edtL9.text[1] in ['0'..'9'])) then
      Arrests := arrests + StrToInt(edtL9.text);
    if ((length(edtL10.text) > 0) and (edtL10.text[1] in ['0'..'9'])) then
      Arrests := arrests + StrToInt(edtL10.text);
    if ((length(edtL11.text) > 0) and (edtL11.text[1] in ['0'..'9'])) then
      Arrests := arrests + StrToInt(edtL11.text);
    if ((length(edtL12.text) > 0) and (edtL12.text[1] in ['0'..'9'])) then
      Arrests := arrests + StrToInt(edtL12.text);
    if ((length(edtL13.text) > 0) and (edtL13.text[1] in ['0'..'9'])) then
      Arrests := arrests + StrToInt(edtL13.text);
    if ((length(edtL14.text) > 0) and (edtL14.text[1] in ['0'..'9'])) then
      Arrests := arrests + StrToInt(edtL14.text);
    if ((length(edtL15.text) > 0) and (edtL15.text[1] in ['0'..'9'])) then
      Arrests := arrests + StrToInt(edtL15.text);
    if ((length(edtL16.text) > 0) and (edtL16.text[1] in ['0'..'9'])) then
      Arrests := arrests + StrToInt(edtL16.text);

    if (Length(edtL17.text) > 0) then
      if edtL17.text[1] in ['0'..'9'] then
        if StrToInt(edtL17.text) > arrests then
        begin
          lblVerify.caption := 'You recorded the veteran had more convictions than the total ' +
            'number of times they were arrested and charged (L3 to L16). This is ' +
            'unusual.' +
            #13 + #10 + 'Click here to continue cross checks.';
          MsgDlg('You recorded the veteran had more convictions than the total ' +
            'number of times they were arrested and charged (L3 to L16). This is ' +
            'unusual. Reconsider the response in L17.',
            'Addiction Severity Index', mtConfirmation, [mbOK],
            0);
          Delete(CrossCheckList, i, Length(WhichOne) + 1);
          pgctrlASI.ActivePage := TabLegal;
          edtL17.text := '';
          //ActiveControl := edtL17;
          initialize := false;
          exit;
        end;
  end;

  // 32. L20
  if (WhichOne = 'L20') and
    ((rgrpL2.ItemIndex = 1) and
    (edtL3.text = '0') and (edtL8.text = '0') and (edtL13.text = '0') and
    (edtL4.text = '0') and (edtL9.text = '0') and (edtL14.text = '0') and
    (edtL5.text = '0') and (edtL10.text = '0') and (edtL15.text = '0') and
    (edtL6.text = '0') and (edtL11.text = '0') and (edtL16.text = '0') and
    (edtL7.text = '0') and (edtL12.text = '0') and (edtL18.text = '0') and
    (edtL19.text = '0') and (edtL20.text = '0')) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'In question L2, you indicated the veteran is on probation ' +
      'or parole. However, no arrests or charges are documented in items L3 through L20.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('In question L2, you indicated the veteran is on probation ' +
      'or parole. However, no arrests or charges are documented in items L3 through L20.' +
      'Do you want to recode any legal charges?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabLegal;
      initialize := false;
      exit;
    end;
  end;

  // 33. L24
  if (WhichOne = 'L24') and
    ((rgrpL24.ItemIndex = 1) and (edtL3.text = '0') and
    (edtL3.text = '0') and (edtL8.text = '0') and (edtL13.text = '0') and
    (edtL4.text = '0') and (edtL9.text = '0') and (edtL14.text = '0') and
    (edtL5.text = '0') and (edtL10.text = '0') and (edtL15.text = '0') and
    (edtL6.text = '0') and (edtL11.text = '0') and (edtL16.text = '0') and
    (edtL7.text = '0') and (edtL12.text = '0') and (edtL18.text = '0') and
    (edtL19.text = '0') and (edtL20.text = '0')) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'You recorded the veteran is awaiting charges, trial or sentence in ' +
      'question L24, but no arrests and/or charges are coded in items L3 through L16 or ' +
      'L18 through L20.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('You recorded the veteran is awaiting charges, trial or sentence in ' +
      'question L24, but no arrests and/or charges are coded in items L3 through L16 or ' +
      'L18 through L20. Do you want to recode any of the charges?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabLegal;
      //ActiveControl := edtL3;
      initialize := false;
      exit;
    end;
  end;

  // 34. L25
  if (WhichOne = 'L25') and
    ((rgrpL24.ItemIndex = 0) and (rgrpL25.ItemIndex < 17)) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'You recorded the veteran was not awaiting charges, trial or sentence in ' +
      'question L24, yet you coded a charge in L25 (which would be not applicable). ' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('You recorded the veteran was not awaiting charges, trial or sentence in ' +
      'question L24, yet you coded a charge in L25 (which would be not applicable). ' +
      'Do you want to recode L24?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabLegal;
      ActiveControl := rgrpL24;
      initialize := False;
      exit;
    end;
  end;

  // 35. L26
  if (WhichOne = 'L26') and
    ((rgrpG19.ItemIndex = 1) and
    ((Length(edtL26.text) > 0) and (edtL26.text[1] in ['0', 'N']))) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'In the general information section, you recorded that the veteran ' +
      'had been in jail in the past 30 days, this is also represented in question ' +
      'L26.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('In the general information section, you recorded that the veteran ' +
      'had been in jail in the past 30 days, this is also represented in question ' +
      'L26. Do you want to change L26?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabLegal;
      edtL26.text := '';
      ActiveControl := edtL26;
      initialize := False;
      exit;
    end;
  end;

  // 36. L27
  if (WhichOne = 'L27') and
    (((Length(edtE17.text) > 0) and (edtE17.text[1] in ['1'..'9'])) and
    (edtL27.text = '0')) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'In the employment section, you recorded that the veteran had illegal ' +
      'income in the past 30 days, this is usually also documented in question L27.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('In the employment section, you recorded that the veteran had illegal ' +
      'income in the past 30 days, this is usually also documented in question L27. Do you ' +
      'wish to change your code in L27?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabLegal;
      edtL27.text := '';
      ActiveControl := edtL27;
      initialize := False;
      exit;
    end;
  end;

  // 37. F34a
  if (WhichOne = 'F34a') and
    (((Length(edtF30.text) > 0) and (edtF30.text[1] in ['1'..'9'])) and
    ((rgrpF32.ItemIndex = 0) or (rgrpF34.ItemIndex = 0))) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'In question F30, you recorded that the veteran had some ' +
      'family conflicts in the past 30 days. If this is true, then we would expect ' +
      'that the veteran would be at least slightly bothered or slightly in need of ' +
      'treatment.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('In question F30, you recorded that the veteran had some ' +
      'family conflicts in the past 30 days. If this is true, then we would expect ' +
      'that the veteran would be at least slightly bothered or slightly in need of ' +
      'treatment. Do you wish to recode F32 or F34?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabSocial;
      //ActiveControl := rgrpF32;
      initialize := false;
      exit;
    end;
  end;

  // 38. F34b
  if (WhichOne = 'F34b') and
    (((Length(edtF30.text) > 0) and (edtF30.text[1] in ['0', 'X', 'N'])) and
    (((rgrpF32.ItemIndex > 0) and (rgrpF32.ItemIndex < 5)) or
    ((rgrpF34.ItemIndex > 0) and (rgrpF34.ItemIndex < 5)))) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'In question F30, you recorded that the veteran had no ' +
      'family conflicts in the past 30 days. Since they report being troubled ' +
      'or wanting treatment, it''s fair to expect that they had some problem days. ' +
      'Do you want to go back to F30 and identify the number of days the problem ' +
      'has bothered them?' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('In question F30, you recorded that the veteran had no ' +
      'family conflicts in the past 30 days. Since they report being troubled ' +
      'or wanting treatment, it''s fair to expect that they had some problem days. ' +
      'Do you want to go back to F30 and identify the number of days the problem ' +
      'has bothered them?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabSocial;
      edtF30.text := '';
      ActiveControl := edtF30;
      initialize := False;
      exit;
    end;
  end;

  // 39. F35a
  if (WhichOne = 'F35a') and (rgrpG8.ItemIndex <> 1 {Full & Followup}) and
    (((Length(edtF31.text) > 0) and (edtF31.text[1] in ['1'..'9'])) and
    ((rgrpF33.ItemIndex = 0) and (rgrpF35.ItemIndex = 0))) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'In question F31, you recorded that the veteran had some ' +
      'conflicts with others in the past 30 days. If this is true, then we would expect ' +
      'that the veteran would be at least slightly bothered or slightly in need of ' +
      'treatment.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('In question F31, you recorded that the veteran had some ' +
      'conflicts with others in the past 30 days. If this is true, then we would expect ' +
      'that the veteran would be at least slightly bothered or slightly in need of ' +
      'treatment. Do you wish to recode F33 or F35?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabSocial;
      //ActiveControl := rgrpF33;
      Initialize := False;
      exit;
    end;
  end;

  // 40. F35b
  if (WhichOne = 'F35b') and (rgrpG8.ItemIndex <> 1 {Full & Followup}) and
    (((Length(edtF31.text) > 0) and (edtF31.text[1] in ['0', 'X', 'N'])) and
    (((rgrpF33.ItemIndex > 0) and (rgrpF33.ItemIndex < 5)) or
    ((rgrpF35.ItemIndex > 0) and (rgrpF35.ItemIndex < 5)))) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'In question F31, you recorded that the veteran had no ' +
      'conflicts with others in the past 30 days. Since they report being troubled ' +
      'or wanting treatment, it''s fair to expect that they had some problem days.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('In question F31, you recorded that the veteran had no ' +
      'conflicts with others in the past 30 days. Since they report being troubled ' +
      'or wanting treatment, it''s fair to expect that they had some problem days. ' +
      'Do you want to go back to F31 and identify the number of days the problem ' +
      'has bothered them?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabSocial;
      edtF31.text := '';
      ActiveControl := edtF31;
      initialize := False;
      exit;
    end;
  end;

  // 41. P9b
  if (WhichOne = 'P9b') and
    ((Length(edtP1a.text) > 0) and (edtP1a.text[1] in ['0', 'N', 'X'])) and
    ((Length(edtP1b.text) > 0) and (edtP1b.text[1] in ['0', 'N', 'X'])) and
    ((rgrpP9.ItemIndex = 1) or (rgrpP9l.ItemIndex = 1)) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'You recorded in question P1 that the veteran has had no inpatient ' +
      'or outpatient treatment for psychiatric problems, yet they have attempted ' +
      'suicide. Please review treatment they may have received for the suicide ' +
      'attempt.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('You recorded in question P1 that the veteran has had no inpatient ' +
      'or outpatient treatment for psychiatric problems, yet they have attempted ' +
      'suicide. Please review treatment they may have received for the suicide ' +
      'attempt. Do you want to recode P1?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabPsych;
      ActiveControl := edtP1a;
      initialize := False;
      exit;
    end;
  end;

   // 42. P10
  if (WhichOne = 'P10') and
    ((Length(edtP1a.text) > 0) and (edtP1a.text[1] in ['0', 'N', 'X'])) and
    ((Length(edtP1b.text) > 0) and (edtP1b.text[1] in ['0', 'N', 'X'])) and
    ((rgrpP10.ItemIndex = 1) or (rgrpP10l.ItemIndex = 1)) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'You recorded in question P1 that the veteran has had no inpatient ' +
      'or outpatient treatment for psychiatric problems, yet they have been prescribed ' +
      'medications for psychiatric problems.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('You recorded in question P1 that the veteran has had no inpatient ' +
      'or outpatient treatment for psychiatric problems, yet they have been prescribed ' +
      'medications for psychiatric problems. Do you want to recode P1?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabPsych;
      //ActiveControl := edtP1a;
      Initialize := False;
      exit;
    end;
  end;

   // 43. P2
  if (WhichOne = 'P2') and
    ((Length(edtE15.text) > 0) and (edtE15.text[1] in ['0', 'N', 'X'])) and
    (rgrpP2.ItemIndex = 1) then
  begin
    lblVerify.caption := 'You recorded that the veteran receives a pension for a psychiatric ' +
      'problem in P2. Unless they did not receive any cash this month, this income ' +
      'is generally recorded in the employment question about pension money received, ' +
      'item E15.' +
      #13 + #10 + 'Click here to continue cross checks.';
    MsgDlg('You recorded that the veteran receives a pension for a psychiatric ' +
      'problem in P2. Unless they did not receive any cash this month, this income ' +
      'is generally recorded in the employment question about pension money received, ' +
      'item E15. Please change E15 in the Employment section.',
      'Addiction Severity Index', mtConfirmation, [mbOk],
      0);
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    pgctrlASI.ActivePage := TabEmploy;
    ActiveControl := edtE15;
    initialize := False;
    exit;
  end;

  // 44. P11a
  if (WhichOne = 'P11a') and
    ((Length(edtP11.text) > 0) and (edtP11.text[1] in ['1'..'9'])) and
    ((rgrpP3.ItemIndex = 0) and (rgrpP4.ItemIndex = 0) and
    (rgrpP5.ItemIndex = 0) and (rgrpP6.ItemIndex = 0) and
    (rgrpP7.ItemIndex = 0) and (rgrpP8.ItemIndex = 0) and
    (rgrpP9.ItemIndex = 0)) then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    lblVerify.caption := 'You report that the veteran had problems in the past 30 days, ' +
      'but none are recorded in the psychiatric symptoms list, P3 through P9. ' +
      'Please review P3 through P9.' +
      #13 + #10 + 'Click here to continue cross checks.';
    if MsgDlg('You report that the veteran had problems in the past 30 days, ' +
      'but none are recorded in the psychiatric symptoms list, P3 through P9. ' +
      'Please review P3 through P9, do you want to go back and change the code ' +
      'of any symptoms?',
      'Addiction Severity Index', mtConfirmation, [mbYes, mbNo],
      0) = mrNo then
    begin
      Initialize := False;
      if not CrossCheckItems then
        exit;
    end
    else
    begin
      pgctrlASI.ActivePage := TabPsych;
      //ActiveControl := rgrpP3;
      Initialize := False;
      exit;
    end;
  end;

  // 45. P11b
  if (WhichOne = 'P11b') and
    ((Length(edtP11.text) > 0) and (edtP11.text[1] in ['0', 'X', 'N'])) and
    ((rgrpP3.ItemIndex = 1) or (rgrpP4.ItemIndex = 1) or
    (rgrpP5.ItemIndex = 1) or (rgrpP6.ItemIndex = 1) or
    (rgrpP7.ItemIndex = 1) or (rgrpP8.ItemIndex = 1) or
    (rgrpP9.ItemIndex = 1)) then
  begin
    lblVerify.caption := 'You report that the veteran had no problems in the past 30 days, ' +
      'but problems are evident from the psychiatric symptoms list, P3 through P9. ' +
      #13 + #10 + 'Click here to continue cross checks.';
    MsgDlg('You report that the veteran had no problems in the past 30 days, ' +
      'but problems are evident from the psychiatric symptoms list, P3 through P9. ' +
      'Please probe about the number of days these symptoms bothered the veteran and  ' +
      'recode P11.',
      'Addiction Severity Index', mtConfirmation, [mbOk],
      0);
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    pgctrlASI.ActivePage := TabPsych;
    ActiveControl := edtP11;
    initialize := False;
    exit;
  end;

  // 47. P13a
  if (WhichOne = 'P13a') and
    ((Length(edtP11.text) > 0) and (edtP11.text[1] in ['1'..'9'])) and
    ((rgrpP12.ItemIndex = 0) or (rgrpP13.ItemIndex = 0)) then
  begin
    lblVerify.caption := 'You report that the veteran had psychiatric problems in the past ' +
      '30 days in question P11. Given these problems, we would expect that the veteran ' +
      'would be at least slightly bothered or slightly in need of treatment ' +
      'for this recent problem.' +
      #13 + #10 + 'Click here to continue cross checks.';
    MsgDlg('You report that the veteran had psychiatric problems in the past ' +
      '30 days in question P11. Given these problems, we would expect that the veteran ' +
      'would be at least slightly bothered or slightly in need of treatment ' +
      'for this recent problem. Please recode P12 and/or P13.',
      'Addiction Severity Index', mtConfirmation, [mbOk],
      0);
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    pgctrlASI.ActivePage := TabPsych;
    Initialize := False;
    //ActiveControl := rgrpP12;
    exit;
  end;

  // 48. P13b
  if (WhichOne = 'P13b') and
    ((Length(edtP11.text) > 0) and (edtP11.text[1] in ['0', 'N', 'X'])) and
    (((rgrpP12.ItemIndex > 0) and (rgrpP12.ItemIndex < 5)) or
    ((rgrpP13.ItemIndex > 0) and (rgrpP13.ItemIndex < 5))) then
  begin
    lblVerify.caption := 'You report that the veteran had no psychiatric problems in the past ' +
      '30 days in question P11. If they report being troubled or wanting treatment, ' +
      'it''s fair to expect that they had some problem days.' +
      #13 + #10 + 'Click here to continue cross checks.';
    MsgDlg('You report that the veteran had no psychiatric problems in the past ' +
      '30 days in question P11. If they report being troubled or wanting treatment, ' +
      'it''s fair to expect that they had some problem days. Please identiry the number of ' +
      'days the symptoms have bothered the veteran.',
      'Addiction Severity Index', mtConfirmation, [mbOk],
      0);
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    pgctrlASI.ActivePage := TabPsych;
    ActiveControl := edtP11;
    initialize := False;
    exit;
  end;

  initialize := False;
  if not CrossCheckItems then Exit;

  // Done with Cross checks
  if Length(Trim(CrossCheckList)) = 0 then
  begin
    ConductCrossChecks(sender); // need to clear its result
    exit;
  end;

  // Item was okay, go to next cross check item if needed
  if i > 0 then
  begin
    Delete(CrossCheckList, i, Length(WhichOne) + 1);
    if CrossCheckItems then
      ConductCrossChecks(sender);
  end;
end;

function TfrmASI.SaveSign: boolean;
var
  Sig,
    MySig: string;
label
  skip;

  function GetElectronicSignature: string;
  begin
    SignatureForASI(Font.Size,
      'Sign the Addiction Severity Index',
      'Electronic Signature',
      MySig);
    if MySig = '' then
    begin
      result := 'Cancel';
      exit;
    end;

    with RPCBrokerV do
    begin
      ClearResults := True;
      ClearParameters := True;
      RemoteProcedure := 'YTAPI ASI SIGNER';
      Param[0].Mult['"CODE"'] := MySig;
      Param[0].Mult['"YSIEN"'] := ASI_Vista.RecordIEN;
      Param[0].PType := List;
      call;
    end;

    if RPCBrokerV.Results[1] = '1^ASI SIGNED' then
      result := 'Done'
    else
      result := 'Error';
  end;

begin
  result := false;

  // Attempt to save ASI
  ASI_Save.SaveASIData; // uMHA_ASI_Save

  // Verificatiuon that ASI was saved
  if not ASI_Vista.ResponsesSaved then exit;

  // Ensure that the interviewer is the person signing this ASI.
  if UpperCase(Trim(User.Name)) <>
    UpperCase(Trim(cbxG11.text)) then
  begin
    MsgDlg('The only person allowed to sign this ASI is the interviewer.',
      'Electronic Signature', mtInformation, [mbOK], 0);
      //Left, Top, Height, Width, 0);
    exit;
  end;

  // User signs ASI
  repeat
    Sig := GetElectronicSignature;
    if sig = 'Cancel' then
      exit;
    if Sig = 'Done' then goto Skip;
    MsgDlg('This signature was not recognized. Capitalization may be required. ' +
      'Please try again.',
      'Electronic Signature', mtInformation, [mbOK], 0);
      //Left, Top, Height, Width, 0);
  until sig = 'Done';
  skip:

  if ((Sig = 'Done') and (rgrpG12.ItemIndex = 3)) then
  begin
    if MsgDlg('Would you like to save a narrative progress note for this ASI?',
      'Mental Health Assistant', mtConfirmation, [mbYes, mbNo],
          //Left, Top, Height, Width,
      0) = mrNo then // nothing
    else
      ASI_Vista.SaveProgressNote(ASI_Vista.RecordIEN); //uMHA_ASI_Vista
  end;
  result := true;
end;

function TfrmASI.SignAndExit: boolean;
begin
  result := false;
  if not SaveSign then
  begin
    CheckSkipItems := False;
    CrossCheckItems := False;
    lblHint.Caption := 'ASI not signed.';
    CrossCheckList := 'G20,M1,M2,M7,M8,E1y,E12a,E12b,E15,E20,D2d,D2y,D12d,' +
      'D14,D16,D17,D21a,D21b,D22a,D22b,D23,D24a,D24b,D30a,D30b,D31a,D31b,L17,L20,L24,' +
      'L25,L26,L27,F34a,F34b,F35a,F35b,P2,P9a,P9b,P10,P11a,P11b,P13a,P13b,';
    exit;
  end;
  lblHint.Caption := 'Data verified, cross checks done, ASI signed...';
  result := true;
end;

procedure TfrmASI.ShowHint(Sender: TObject);
begin
  lblHint.Caption := GetLongHint(Application.Hint);
end;

//---------------------------------------------------------------------
//       Radio Group: OnEnter
//---------------------------------------------------------------------

procedure TfrmASI.HighlightRadioGroup(Item: TRzRadioGroup);
begin
  with Item do
    begin
      // While it may not seem obvious, "OnEnter" sets the sender variables but
      // does not set ActiveControl.
      Item.SetFocus;
      Color := MHA_Registry.HighlightBackground;
      Font.Color := MHA_Registry.HighlightForeground;
      lblHint.Caption := Hint;
      // Add to list of items the user clicked on
      ResponseList := ResponseList + UpperCase(Name) + ' ';
    end;
end;

procedure TfrmASI.rgrpG8Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpG8);
end;

procedure TfrmASI.rgrpG9Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpG9);
end;

procedure TfrmASI.rgrpG12Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpG12);
end;

procedure TfrmASI.rgrpG15Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpG15);
end;

procedure TfrmASI.rgrpG17Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpG17);
end;

procedure TfrmASI.rgrpG18Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpG18);
end;

procedure TfrmASI.rgrpG19Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpG19);
end;

procedure TfrmASI.rgrpM3Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpM3);
end;

procedure TfrmASI.rgrpM4Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpM4);
end;

procedure TfrmASI.rgrpM5Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpM5);
end;

procedure TfrmASI.rgrpM7Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpM7);
end;

procedure TfrmASI.rgrpM8Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpM8);
end;

procedure TfrmASI.rgrpM9Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpM9);
end;

procedure TfrmASI.rgrpM10Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpM10);
end;

procedure TfrmASI.rgrpM11Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpM11);
end;

procedure TfrmASI.rgrpE3Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpE3);
end;

procedure TfrmASI.rgrpE4Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpE4);
end;

procedure TfrmASI.rgrpE5Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpE5);
end;

procedure TfrmASI.rgrpE7Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpE7);
end;

procedure TfrmASI.rgrpE8Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpE8);
end;

procedure TfrmASI.rgrpE9Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpE9);
end;

procedure TfrmASI.rgrpE10Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpE10);
end;

procedure TfrmASI.rgrpE20Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpE20);
end;

procedure TfrmASI.rgrpE21Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpE21);
end;

procedure TfrmASI.rgrpE22Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpE22);
end;

procedure TfrmASI.rgrpE23Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpE23);
end;

procedure TfrmASI.rgrpE24Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpE24);
end;

procedure TfrmASI.rgrpD14Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpD14);
end;

procedure TfrmASI.rgrpD28Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpD28);
end;

procedure TfrmASI.rgrpD29Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpD29);
end;

procedure TfrmASI.rgrpD30Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpD30);
end;

procedure TfrmASI.rgrpD31Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpD31);
end;

procedure TfrmASI.rgrpD32Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpD32);
end;

procedure TfrmASI.rgrpD33Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpD33);
end;

procedure TfrmASI.rgrpD34Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpD34);
end;

procedure TfrmASI.rgrpD35Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpD35);
end;

procedure TfrmASI.rgrpL1Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpL1);
end;

procedure TfrmASI.rgrpL2Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpL2);
end;

procedure TfrmASI.rgrpL23Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpL23);
end;

procedure TfrmASI.rgrpL24Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpL24);
end;

procedure TfrmASI.rgrpL25Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpL25);
end;

procedure TfrmASI.rgrpL28Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpL28);
end;

procedure TfrmASI.rgrpL29Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpL29);
end;

procedure TfrmASI.rgrpL30Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpL30);
end;

procedure TfrmASI.rgrpL31Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpL31);
end;

procedure TfrmASI.rgrpL32Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpL32);
end;

procedure TfrmASI.rgrpF1Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF1);
end;

procedure TfrmASI.rgrpF3Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF3);
end;

procedure TfrmASI.rgrpF4Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF4);
end;

procedure TfrmASI.rgrpF6Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF6);
end;

procedure TfrmASI.rgrpF7Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF7);
end;

procedure TfrmASI.rgrpF8Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF8);
end;

procedure TfrmASI.rgrpF9Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF9);
end;

procedure TfrmASI.rgrpF10Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF10);
end;

procedure TfrmASI.rgrpF19Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF19);
end;

procedure TfrmASI.rgrpF20Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF20);
end;

procedure TfrmASI.rgrpF21Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF21);
end;

procedure TfrmASI.rgrpF18Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF18);
end;

procedure TfrmASI.rgrpF18lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF18l);
end;

procedure TfrmASI.rgrpF19lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF19l);
end;

procedure TfrmASI.rgrpF20lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF20l);
end;

procedure TfrmASI.rgrpF21lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF21l);
end;

procedure TfrmASI.rgrpF22Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF22);
end;

procedure TfrmASI.rgrpF22lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF22l);
end;

procedure TfrmASI.rgrpF23Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF23);
end;

procedure TfrmASI.rgrpF23lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF23l);
end;

procedure TfrmASI.rgrpF24Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF24);
end;

procedure TfrmASI.rgrpF24lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF24l);
end;

procedure TfrmASI.rgrpF25Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF25);
end;

procedure TfrmASI.rgrpF25lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF25l);
end;

procedure TfrmASI.rgrpF26Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF26);
end;

procedure TfrmASI.rgrpF26lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF26l);
end;

procedure TfrmASI.rgrpF27Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF27);
end;

procedure TfrmASI.rgrpF27lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF27l);
end;

procedure TfrmASI.rgrpF28Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF28);
end;

procedure TfrmASI.rgrpF28lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF28l);
end;

procedure TfrmASI.rgrpF29Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF29);
end;

procedure TfrmASI.rgrpF29lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF29l);
end;

procedure TfrmASI.rgrpF32Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF32);
end;

procedure TfrmASI.rgrpF33Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF33);
end;

procedure TfrmASI.rgrpF34Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF34);
end;

procedure TfrmASI.rgrpF35Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF35);
end;

procedure TfrmASI.rgrpF36Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF36);
end;

procedure TfrmASI.rgrpF37Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF37);
end;

procedure TfrmASI.rgrpF38Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpF38);
end;

procedure TfrmASI.rgrpP2Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP2);
end;

procedure TfrmASI.rgrpP3Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP3);
end;

procedure TfrmASI.rgrpP3lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP3l);
end;

procedure TfrmASI.rgrpP4Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP4);
end;

procedure TfrmASI.rgrpP4lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP4l);
end;

procedure TfrmASI.rgrpP5Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP5);
end;

procedure TfrmASI.rgrpP5lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP5l);
end;

procedure TfrmASI.rgrpP6Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP6);
end;

procedure TfrmASI.rgrpP6lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP6l);
end;

procedure TfrmASI.rgrpP7Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP7);
end;

procedure TfrmASI.rgrpP7lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP7l);
end;

procedure TfrmASI.rgrpP8Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP8);
end;

procedure TfrmASI.rgrpP8lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP8l);
end;

procedure TfrmASI.rgrpP9Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP9);
end;

procedure TfrmASI.rgrpP9lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP9l);
end;

procedure TfrmASI.rgrpP10Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP10);
end;

procedure TfrmASI.rgrpP10lEnter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP10l);
end;

procedure TfrmASI.rgrpP12Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP12);
end;

procedure TfrmASI.rgrpP13Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP13);
end;

procedure TfrmASI.rgrpP14Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP14);
end;

procedure TfrmASI.rgrpP15Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP15);
end;

procedure TfrmASI.rgrpP16Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP16);
end;

procedure TfrmASI.rgrpP17Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP17);
end;

procedure TfrmASI.rgrpP18Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP18);
end;

procedure TfrmASI.rgrpP19Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP19);
end;

procedure TfrmASI.rgrpP20Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP20);
end;

procedure TfrmASI.rgrpP21Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP21);
end;

procedure TfrmASI.rgrpP22Enter(Sender: TObject);
begin
  aHighLightOn(Sender);
  HighlightRadioGroup(rgrpP22);
end;

procedure TfrmASI.CleanupComments;
// The M side stores all carriage returns causing excessive word wrapping.
begin
  // Mark the double carriage returns. This keeps the paragraphs intact.
  memDrug.Text := StringReplace(memDrug.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  memEmploy.Text := StringReplace(memEmploy.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  memFamily.Text := StringReplace(memFamily.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  memGeneral.Text := StringReplace(memGeneral.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  memLegal.Text := StringReplace(memLegal.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  memLeisure.Text := StringReplace(memLeisure.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  memMedical.Text := StringReplace(memMedical.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  memPsych.Text := StringReplace(memPsych.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  memSocial.Text := StringReplace(memSocial.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  memSpirit.Text := StringReplace(memSpirit.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  // Remove all carriage returns
  memDrug.Text := StringReplace(memDrug.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  memEmploy.Text := StringReplace(memEmploy.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  memFamily.Text := StringReplace(memFamily.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  memGeneral.Text := StringReplace(memGeneral.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  memLegal.Text := StringReplace(memLegal.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  memLeisure.Text := StringReplace(memLeisure.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  memMedical.Text := StringReplace(memMedical.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  memPsych.Text := StringReplace(memPsych.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  memSocial.Text := StringReplace(memSocial.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  memSpirit.Text := StringReplace(memSpirit.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  // Replace the paragraph markers
  memDrug.Text := StringReplace(memDrug.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
  memEmploy.Text := StringReplace(memEmploy.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
  memFamily.Text := StringReplace(memFamily.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
  memGeneral.Text := StringReplace(memGeneral.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
  memLegal.Text := StringReplace(memLegal.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
  memLeisure.Text := StringReplace(memLeisure.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
  memMedical.Text := StringReplace(memMedical.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
  memPsych.Text := StringReplace(memPsych.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
  memSocial.Text := StringReplace(memSocial.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
  memSpirit.Text := StringReplace(memSpirit.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
end;

end.

